@@ -1,31 +1,18 @@
-'''CTS: Cluster Testing System: Audit module
- '''
+""" Auditing classes for Pacemaker's Cluster Test Suite (CTS)
+"""
 
-__copyright__ = '''
-Copyright (C) 2000, 2001,2005 Alan Robertson <alanr@unix.sh>
-Licensed under the GNU GPL.
-'''
+# Pacemaker targets compatibility with Python 2.7 and 3.2+
+from __future__ import print_function, unicode_literals, absolute_import, division
 
-#
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License
-# as published by the Free Software Foundation; either version 2
-# of the License, or (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
+__copyright__ = "Copyright 2000-2018 the Pacemaker project contributors"
+__license__ = "GNU General Public License version 2 or later (GPLv2+) WITHOUT ANY WARRANTY"
 
 import time, re, uuid
-from watcher import LogWatcher
+from cts.watcher import LogWatcher
+from cts.remote import input_wrapper
+from cts.CTSvars import *
 
-
-class ClusterAudit:
+class ClusterAudit(object):
 
     def __init__(self, cm):
         self.CM = cm
@@ -160,7 +147,8 @@ class DiskAudit(ClusterAudit):
 
     def __call__(self):
         result = 1
-        dfcmd = "df -BM /var/log | tail -1 | awk '{print $(NF-1)\" \"$(NF-2)}' | tr -d 'M%'"
+        # @TODO Use directory of PCMK_logfile if set on host
+        dfcmd = "df -BM " + CTSvars.CRM_LOG_DIR + " | tail -1 | awk '{print $(NF-1)\" \"$(NF-2)}' | tr -d 'M%'"
 
         self.CM.ns.WaitForAllNodesToComeUp(self.CM.Env["nodes"])
         for node in self.CM.Env["nodes"]:
@@ -184,7 +172,7 @@ class DiskAudit(ClusterAudit):
                             answer = "Y"
                         else:
                             try:
-                                answer = raw_input('Continue? [nY]')
+                                answer = input_wrapper('Continue? [nY]')
                             except EOFError as e:
                                 answer = "n"
 
@@ -216,7 +204,7 @@ class FileAudit(ClusterAudit):
         self.CM.ns.WaitForAllNodesToComeUp(self.CM.Env["nodes"])
         for node in self.CM.Env["nodes"]:
 
-            (rc, lsout) = self.CM.rsh(node, "ls -al /var/lib/heartbeat/cores/* | grep core.[0-9]", None)
+            (rc, lsout) = self.CM.rsh(node, "ls -al /var/lib/pacemaker/cores/* | grep core.[0-9]", None)
             for line in lsout:
                 line = line.strip()
                 if line not in self.known:
@@ -245,7 +233,7 @@ class FileAudit(ClusterAudit):
                     for line in lsout:
                         self.CM.debug("ps[%s]: %s" % (node, line))
 
-                    self.CM.rsh(node, "rm -rf /dev/shm/qb-*")
+                    self.CM.rsh(node, "rm -f /dev/shm/qb-*")
 
             else:
                 self.CM.debug("Skipping %s" % node)
@@ -256,7 +244,7 @@ class FileAudit(ClusterAudit):
         return 1
 
 
-class AuditResource:
+class AuditResource(object):
     def __init__(self, cm, line):
         fields = line.split()
         self.CM = cm
@@ -292,7 +280,7 @@ class AuditResource:
         return 0
             
 
-class AuditConstraint:
+class AuditConstraint(object):
     def __init__(self, cm, line):
         fields = line.split()
         self.CM = cm
@@ -418,10 +406,12 @@ class PrimitiveAudit(ClusterAudit):
         return rc
 
     def is_applicable(self):
-        if self.CM["Name"] == "crm-lha":
-            return 1
-        if self.CM["Name"] == "crm-ais":
-            return 1
+        # @TODO Due to long-ago refactoring, this name test would never match,
+        # so this audit (and those derived from it) would never run.
+        # Uncommenting the next lines fixes the name test, but that then
+        # exposes pre-existing bugs that need to be fixed.
+        #if self.CM["Name"] == "crm-corosync":
+        #    return 1
         return 0
 
 
@@ -528,7 +518,7 @@ class ColocationAudit(PrimitiveAudit):
         return rc
 
 
-class CrmdStateAudit(ClusterAudit):
+class ControllerStateAudit(ClusterAudit):
     def __init__(self, cm):
         self.CM = cm
         self.Stats = {"calls":0
@@ -587,13 +577,15 @@ class CrmdStateAudit(ClusterAudit):
         return passed
 
     def name(self):
-        return "CrmdStateAudit"
+        return "ControllerStateAudit"
     
     def is_applicable(self):
-        if self.CM["Name"] == "crm-lha":
-            return 1
-        if self.CM["Name"] == "crm-ais":
-            return 1
+        # @TODO Due to long-ago refactoring, this name test would never match,
+        # so this audit (and those derived from it) would never run.
+        # Uncommenting the next lines fixes the name test, but that then
+        # exposes pre-existing bugs that need to be fixed.
+        #if self.CM["Name"] == "crm-corosync":
+        #    return 1
         return 0
 
 
@@ -702,10 +694,12 @@ class CIBAudit(ClusterAudit):
         return "CibAudit"
     
     def is_applicable(self):
-        if self.CM["Name"] == "crm-lha":
-            return 1
-        if self.CM["Name"] == "crm-ais":
-            return 1
+        # @TODO Due to long-ago refactoring, this name test would never match,
+        # so this audit (and those derived from it) would never run.
+        # Uncommenting the next lines fixes the name test, but that then
+        # exposes pre-existing bugs that need to be fixed.
+        #if self.CM["Name"] == "crm-corosync":
+        #    return 1
         return 0
 
 
@@ -722,7 +716,7 @@ class PartitionAudit(ClusterAudit):
         self.NodeQuorum = {}
 
     def has_key(self, key):
-        return self.Stats.has_key(key)
+        return key in self.Stats
 
     def __setitem__(self, key, value):
         self.Stats[key] = value
@@ -842,16 +836,18 @@ class PartitionAudit(ClusterAudit):
         return "PartitionAudit"
     
     def is_applicable(self):
-        if self.CM["Name"] == "crm-lha":
-            return 1
-        if self.CM["Name"] == "crm-ais":
-            return 1
+        # @TODO Due to long-ago refactoring, this name test would never match,
+        # so this audit (and those derived from it) would never run.
+        # Uncommenting the next lines fixes the name test, but that then
+        # exposes pre-existing bugs that need to be fixed.
+        #if self.CM["Name"] == "crm-corosync":
+        #    return 1
         return 0
 
 AllAuditClasses.append(DiskAudit)
 AllAuditClasses.append(FileAudit)
 AllAuditClasses.append(LogAudit)
-AllAuditClasses.append(CrmdStateAudit)
+AllAuditClasses.append(ControllerStateAudit)
 AllAuditClasses.append(PartitionAudit)
 AllAuditClasses.append(PrimitiveAudit)
 AllAuditClasses.append(GroupAudit)
