@@ -1,9 +1,19 @@
-from CTS import *
-from CTStests import CTSTest
-from CTSaudits import ClusterAudit
-from cts.watcher  import LogWatcher
+""" Test scenario classes for Pacemaker's Cluster Test Suite (CTS)
+"""
 
-class ScenarioComponent:
+# Pacemaker targets compatibility with Python 2.7 and 3.2+
+from __future__ import print_function, unicode_literals, absolute_import, division
+
+__copyright__ = "Copyright 2000-2018 the Pacemaker project contributors"
+__license__ = "GNU General Public License version 2 or later (GPLv2+) WITHOUT ANY WARRANTY"
+
+from cts.CTS import *
+from cts.CTStests import CTSTest
+from cts.CTSaudits import ClusterAudit
+from cts.watcher  import LogWatcher
+from cts.remote import input_wrapper
+
+class ScenarioComponent(object):
 
     def __init__(self, Env):
         self.Env = Env
@@ -24,7 +34,7 @@ class ScenarioComponent:
         raise ValueError("Abstract Class member (Setup)")
 
 
-class Scenario:
+class Scenario(object):
     (
 '''The basic idea of a scenario is that of an ordered list of
 ScenarioComponent objects.  Each ScenarioComponent is SetUp() in turn,
@@ -85,8 +95,7 @@ A partially set up scenario is torn down if it fails during setup.
         self.ClusterManager.ns.WaitForAllNodesToComeUp(self.ClusterManager.Env["nodes"])
 
         self.audit()
-        if self.ClusterManager.Env["valgrind-tests"]:
-            self.ClusterManager.install_helper("cts.supp")
+        self.ClusterManager.install_support()
 
         self.BadNews = LogWatcher(self.ClusterManager.Env["LogFileName"],
                                   self.ClusterManager.templates.get_patterns(
@@ -121,6 +130,7 @@ A partially set up scenario is torn down if it fails during setup.
 
         self.audit()
         self.ClusterManager.StatsExtract()
+        self.ClusterManager.install_support("uninstall")
 
     def incr(self, name):
         '''Increment (or initialize) the value associated with the given name'''
@@ -170,7 +180,7 @@ A partially set up scenario is torn down if it fails during setup.
                 answer = "Y"
             else:
                 try:
-                    answer = raw_input('Continue? [nY]')
+                    answer = input_wrapper('Continue? [nY]')
                 except EOFError as e:
                     answer = "n"
             if answer and answer == "n":
@@ -265,7 +275,7 @@ A partially set up scenario is torn down if it fails during setup.
                 answer = "Y"
             else:
                 try:
-                    answer = raw_input('Big problems. Continue? [nY]')
+                    answer = input_wrapper('Big problems. Continue? [nY]')
                 except EOFError as e:
                     answer = "n"
             if answer and answer == "n":
@@ -399,7 +409,7 @@ According to the manual page for ping:
         '''Start the PingFest!'''
 
         self.PingSize = 1024
-        if "PingSize" in CM.Env.keys():
+        if "PingSize" in list(CM.Env.keys()):
                 self.PingSize = CM.Env["PingSize"]
 
         CM.log("Starting %d byte flood pings" % self.PingSize)
