@@ -1,5 +1,7 @@
 /*
- * Copyright 2012-2018 David Vossel <davidvossel@gmail.com>
+ * Copyright 2012-2019 the Pacemaker project contributors
+ *
+ * The version control history for this file may have further details.
  *
  * This source code is licensed under the GNU Lesser General Public License
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
@@ -8,6 +10,19 @@
 #include <crm_internal.h>
 
 #include <glib.h>
+
+#undef PCMK__TIME_USE_CGT
+#if HAVE_DECL_CLOCK_MONOTONIC && defined(CLOCK_MONOTONIC) \
+    && !defined(PCMK_TIME_EMERGENCY_CGT)
+#define PCMK__TIME_USE_CGT
+#endif
+
+#ifdef PCMK__TIME_USE_CGT
+#  include <time.h>  /* clock_gettime */
+#elif defined(HAVE_SYS_TIMEB_H)
+#  include <sys/timeb.h>
+#endif
+
 #include <unistd.h>
 
 #include <crm/crm.h>
@@ -19,10 +34,6 @@
 
 #include "pacemaker-execd.h"
 
-#ifdef HAVE_SYS_TIMEB_H
-#  include <sys/timeb.h>
-#endif
-
 #define EXIT_REASON_MAX_LEN 128
 
 GHashTable *rsc_list = NULL;
@@ -55,11 +66,17 @@ typedef struct lrmd_cmd_s {
     char *output;
     char *userdata_str;
 
-#ifdef HAVE_SYS_TIMEB_H
     /* Recurring and systemd operations may involve more than one executor
      * command per operation, so they need info about the original and the most
      * recent.
      */
+#ifdef PCMK__TIME_USE_CGT
+    struct timespec t_first_run;   /* Timestamp of when op first ran */
+    struct timespec t_run;         /* Timestamp of when op most recently ran */
+    struct timespec t_first_queue; /* Timestamp of when op first was queued */
+    struct timespec t_queue;       /* Timestamp of when op most recently was queued */
+    struct timespec t_rcchange;    /* Timestamp of last rc change */
+#elif defined(HAVE_SYS_TIMEB_H)
     struct timeb t_first_run;   /* Timestamp of when op first ran */
     struct timeb t_run;         /* Timestamp of when op most recently ran */
     struct timeb t_first_queue; /* Timestamp of when op first was queued */
@@ -92,7 +109,7 @@ log_finished(lrmd_cmd_t * cmd, int exec_time, int queue_time)
     if (safe_str_eq(cmd->action, "monitor")) {
         log_level = LOG_DEBUG;
     }
-#ifdef HAVE_SYS_TIMEB_H
+#if defined(PCMK__TIME_USE_CGT) || defined(HAVE_SYS_TIMEB_H)
     do_crm_log(log_level,
                "finished - rsc:%s action:%s call_id:%d %s%s exit-code:%d exec-time:%dms queue-time:%dms",
                cmd->rsc_id, cmd->action, cmd->call_id, cmd->last_pid ? "pid:" : "", pid_str,
@@ -142,6 +159,7 @@ build_rsc_from_xml(xmlNode * msg)
     rsc->provider = crm_element_value_copy(rsc_xml, F_LRMD_PROVIDER);
     rsc->type = crm_element_value_copy(rsc_xml, F_LRMD_TYPE);
     rsc->work = mainloop_add_trigger(G_PRIORITY_HIGH, lrmd_rsc_dispatch, rsc);
+    rsc->st_probe_rc = -ENODEV; // if stonith, initialize to "not running"
     return rsc;
 }
 
@@ -179,12 +197,21 @@ create_lrmd_cmd(xmlNode * msg, crm_client_t * client)
     return cmd;
 }
 
+static void
+stop_recurring_timer(lrmd_cmd_t *cmd)
+{
+    if (cmd) {
+        if (cmd->stonith_recurring_id) {
+            g_source_remove(cmd->stonith_recurring_id);
+        }
+        cmd->stonith_recurring_id = 0;
+    }
+}
+
 static void
 free_lrmd_cmd(lrmd_cmd_t * cmd)
 {
-    if (cmd->stonith_recurring_id) {
-        g_source_remove(cmd->stonith_recurring_id);
-    }
+    stop_recurring_timer(cmd);
     if (cmd->delay_id) {
         g_source_remove(cmd->delay_id);
     }
@@ -221,7 +248,12 @@ stonith_recurring_op_helper(gpointer data)
      * to be executed */
     rsc->recurring_ops = g_list_remove(rsc->recurring_ops, cmd);
     rsc->pending_ops = g_list_append(rsc->pending_ops, cmd);
-#ifdef HAVE_SYS_TIMEB_H
+#ifdef PCMK__TIME_USE_CGT
+    clock_gettime(CLOCK_MONOTONIC, &cmd->t_queue);
+    if (cmd->t_first_queue.tv_sec == 0) {
+        cmd->t_first_queue = cmd->t_queue;
+    }
+#elif defined(HAVE_SYS_TIMEB_H)
     ftime(&cmd->t_queue);
     if (cmd->t_first_queue.time == 0) {
         cmd->t_first_queue = cmd->t_queue;
@@ -232,6 +264,16 @@ stonith_recurring_op_helper(gpointer data)
     return FALSE;
 }
 
+static inline void
+start_recurring_timer(lrmd_cmd_t *cmd)
+{
+    if (cmd && (cmd->interval_ms > 0)) {
+        cmd->stonith_recurring_id = g_timeout_add(cmd->interval_ms,
+                                                  stonith_recurring_op_helper,
+                                                  cmd);
+    }
+}
+
 static gboolean
 start_delay_helper(gpointer data)
 {
@@ -300,8 +342,7 @@ merge_dup:
     if (safe_str_eq(rsc->class, PCMK_RESOURCE_CLASS_STONITH)) {
         /* if we are waiting for the next interval, kick it off now */
         if (dup_pending == TRUE) {
-            g_source_remove(cmd->stonith_recurring_id);
-            cmd->stonith_recurring_id = 0;
+            stop_recurring_timer(cmd);
             stonith_recurring_op_helper(cmd);
         }
 
@@ -339,7 +380,12 @@ schedule_lrmd_cmd(lrmd_rsc_t * rsc, lrmd_cmd_t * cmd)
     }
 
     rsc->pending_ops = g_list_append(rsc->pending_ops, cmd);
-#ifdef HAVE_SYS_TIMEB_H
+#ifdef PCMK__TIME_USE_CGT
+    clock_gettime(CLOCK_MONOTONIC, &cmd->t_queue);
+    if (cmd->t_first_queue.tv_sec == 0) {
+        cmd->t_first_queue = cmd->t_queue;
+    }
+#elif defined(HAVE_SYS_TIMEB_H)
     ftime(&cmd->t_queue);
     if (cmd->t_first_queue.time == 0) {
         cmd->t_first_queue = cmd->t_queue;
@@ -369,24 +415,41 @@ send_client_notify(gpointer key, gpointer value, gpointer user_data)
     xmlNode *update_msg = user_data;
     crm_client_t *client = value;
     int rc;
+    int log_level = LOG_WARNING;
+    const char *msg = NULL;
 
-    if (client == NULL) {
-        crm_err("Asked to send event to  NULL client");
-        return;
-    } else if (client->name == NULL) {
-        crm_trace("Asked to send event to client with no name");
+    CRM_CHECK(client != NULL, return);
+    if (client->name == NULL) {
+        crm_trace("Skipping notification to client without name");
         return;
     }
 
     rc = lrmd_server_send_notify(client, update_msg);
-    if ((rc <= 0) && (rc != -ENOTCONN)) {
-        crm_warn("Could not notify client %s/%s: %s " CRM_XS " rc=%d",
-                 client->name, client->id,
-                 (rc? pcmk_strerror(rc) : "no data sent"), rc);
+    if (rc > 0) {
+        return; // success
     }
+
+    switch (rc) {
+        case 0:
+            msg = "no data sent";
+            break;
+
+        case -ENOTCONN:
+        case -EPIPE: // Client exited without waiting for notification
+            log_level = LOG_INFO;
+            msg = "Disconnected";
+            break;
+
+        default:
+            msg = pcmk_strerror(rc);
+            break;
+    }
+    do_crm_log(log_level,
+               "Could not notify client %s/%s: %s " CRM_XS " rc=%d",
+               client->name, client->id, msg, rc);
 }
 
-#ifdef HAVE_SYS_TIMEB_H
+#if defined(PCMK__TIME_USE_CGT) || defined(HAVE_SYS_TIMEB_H)
 /*!
  * \internal
  * \brief Return difference between two times in milliseconds
@@ -394,21 +457,44 @@ send_client_notify(gpointer key, gpointer value, gpointer user_data)
  * \param[in] now  More recent time (or NULL to use current time)
  * \param[in] old  Earlier time
  *
- * \return milliseconds difference (or 0 if old is NULL or has time zero)
+ * \return milliseconds difference (or 0 if old is NULL or has tv_sec/time zero)
+ *
+ * \note Can overflow on 32bit machines when the differences is around
+ *       24 days or more.
  */
 static int
+#ifdef PCMK__TIME_USE_CGT
+time_diff_ms(struct timespec *now, struct timespec *old)
+{
+    struct timespec local_now = { 0, };
+#else
 time_diff_ms(struct timeb *now, struct timeb *old)
 {
     struct timeb local_now = { 0, };
+#endif
 
     if (now == NULL) {
+#ifdef PCMK__TIME_USE_CGT
+        clock_gettime(CLOCK_MONOTONIC, &local_now);
+#else
         ftime(&local_now);
+#endif
         now = &local_now;
     }
-    if ((old == NULL) || (old->time == 0)) {
+    if ((old == NULL)
+#ifdef PCMK__TIME_USE_CGT
+            || (old->tv_sec == 0)) {
+#else
+            || (old->time == 0)) {
+#endif
         return 0;
     }
-    return difftime(now->time, old->time) * 1000 + now->millitm - old->millitm;
+#ifdef PCMK__TIME_USE_CGT
+    return (now->tv_sec - old->tv_sec) * 1000
+            + (now->tv_nsec - old->tv_nsec) / 1000;
+#else
+    return (now->time - old->time) * 1000 + now->millitm - old->millitm;
+#endif
 }
 
 /*!
@@ -446,7 +532,7 @@ send_cmd_complete_notify(lrmd_cmd_t * cmd)
     int queue_time = 0;
     xmlNode *notify = NULL;
 
-#ifdef HAVE_SYS_TIMEB_H
+#if defined(PCMK__TIME_USE_CGT) || defined(HAVE_SYS_TIMEB_H)
     exec_time = time_diff_ms(NULL, &cmd->t_run);
     queue_time = time_diff_ms(&cmd->t_run, &cmd->t_queue);
 #endif
@@ -481,9 +567,14 @@ send_cmd_complete_notify(lrmd_cmd_t * cmd)
     crm_xml_add_int(notify, F_LRMD_CALLID, cmd->call_id);
     crm_xml_add_int(notify, F_LRMD_RSC_DELETED, cmd->rsc_deleted);
 
-#ifdef HAVE_SYS_TIMEB_H
-    crm_xml_add_int(notify, F_LRMD_RSC_RUN_TIME, cmd->t_run.time);
-    crm_xml_add_int(notify, F_LRMD_RSC_RCCHANGE_TIME, cmd->t_rcchange.time);
+#if defined(PCMK__TIME_USE_CGT) || defined(HAVE_SYS_TIMEB_H)
+#ifdef PCMK__TIME_USE_CGT
+    crm_xml_add_ll(notify, F_LRMD_RSC_RUN_TIME, (long long) cmd->t_run.tv_sec);
+    crm_xml_add_ll(notify, F_LRMD_RSC_RCCHANGE_TIME, (long long) cmd->t_rcchange.tv_sec);
+#else
+    crm_xml_add_ll(notify, F_LRMD_RSC_RUN_TIME, (long long) cmd->t_run.time);
+    crm_xml_add_ll(notify, F_LRMD_RSC_RCCHANGE_TIME, (long long) cmd->t_rcchange.time);
+#endif
     crm_xml_add_int(notify, F_LRMD_RSC_EXEC_TIME, exec_time);
     crm_xml_add_int(notify, F_LRMD_RSC_QUEUE_TIME, queue_time);
 #endif
@@ -554,7 +645,7 @@ cmd_reset(lrmd_cmd_t * cmd)
 {
     cmd->lrmd_op_status = 0;
     cmd->last_pid = 0;
-#ifdef HAVE_SYS_TIMEB_H
+#if defined(PCMK__TIME_USE_CGT) || defined(HAVE_SYS_TIMEB_H)
     memset(&cmd->t_run, 0, sizeof(cmd->t_run));
     memset(&cmd->t_queue, 0, sizeof(cmd->t_queue));
 #endif
@@ -615,16 +706,36 @@ ocf2uniform_rc(int rc)
 static int
 stonith2uniform_rc(const char *action, int rc)
 {
-    if (rc == -ENODEV) {
-        if (safe_str_eq(action, "stop")) {
+    switch (rc) {
+        case pcmk_ok:
             rc = PCMK_OCF_OK;
-        } else if (safe_str_eq(action, "start")) {
-            rc = PCMK_OCF_NOT_INSTALLED;
-        } else {
-            rc = PCMK_OCF_NOT_RUNNING;
-        }
-    } else if (rc != 0) {
-        rc = PCMK_OCF_UNKNOWN_ERROR;
+            break;
+
+        case -ENODEV:
+            /* This should be possible only for probes in practice, but
+             * interpret for all actions to be safe.
+             */
+            if (safe_str_eq(action, "monitor")) {
+                rc = PCMK_OCF_NOT_RUNNING;
+            } else if (safe_str_eq(action, "stop")) {
+                rc = PCMK_OCF_OK;
+            } else {
+                rc = PCMK_OCF_NOT_INSTALLED;
+            }
+            break;
+
+        case -EOPNOTSUPP:
+            rc = PCMK_OCF_UNIMPLEMENT_FEATURE;
+            break;
+
+        case -ETIME:
+        case -ETIMEDOUT:
+            rc = PCMK_OCF_TIMEOUT;
+            break;
+
+        default:
+            rc = PCMK_OCF_UNKNOWN_ERROR;
+            break;
     }
     return rc;
 }
@@ -766,14 +877,21 @@ action_complete(svc_action_t * action)
     lrmd_cmd_t *cmd = action->cb_data;
     const char *rclass = NULL;
 
+#if defined(PCMK__TIME_USE_CGT) || defined(HAVE_SYS_TIMEB_H)
     bool goagain = false;
+#endif
 
     if (!cmd) {
         crm_err("Completed executor action (%s) does not match any known operations",
                 action->id);
         return;
     }
-#ifdef HAVE_SYS_TIMEB_H
+
+#ifdef PCMK__TIME_USE_CGT
+    if (cmd->exec_rc != action->rc) {
+        clock_gettime(CLOCK_MONOTONIC, &cmd->t_rcchange);
+    }
+#elif defined(HAVE_SYS_TIMEB_H)
     if (cmd->exec_rc != action->rc) {
         ftime(&cmd->t_rcchange);
     }
@@ -801,24 +919,32 @@ action_complete(svc_action_t * action)
              * report 'complete' to the rest of pacemaker until, you know,
              * it's actually done.
              */
+#if defined(PCMK__TIME_USE_CGT) || defined(HAVE_SYS_TIMEB_H)
             goagain = true;
+#endif
             cmd->real_action = cmd->action;
             cmd->action = strdup("monitor");
 
         } else if(cmd->exec_rc == PCMK_OCF_OK && safe_str_eq(cmd->action, "stop")) {
+#if defined(PCMK__TIME_USE_CGT) || defined(HAVE_SYS_TIMEB_H)
             goagain = true;
+#endif
             cmd->real_action = cmd->action;
             cmd->action = strdup("monitor");
 
         } else if(cmd->real_action) {
             /* Ok, so this is the follow up monitor action to check if start actually completed */
             if(cmd->lrmd_op_status == PCMK_LRM_OP_DONE && cmd->exec_rc == PCMK_OCF_PENDING) {
+#if defined(PCMK__TIME_USE_CGT) || defined(HAVE_SYS_TIMEB_H)
                 goagain = true;
+#endif
             } else if(cmd->exec_rc == PCMK_OCF_OK && safe_str_eq(cmd->real_action, "stop")) {
+#if defined(PCMK__TIME_USE_CGT) || defined(HAVE_SYS_TIMEB_H)
                 goagain = true;
+#endif
 
             } else {
-#ifdef HAVE_SYS_TIMEB_H
+#if defined(PCMK__TIME_USE_CGT) || defined(HAVE_SYS_TIMEB_H)
                 int time_sum = time_diff_ms(NULL, &cmd->t_first_run);
                 int timeout_left = cmd->timeout_orig - time_sum;
 
@@ -827,8 +953,15 @@ action_complete(svc_action_t * action)
                 cmd_original_times(cmd);
 #endif
 
-                if(cmd->lrmd_op_status == PCMK_LRM_OP_DONE && cmd->exec_rc == PCMK_OCF_NOT_RUNNING && safe_str_eq(cmd->real_action, "stop")) {
-                    cmd->exec_rc = PCMK_OCF_OK;
+                // Monitors may return "not running", but start/stop shouldn't
+                if ((cmd->lrmd_op_status == PCMK_LRM_OP_DONE)
+                    && (cmd->exec_rc == PCMK_OCF_NOT_RUNNING)) {
+
+                    if (safe_str_eq(cmd->real_action, "start")) {
+                        cmd->exec_rc = PCMK_OCF_UNKNOWN_ERROR;
+                    } else if (safe_str_eq(cmd->real_action, "stop")) {
+                        cmd->exec_rc = PCMK_OCF_OK;
+                    }
                 }
             }
         }
@@ -842,18 +975,24 @@ action_complete(svc_action_t * action)
             cmd->exec_rc = PCMK_OCF_NOT_RUNNING;
 
         } else if (safe_str_eq(cmd->action, "start") && cmd->exec_rc != PCMK_OCF_OK) {
+#if defined(PCMK__TIME_USE_CGT) || defined(HAVE_SYS_TIMEB_H)
             goagain = true;
+#endif
         }
     }
 #endif
 
-    /* Wrapping this section in ifdef implies that systemd resources are not
-     * fully supported on platforms without sys/timeb.h. Since timeb is
-     * obsolete, we should eventually prefer a clock_gettime() implementation
-     * (wrapped in its own ifdef) with timeb as a fallback.
-     */
-    if(goagain) {
-#ifdef HAVE_SYS_TIMEB_H
+#if defined(PCMK__TIME_USE_CGT) || defined(HAVE_SYS_TIMEB_H)
+    /* Wrapping this section in ifdef implies that systemd resources... */
+#ifdef PCMK__TIME_USE_CGT
+    /* .... are not fully supported on platforms without CLOCK_MONOTONIC,
+       which is most likely contradiction, but for completeness... */
+#elif defined(HAVE_SYS_TIMEB_H)
+    /* ... are not fully supported on platforms without sys/timeb.h.
+       Since timeb is obsolete, we should eventually prefer a clock_gettime()
+       implementation (wrapped in its own ifdef) with timeb as a fallback. */
+#endif
+    if (goagain) {
         int time_sum = time_diff_ms(NULL, &cmd->t_first_run);
         int timeout_left = cmd->timeout_orig - time_sum;
         int delay = cmd->timeout_orig / 10;
@@ -896,8 +1035,8 @@ action_complete(svc_action_t * action)
             cmd->exec_rc = PCMK_OCF_TIMEOUT;
             cmd_original_times(cmd);
         }
-#endif
     }
+#endif
 
     if (action->stderr_data) {
         cmd->output = strdup(action->stderr_data);
@@ -910,64 +1049,94 @@ action_complete(svc_action_t * action)
     cmd_finalize(cmd, rsc);
 }
 
+/*!
+ * \internal
+ * \brief Determine operation status of a stonith operation
+ *
+ * Non-stonith resource operations get their operation status directly from the
+ * service library, but the fencer does not have an equivalent, so we must infer
+ * an operation status from the fencer API's return code.
+ *
+ * \param[in] action       Name of action performed on stonith resource
+ * \param[in] interval_ms  Action interval
+ * \param[in] rc           Action result from fencer
+ *
+ * \return Operation status corresponding to fencer API return code
+ */
+static int
+stonith_rc2status(const char *action, guint interval_ms, int rc)
+{
+    int status = PCMK_LRM_OP_DONE;
+
+    switch (rc) {
+        case pcmk_ok:
+            break;
+
+        case -EOPNOTSUPP:
+        case -EPROTONOSUPPORT:
+            status = PCMK_LRM_OP_NOTSUPPORTED;
+            break;
+
+        case -ETIME:
+        case -ETIMEDOUT:
+            status = PCMK_LRM_OP_TIMEOUT;
+            break;
+
+        case -ENOTCONN:
+        case -ECOMM:
+            // Couldn't talk to fencer
+            status = PCMK_LRM_OP_ERROR;
+            break;
+
+        case -ENODEV:
+            // The device is not registered with the fencer
+            status = PCMK_LRM_OP_ERROR;
+            break;
+
+        default:
+            break;
+    }
+    return status;
+}
+
 static void
 stonith_action_complete(lrmd_cmd_t * cmd, int rc)
 {
-    bool recurring = (cmd->interval_ms > 0);
-    lrmd_rsc_t *rsc = NULL;
+    // This can be NULL if resource was removed before command completed
+    lrmd_rsc_t *rsc = g_hash_table_lookup(rsc_list, cmd->rsc_id);
 
-    cmd->exec_rc = get_uniform_rc(PCMK_RESOURCE_CLASS_STONITH, cmd->action, rc);
+    cmd->exec_rc = stonith2uniform_rc(cmd->action, rc);
 
-    rsc = g_hash_table_lookup(rsc_list, cmd->rsc_id);
+    /* This function may be called with status already set to cancelled, if a
+     * pending action was aborted. Otherwise, we need to determine status from
+     * the fencer return code.
+     */
+    if (cmd->lrmd_op_status != PCMK_LRM_OP_CANCELLED) {
+        cmd->lrmd_op_status = stonith_rc2status(cmd->action, cmd->interval_ms,
+                                                rc);
 
-    if (cmd->lrmd_op_status == PCMK_LRM_OP_CANCELLED) {
-        recurring = FALSE;
-        /* do nothing */
-
-    } else if (rc == -ENODEV && safe_str_eq(cmd->action, "monitor")) {
-        // The device is not registered with the fencer
-
-        if (recurring) {
-            /* If we get here, the fencer somehow lost the registration of a
-             * previously active device (possibly due to crash and respawn). In
-             * that case, we need to indicate that the recurring monitor needs
-             * to be cancelled.
-             */
-            cmd->lrmd_op_status = PCMK_LRM_OP_CANCELLED;
-            recurring = FALSE;
-        } else {
-            cmd->lrmd_op_status = PCMK_LRM_OP_DONE;
-        }
-        cmd->exec_rc = PCMK_OCF_NOT_RUNNING;
-
-    } else if (rc) {
-        /* Attempt to map return codes to op status if possible */
-        switch (rc) {
-            case -EPROTONOSUPPORT:
-                cmd->lrmd_op_status = PCMK_LRM_OP_NOTSUPPORTED;
-                break;
-            case -ETIME:
-                cmd->lrmd_op_status = PCMK_LRM_OP_TIMEOUT;
-                break;
-            default:
-                /* TODO: This looks wrong.  Status should be _DONE and exec_rc set to an error */
-                cmd->lrmd_op_status = PCMK_LRM_OP_ERROR;
-        }
-    } else {
-        /* command successful */
-        cmd->lrmd_op_status = PCMK_LRM_OP_DONE;
-        if (safe_str_eq(cmd->action, "start") && rsc) {
-            rsc->stonith_started = 1;
+        // Certain successful actions change the known state of the resource
+        if (rsc && (cmd->exec_rc == PCMK_OCF_OK)) {
+            if (safe_str_eq(cmd->action, "start")) {
+                rsc->st_probe_rc = pcmk_ok; // maps to PCMK_OCF_OK
+            } else if (safe_str_eq(cmd->action, "stop")) {
+                rsc->st_probe_rc = -ENODEV; // maps to PCMK_OCF_NOT_RUNNING
+            }
         }
     }
 
-    if (recurring && rsc) {
-        if (cmd->stonith_recurring_id) {
-            g_source_remove(cmd->stonith_recurring_id);
-        }
-        cmd->stonith_recurring_id = g_timeout_add(cmd->interval_ms,
-                                                  stonith_recurring_op_helper,
-                                                  cmd);
+    /* The recurring timer should not be running at this point in any case, but
+     * as a failsafe, stop it if it is.
+     */
+    stop_recurring_timer(cmd);
+
+    /* Reschedule this command if appropriate. If a recurring command is *not*
+     * rescheduled, its status must be PCMK_LRM_OP_CANCELLED, otherwise it will
+     * not be removed from recurring_ops by cmd_finalize().
+     */
+    if (rsc && (cmd->interval_ms > 0)
+        && (cmd->lrmd_op_status != PCMK_LRM_OP_CANCELLED)) {
+        start_recurring_timer(cmd);
     }
 
     cmd_finalize(cmd, rsc);
@@ -991,6 +1160,18 @@ stonith_connection_failed(void)
     g_hash_table_iter_init(&iter, rsc_list);
     while (g_hash_table_iter_next(&iter, (gpointer *) & key, (gpointer *) & rsc)) {
         if (safe_str_eq(rsc->class, PCMK_RESOURCE_CLASS_STONITH)) {
+            /* If we registered this fence device, we don't know whether the
+             * fencer still has the registration or not. Cause future probes to
+             * return PCMK_OCF_UNKNOWN_ERROR until the resource is stopped or
+             * started successfully. This is especially important if the
+             * controller also went away (possibly due to a cluster layer
+             * restart) and won't receive our client notification of any
+             * monitors finalized below.
+             */
+            if (rsc->st_probe_rc == pcmk_ok) {
+                rsc->st_probe_rc = pcmk_err_generic;
+            }
+
             if (rsc->active) {
                 cmd_list = g_list_append(cmd_list, rsc->active);
             }
@@ -1016,85 +1197,138 @@ stonith_connection_failed(void)
     g_list_free(cmd_list);
 }
 
+/*!
+ * \internal
+ * \brief Execute a stonith resource "start" action
+ *
+ * Start a stonith resource by registering it with the fencer.
+ * (Stonith agents don't have a start command.)
+ *
+ * \param[in] stonith_api  Connection to fencer
+ * \param[in] rsc          Stonith resource to start
+ * \param[in] cmd          Start command to execute
+ *
+ * \return pcmk_ok on success, -errno otherwise
+ */
 static int
+execd_stonith_start(stonith_t *stonith_api, lrmd_rsc_t *rsc, lrmd_cmd_t *cmd)
+{
+    char *key = NULL;
+    char *value = NULL;
+    stonith_key_value_t *device_params = NULL;
+    int rc = pcmk_ok;
+
+    // Convert command parameters to stonith API key/values
+    if (cmd->params) {
+        GHashTableIter iter;
+
+        g_hash_table_iter_init(&iter, cmd->params);
+        while (g_hash_table_iter_next(&iter, (gpointer *) & key, (gpointer *) & value)) {
+            device_params = stonith_key_value_add(device_params, key, value);
+        }
+    }
+
+    /* The fencer will automatically register devices via CIB notifications
+     * when the CIB changes, but to avoid a possible race condition between
+     * the fencer receiving the notification and the executor requesting that
+     * resource, the executor registers the device as well. The fencer knows how
+     * to handle duplicate registrations.
+     */
+    rc = stonith_api->cmds->register_device(stonith_api, st_opt_sync_call,
+                                            cmd->rsc_id, rsc->provider,
+                                            rsc->type, device_params);
+
+    stonith_key_value_freeall(device_params, 1, 1);
+    return rc;
+}
+
+/*!
+ * \internal
+ * \brief Execute a stonith resource "stop" action
+ *
+ * Stop a stonith resource by unregistering it with the fencer.
+ * (Stonith agents don't have a stop command.)
+ *
+ * \param[in] stonith_api  Connection to fencer
+ * \param[in] rsc          Stonith resource to stop
+ *
+ * \return pcmk_ok on success, -errno otherwise
+ */
+static inline int
+execd_stonith_stop(stonith_t *stonith_api, const lrmd_rsc_t *rsc)
+{
+    /* @TODO Failure would indicate a problem communicating with fencer;
+     * perhaps we should try reconnecting and retrying a few times?
+     */
+    return stonith_api->cmds->remove_device(stonith_api, st_opt_sync_call,
+                                            rsc->rsc_id);
+}
+
+/*!
+ * \internal
+ * \brief Initiate a stonith resource agent recurring "monitor" action
+ *
+ * \param[in] stonith_api  Connection to fencer
+ * \param[in] rsc          Stonith resource to monitor
+ * \param[in] cmd          Monitor command being executed
+ *
+ * \return pcmk_ok if monitor was successfully initiated, -errno otherwise
+ */
+static inline int
+execd_stonith_monitor(stonith_t *stonith_api, lrmd_rsc_t *rsc, lrmd_cmd_t *cmd)
+{
+    int rc = stonith_api->cmds->monitor(stonith_api, 0, cmd->rsc_id,
+                                        cmd->timeout / 1000);
+
+    rc = stonith_api->cmds->register_callback(stonith_api, rc, 0, 0, cmd,
+                                              "lrmd_stonith_callback",
+                                              lrmd_stonith_callback);
+    if (rc == TRUE) {
+        rsc->active = cmd;
+        rc = pcmk_ok;
+    } else {
+        rc = -pcmk_err_generic;
+    }
+    return rc;
+}
+
+static void
 lrmd_rsc_execute_stonith(lrmd_rsc_t * rsc, lrmd_cmd_t * cmd)
 {
     int rc = 0;
-    int do_monitor = 0;
+    bool do_monitor = FALSE;
 
     stonith_t *stonith_api = get_stonith_connection();
 
     if (!stonith_api) {
-        cmd->exec_rc = get_uniform_rc(PCMK_RESOURCE_CLASS_STONITH, cmd->action,
-                                      -ENOTCONN);
-        cmd->lrmd_op_status = PCMK_LRM_OP_ERROR;
-        cmd_finalize(cmd, rsc);
-        return -EUNATCH;
-    }
+        rc = -ENOTCONN;
 
-    if (safe_str_eq(cmd->action, "start")) {
-        char *key = NULL;
-        char *value = NULL;
-        stonith_key_value_t *device_params = NULL;
-
-        if (cmd->params) {
-            GHashTableIter iter;
-
-            g_hash_table_iter_init(&iter, cmd->params);
-            while (g_hash_table_iter_next(&iter, (gpointer *) & key, (gpointer *) & value)) {
-                device_params = stonith_key_value_add(device_params, key, value);
-            }
-        }
-
-        /* Stonith automatically registers devices from the IPC when changes
-         * occur, but to avoid a possible race condition between stonith
-         * receiving the IPC update and the executor requesting that resource,
-         * the executor still registers the device as well. Stonith knows how to
-         * handle duplicate device registrations correctly.
-         */
-        rc = stonith_api->cmds->register_device(stonith_api,
-                                                st_opt_sync_call,
-                                                cmd->rsc_id,
-                                                rsc->provider, rsc->type, device_params);
-
-        stonith_key_value_freeall(device_params, 1, 1);
+    } else if (safe_str_eq(cmd->action, "start")) {
+        rc = execd_stonith_start(stonith_api, rsc, cmd);
         if (rc == 0) {
-            do_monitor = 1;
+            do_monitor = TRUE;
         }
+
     } else if (safe_str_eq(cmd->action, "stop")) {
-        rc = stonith_api->cmds->remove_device(stonith_api, st_opt_sync_call, cmd->rsc_id);
-        rsc->stonith_started = 0;
+        rc = execd_stonith_stop(stonith_api, rsc);
+
     } else if (safe_str_eq(cmd->action, "monitor")) {
         if (cmd->interval_ms > 0) {
-            do_monitor = 1;
+            do_monitor = TRUE;
         } else {
-            rc = rsc->stonith_started ? 0 : -ENODEV;
+            rc = rsc->st_probe_rc;
         }
     }
 
-    if (!do_monitor) {
-        goto cleanup_stonith_exec;
+    if (do_monitor) {
+        rc = execd_stonith_monitor(stonith_api, rsc, cmd);
+        if (rc == pcmk_ok) {
+            // Don't clean up yet, we will find out result of the monitor later
+            return;
+        }
     }
 
-    rc = stonith_api->cmds->monitor(stonith_api, 0, cmd->rsc_id, cmd->timeout / 1000);
-
-    rc = stonith_api->cmds->register_callback(stonith_api,
-                                              rc,
-                                              0,
-                                              0,
-                                              cmd, "lrmd_stonith_callback", lrmd_stonith_callback);
-
-    /* don't cleanup yet, we will find out the result of the monitor later */
-    if (rc > 0) {
-        rsc->active = cmd;
-        return rc;
-    } else if (rc == 0) {
-        rc = -1;
-    }
-
-  cleanup_stonith_exec:
     stonith_action_complete(cmd, rc);
-    return rc;
 }
 
 static int
@@ -1184,7 +1418,12 @@ lrmd_rsc_execute(lrmd_rsc_t * rsc)
         rsc->pending_ops = g_list_remove_link(rsc->pending_ops, first);
         g_list_free_1(first);
 
-#ifdef HAVE_SYS_TIMEB_H
+#ifdef PCMK__TIME_USE_CGT
+        if (cmd->t_first_run.tv_sec == 0) {
+            clock_gettime(CLOCK_MONOTONIC, &cmd->t_first_run);
+        }
+        clock_gettime(CLOCK_MONOTONIC, &cmd->t_run);
+#elif defined(HAVE_SYS_TIMEB_H)
         if (cmd->t_first_run.time == 0) {
             ftime(&cmd->t_first_run);
         }
@@ -1317,17 +1556,13 @@ process_lrmd_rsc_register(crm_client_t * client, uint32_t id, xmlNode * request)
         safe_str_eq(rsc->class, dup->class) &&
         safe_str_eq(rsc->provider, dup->provider) && safe_str_eq(rsc->type, dup->type)) {
 
-        crm_warn("Can't add, RSC '%s' already present in the rsc list (%d active resources)",
-                 rsc->rsc_id, g_hash_table_size(rsc_list));
-
+        crm_notice("Ignoring duplicate registration of '%s'", rsc->rsc_id);
         free_rsc(rsc);
         return rc;
     }
 
     g_hash_table_replace(rsc_list, rsc->rsc_id, rsc);
-    crm_info("Added '%s' to the rsc list (%d active resources)",
-             rsc->rsc_id, g_hash_table_size(rsc_list));
-
+    crm_info("Cached agent information for '%s'", rsc->rsc_id);
     return rc;
 }
 
@@ -1345,8 +1580,7 @@ process_lrmd_get_rsc_info(xmlNode *request, int call_id)
     } else {
         rsc = g_hash_table_lookup(rsc_list, rsc_id);
         if (rsc == NULL) {
-            crm_info("Resource '%s' not found (%d active resources)",
-                     rsc_id, g_hash_table_size(rsc_list));
+            crm_info("Agent information for '%s' not in cache", rsc_id);
             rc = -ENODEV;
         }
     }
@@ -1373,15 +1607,17 @@ process_lrmd_rsc_unregister(crm_client_t * client, uint32_t id, xmlNode * reques
         return -ENODEV;
     }
 
-    if (!(rsc = g_hash_table_lookup(rsc_list, rsc_id))) {
-        crm_info("Resource '%s' not found (%d active resources)",
-                 rsc_id, g_hash_table_size(rsc_list));
+    rsc = g_hash_table_lookup(rsc_list, rsc_id);
+    if (rsc == NULL) {
+        crm_info("Ignoring unregistration of resource '%s', which is not registered",
+                 rsc_id);
         return pcmk_ok;
     }
 
     if (rsc->active) {
         /* let the caller know there are still active ops on this rsc to watch for */
-        crm_trace("Operation still in progress: %p", rsc->active);
+        crm_trace("Operation (0x%p) still in progress for unregistered resource %s",
+                  rsc->active, rsc_id);
         rc = -EINPROGRESS;
     }
 
