@@ -1,5 +1,5 @@
 /*
- * Copyright 2009-2018 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2009-2019 the Pacemaker project contributors
  *
  * This source code is licensed under the GNU General Public License version 2
  * or later (GPLv2+) WITHOUT ANY WARRANTY.
@@ -149,12 +149,21 @@ typedef struct remote_fencing_op_s {
 
 } remote_fencing_op_t;
 
+/*!
+ * \internal
+ * \brief Broadcast the result of an operation to the peers.
+ * \param op, Operation whose result should be broadcast
+ * \param rc, Result of the operation
+ */
+void stonith_bcast_result_to_peers(remote_fencing_op_t * op, int rc);
+
 enum st_callback_flags {
-    st_callback_unknown        = 0x0000,
-    st_callback_notify_fence   = 0x0001,
-    st_callback_device_add     = 0x0004,
-    st_callback_device_del     = 0x0010,
-    st_callback_notify_history = 0x0020
+    st_callback_unknown               = 0x0000,
+    st_callback_notify_fence          = 0x0001,
+    st_callback_device_add            = 0x0004,
+    st_callback_device_del            = 0x0010,
+    st_callback_notify_history        = 0x0020,
+    st_callback_notify_history_synced = 0x0040
 };
 
 /*
@@ -245,16 +254,6 @@ gboolean string_in_list(GListPtr list, const char *item);
 
 gboolean node_has_attr(const char *node, const char *name, const char *value);
 
-void
-schedule_internal_command(const char *origin,
-                          stonith_device_t * device,
-                          const char *action,
-                          const char *victim,
-                          int timeout,
-                          void *internal_user_data,
-                          void (*done_cb) (GPid pid, int rc, const char *output,
-                                           gpointer user_data));
-
 extern char *stonith_our_uname;
 extern gboolean stand_alone;
 extern GHashTable *device_list;
