@@ -1,19 +1,10 @@
-/* 
- * Copyright (C) 2005 Andrew Beekhof <andrew@beekhof.net>
- * 
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- * 
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+/*
+ * Copyright 2005-2019 the Pacemaker project contributors
+ *
+ * The version control history for this file may have further details.
+ *
+ * This source code is licensed under the GNU Lesser General Public License
+ * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
 #ifndef CRM_COMMON_ISO8601
@@ -30,13 +21,13 @@ extern "C" {
  */
 
 /*
- * http://en.wikipedia.org/wiki/ISO_8601
- *
+ * See https://en.wikipedia.org/wiki/ISO_8601
  */
 
 #  include <time.h>
 #  include <ctype.h>
-#  include <stdbool.h>
+#  include <stdint.h>   // uint32_t
+#  include <stdbool.h>  // bool
 
 typedef struct crm_time_s crm_time_t;
 
@@ -46,10 +37,7 @@ typedef struct crm_time_period_s {
     crm_time_t *diff;
 } crm_time_period_t;
 
-/* Creates a new date/time object conforming to iso8601:
- *     http://en.wikipedia.org/wiki/ISO_8601
- *
- * Eg.
+/* Creates a new date/time object conforming to ISO 8601, for example:
  *   Ordinal:   2010-01 12:00:00 +10:00
  *   Gregorian: 2010-01-01 12:00:00 +10:00
  *   ISO Week:  2010-W53-6 12:00:00 +10:00
@@ -58,13 +46,15 @@ typedef struct crm_time_period_s {
  *   Only one of date, time is required
  *   If date or timezone is unspecified, they default to the current one
  *   Supplying NULL results in the current date/time
- *   Dashes may be ommitted from dates
- *   Colons may be ommitted from times and timezones
- *   A timezone of 'Z' denoted UTC time
+ *   Dashes may be omitted from dates
+ *   Colons may be omitted from times and timezones
+ *   A timezone of 'Z' denotes UTC time
  */
 crm_time_t *crm_time_new(const char *string);
+crm_time_t *crm_time_new_undefined(void);
 void crm_time_free(crm_time_t * dt);
 
+bool crm_time_is_defined(const crm_time_t *t);
 char *crm_time_as_string(crm_time_t * dt, int flags);
 
 #  define crm_time_log(level, prefix, dt, flags) crm_time_log_alias(level, __FILE__, __FUNCTION__, __LINE__, prefix, dt, flags)
@@ -84,6 +74,7 @@ void crm_time_log_alias(int log_level, const char *file, const char *function, i
 crm_time_t *crm_time_parse_duration(const char *duration_str);
 crm_time_t *crm_time_calculate_duration(crm_time_t * dt, crm_time_t * value);
 crm_time_period_t *crm_time_parse_period(const char *period_str);
+void crm_time_free_period(crm_time_period_t *period);
 
 int crm_time_compare(crm_time_t * dt, crm_time_t * rhs);
 
