@@ -1,66 +1,51 @@
-/* 
- * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
- * 
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- * 
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+/*
+ * Copyright 2004-2019 the Pacemaker project contributors
+ *
+ * The version control history for this file may have further details.
+ *
+ * This source code is licensed under the GNU Lesser General Public License
+ * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
+
 #ifndef CRM_COMMON_UTIL__H
 #  define CRM_COMMON_UTIL__H
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /**
  * \file
  * \brief Utility functions
  * \ingroup core
  */
 
-#  include <sys/types.h>
+#  include <sys/types.h>    // gid_t, mode_t, size_t, time_t, uid_t
 #  include <stdlib.h>
 #  include <stdbool.h>
-#  include <stdint.h> // uint32_t
+#  include <stdint.h>       // uint32_t
 #  include <limits.h>
 #  include <signal.h>
-#  include <sysexits.h>
 #  include <glib.h>
 
 #  include <libxml/tree.h>
 
 #  include <crm/lrmd.h>
+#  include <crm/common/results.h>
 
-#  if SUPPORT_HEARTBEAT
-#    include <heartbeat.h>
-#  else
-#    define	NORMALNODE	"normal"
-#    define	ACTIVESTATUS	"active"/* fully functional, and all links are up */
-#    define	DEADSTATUS	"dead"
-                                /* Status of non-working link or machine */
-#    define	PINGSTATUS	"ping"
-                                /* Status of a working ping node */
-#    define	JOINSTATUS	"join"
-                                /* Status when an api client joins */
-#    define	LEAVESTATUS	"leave"
-                                /* Status when an api client leaves */
-#    define	ONLINESTATUS	"online"/* Status of an online client */
-#    define	OFFLINESTATUS	"offline"
-                                        /* Status of an offline client */
-#  endif
+#  define ONLINESTATUS  "online"  // Status of an online client
+#  define OFFLINESTATUS "offline" // Status of an offline client
+
+// public name/value pair functions (from nvpair.c)
+int pcmk_scan_nvpair(const char *input, char **name, char **value);
+char *pcmk_format_nvpair(const char *name, const char *value, const char *units);
+char *pcmk_format_named_time(const char *name, time_t epoch_time);
 
 /* public Pacemaker Remote functions (from remote.c) */
 int crm_default_remote_port(void);
 
 /* public string functions (from strings.c) */
 char *crm_itoa_stack(int an_int, char *buf, size_t len);
-char *crm_itoa(int an_int);
 gboolean crm_is_true(const char *s);
 int crm_str_to_boolean(const char *s, int *ret);
 long long crm_parse_ll(const char *text, const char *default_text);
@@ -68,23 +53,36 @@ int crm_parse_int(const char *text, const char *default_text);
 char * crm_strip_trailing_newline(char *str);
 gboolean crm_str_eq(const char *a, const char *b, gboolean use_case);
 gboolean safe_str_neq(const char *a, const char *b);
+gboolean crm_strcase_equal(gconstpointer a, gconstpointer b);
 guint crm_strcase_hash(gconstpointer v);
 guint g_str_hash_traditional(gconstpointer v);
+char *crm_strdup_printf(char const *format, ...) __attribute__ ((__format__ (__printf__, 1, 2)));
 
 #  define safe_str_eq(a, b) crm_str_eq(a, b, FALSE)
 #  define crm_str_hash g_str_hash_traditional
 
-/* used with hash tables where case does not matter */
-static inline gboolean
-crm_strcase_equal(gconstpointer a, gconstpointer b)
+static inline char *
+crm_itoa(int an_int)
 {
-    return crm_str_eq((const char *) a, (const char *) b, FALSE);
+    return crm_strdup_printf("%d", an_int);
+}
+
+static inline char *
+crm_ftoa(double a_float)
+{
+    return crm_strdup_printf("%f", a_float);
+}
+
+static inline char *
+crm_ttoa(time_t epoch_time)
+{
+    return crm_strdup_printf("%lld", (long long) epoch_time);
 }
 
 /*!
  * \brief Create hash table with dynamically allocated string keys/values
  *
- * \return Newly hash table
+ * \return Newly allocated hash table
  * \note It is the caller's responsibility to free the result, using
  *       g_hash_table_destroy().
  */
@@ -97,7 +95,7 @@ crm_str_table_new()
 /*!
  * \brief Create hash table with case-insensitive dynamically allocated string keys/values
  *
- * \return Newly hash table
+ * \return Newly allocated hash table
  * \note It is the caller's responsibility to free the result, using
  *       g_hash_table_destroy().
  */
@@ -115,14 +113,17 @@ GHashTable *crm_str_table_dup(GHashTable *old_table);
 void crm_build_path(const char *path_c, mode_t mode);
 
 long long crm_get_msec(const char *input);
-unsigned long long crm_get_interval(const char *input);
+guint crm_parse_interval_spec(const char *input);
 int char2score(const char *score);
 char *score2char(int score);
 char *score2char_stack(int score, char *buf, size_t len);
 
+// deprecated
+#define crm_get_interval crm_parse_interval_spec
+
 /* public operation functions (from operations.c) */
 gboolean parse_op_key(const char *key, char **rsc_id, char **op_type,
-                      int *interval);
+                      guint *interval_ms);
 gboolean decode_transition_key(const char *key, char **uuid, int *action,
                                int *transition_id, int *target_rc);
 gboolean decode_transition_magic(const char *magic, char **uuid,
@@ -132,7 +133,7 @@ int rsc_op_expected_rc(lrmd_event_data_t *event);
 gboolean did_rsc_op_fail(lrmd_event_data_t *event, int target_rc);
 bool crm_op_needs_metadata(const char *rsc_class, const char *op);
 xmlNode *crm_create_op_xml(xmlNode *parent, const char *prefix,
-                           const char *task, const char *interval,
+                           const char *task, const char *interval_spec,
                            const char *timeout);
 #define CRM_DEFAULT_OP_TIMEOUT_S "20s"
 
@@ -203,7 +204,12 @@ int crm_user_lookup(const char *name, uid_t * uid, gid_t * gid);
 void crm_gnutls_global_init(void);
 #endif
 
-int crm_exit(int rc);
 bool pcmk_acl_required(const char *user);
 
+char *pcmk_hostname(void);
+
+#ifdef __cplusplus
+}
+#endif
+
 #endif
