@@ -1,5 +1,7 @@
 /*
- * Copyright 2012-2018 David Vossel <davidvossel@gmail.com>
+ * Copyright 2012-2019 the Pacemaker project contributors
+ *
+ * The version control history for this file may have further details.
  *
  * This source code is licensed under the GNU Lesser General Public License
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
@@ -17,7 +19,8 @@ extern "C" {
  * \brief Resource agent executor
  * \ingroup lrmd
  */
-#include <stdbool.h>
+#include <stdbool.h>      // bool
+#include <glib.h>         // guint, GList
 #include <crm_config.h>
 #include <crm/services.h>
 
@@ -200,7 +203,7 @@ typedef struct lrmd_event_data_s {
     const char *rsc_id;
     /*! The action performed, start, stop, monitor... */
     const char *op_type;
-    /*! The userdata string given do exec() api function */
+    /*! The user data passed by caller of exec() API function */
     const char *user_data;
 
     /*! The client api call id associated with this event */
