@@ -1,34 +1,25 @@
 /*
- * Copyright (C) 2010 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2010-2019 the Pacemaker project contributors
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * The version control history for this file may have further details.
  *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * This source code is licensed under the GNU Lesser General Public License
+ * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
+#ifndef __PCMK_SERVICES__
+#  define __PCMK_SERVICES__
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /**
  * \file
  * \brief Services API
  * \ingroup core
  */
 
-#ifndef __PCMK_SERVICES__
-#  define __PCMK_SERVICES__
-
-#  ifdef __cplusplus
-extern "C" {
-#  endif
-
 #  include <glib.h>
 #  include <stdio.h>
 #  include <string.h>
@@ -48,25 +39,19 @@ extern "C" {
 #    define SYSTEMCTL "/bin/systemctl"
 #  endif
 
-/* Deprecated and unused by Pacemaker, kept for API backward compatibility */
-#  ifndef SERVICE_SCRIPT
-#    define SERVICE_SCRIPT "/sbin/service"
-#  endif
-
 /* Known resource classes */
 #define PCMK_RESOURCE_CLASS_OCF     "ocf"
 #define PCMK_RESOURCE_CLASS_SERVICE "service"
 #define PCMK_RESOURCE_CLASS_LSB     "lsb"
 #define PCMK_RESOURCE_CLASS_SYSTEMD "systemd"
 #define PCMK_RESOURCE_CLASS_UPSTART "upstart"
-#define PCMK_RESOURCE_CLASS_HB      "heartbeat"
 #define PCMK_RESOURCE_CLASS_NAGIOS  "nagios"
 #define PCMK_RESOURCE_CLASS_STONITH "stonith"
 
-/* This is the string passed in the OCF_EXIT_REASON_PREFIX
- * environment variable. The stderr output that occurs
- * after this prefix is encountered is considered the exit
- * reason for a completed operationt */
+/* This is the string passed in the OCF_EXIT_REASON_PREFIX environment variable.
+ * The stderr output that occurs after this prefix is encountered is considered
+ * the exit reason for a completed operation.
+ */
 #define PCMK_OCF_REASON_PREFIX "ocf-exit-reason:"
 
 // Agent version to use if agent doesn't specify one
@@ -115,7 +100,7 @@ enum ocf_exitcode {
 
 
     /* 150-199	reserved for application use */
-    PCMK_OCF_CONNECTION_DIED = 189, /* Operation failure implied by disconnection of the LRM API to a local or remote node */
+    PCMK_OCF_CONNECTION_DIED = 189, // Deprecated (see PCMK_LRM_OP_NOT_CONNECTED)
 
     PCMK_OCF_DEGRADED        = 190, /* Active resource that is no longer 100% functional */
     PCMK_OCF_DEGRADED_MASTER = 191, /* Promoted resource that is no longer 100% functional */
@@ -131,6 +116,7 @@ enum ocf_exitcode {
 };
 
 enum op_status {
+    PCMK_LRM_OP_UNKNOWN = -2,
     PCMK_LRM_OP_PENDING = -1,
     PCMK_LRM_OP_DONE,
     PCMK_LRM_OP_CANCELLED,
@@ -140,6 +126,8 @@ enum op_status {
     PCMK_LRM_OP_ERROR_HARD,
     PCMK_LRM_OP_ERROR_FATAL,
     PCMK_LRM_OP_NOT_INSTALLED,
+    PCMK_LRM_OP_NOT_CONNECTED,
+    PCMK_LRM_OP_INVALID,
 };
 
 enum nagios_exitcode {
@@ -156,6 +144,7 @@ enum nagios_exitcode {
 enum svc_action_flags {
     /* On timeout, only kill pid, do not kill entire pid group */
     SVC_ACTION_LEAVE_GROUP = 0x01,
+    SVC_ACTION_NON_BLOCKED = 0x02,
 };
 
 typedef struct svc_action_private_s svc_action_private_t;
@@ -163,14 +152,17 @@ typedef struct svc_action_s {
     char *id;
     char *rsc;
     char *action;
-    int interval;
+    guint interval_ms;
 
     char *standard;
     char *provider;
     char *agent;
 
     int timeout;
-    GHashTable *params; /* used by OCF agents and alert agents */
+    GHashTable *params; /* used for setting up environment for ocf-ra &
+                           alert agents
+                           and to be sent via stdin for fence-agents
+                         */
 
     int rc;
     int pid;
@@ -244,20 +236,31 @@ typedef struct svc_action_s {
  */
     GList *resources_list_standards(void);
 
-    svc_action_t *services_action_create(const char *name, const char *action,
-                                         int interval /* ms */ , int timeout /* ms */ );
+/**
+ * Does the given standard, provider, and agent describe a resource that can exist?
+ *
+ * \param[in] standard  Which class of agent does the resource belong to?
+ * \param[in] provider  What provides the agent (NULL for most standards)?
+ * \param[in] agent     What is the name of the agent?
+ *
+ * \return A boolean
+ */
+    gboolean resources_agent_exists(const char *standard, const char *provider, const char *agent);
+
+svc_action_t *services_action_create(const char *name, const char *action,
+                                     guint interval_ms, int timeout /* ms */);
 
 /**
  * \brief Create a new resource action
  *
- * \param[in] name     name of resource
- * \param[in] standard resource agent standard (ocf, lsb, etc.)
- * \param[in] provider resource agent provider
- * \param[in] agent    resource agent name
- * \param[in] action   action (start, stop, monitor, etc.)
- * \param[in] interval how often to repeat this action, in milliseconds (if 0, execute only once)
- * \param[in] timeout  consider action failed if it does not complete in this many milliseconds
- * \param[in] params   action parameters
+ * \param[in] name        Name of resource
+ * \param[in] standard    Resource agent standard (ocf, lsb, etc.)
+ * \param[in] provider    Resource agent provider
+ * \param[in] agent       Resource agent name
+ * \param[in] action      action (start, stop, monitor, etc.)
+ * \param[in] interval_ms How often to repeat this action (if 0, execute once)
+ * \param[in] timeout     Consider action failed if it does not complete in this many milliseconds
+ * \param[in] params      Action parameters
  *
  * \return newly allocated action instance
  *
@@ -265,16 +268,17 @@ typedef struct svc_action_s {
  * \note The caller is responsible for freeing the return value using
  *       services_action_free().
  */
-    svc_action_t *resources_action_create(const char *name, const char *standard,
-                                          const char *provider, const char *agent,
-                                          const char *action, int interval /* ms */ ,
-                                          int timeout /* ms */ , GHashTable * params,
-                                          enum svc_action_flags flags);
+svc_action_t *resources_action_create(const char *name, const char *standard,
+                                      const char *provider, const char *agent,
+                                      const char *action, guint interval_ms,
+                                      int timeout /* ms */, GHashTable *params,
+                                      enum svc_action_flags flags);
 
 /**
  * Kick a recurring action so it is scheduled immediately for re-execution
  */
-    gboolean services_action_kick(const char *name, const char *action, int interval /* ms */);
+gboolean services_action_kick(const char *name, const char *action,
+                              guint interval_ms);
 
     const char *resources_find_service_class(const char *agent);
 
@@ -304,13 +308,20 @@ typedef struct svc_action_s {
  *
  * \param[in] op services action data
  * \param[in] action_callback callback for when the action completes
+ * \param[in] action_fork_callback callback for when action forked successfully
  *
  * \retval TRUE succesfully started execution
  * \retval FALSE failed to start execution, no callback will be received
  */
-    gboolean services_action_async(svc_action_t * op, void (*action_callback) (svc_action_t *));
+    gboolean services_action_async_fork_notify(svc_action_t * op,
+        void (*action_callback) (svc_action_t *),
+        void (*action_fork_callback) (svc_action_t *));
 
-    gboolean services_action_cancel(const char *name, const char *action, int interval);
+    gboolean services_action_async(svc_action_t * op,
+                                   void (*action_callback) (svc_action_t *));
+
+gboolean services_action_cancel(const char *name, const char *action,
+                                guint interval_ms);
 
 /* functions for alert agents */
 svc_action_t *services_alert_create(const char *id, const char *exec,
@@ -329,6 +340,8 @@ gboolean services_alert_async(svc_action_t *action,
                 case PCMK_LRM_OP_NOTSUPPORTED:return "NOT SUPPORTED";
                 case PCMK_LRM_OP_ERROR:return "Error";
                 case PCMK_LRM_OP_NOT_INSTALLED:return "Not installed";
+                case PCMK_LRM_OP_NOT_CONNECTED:return "No executor connection";
+                case PCMK_LRM_OP_INVALID:return "Cannot execute now";
                 default:return "UNKNOWN!";
         }
     }
@@ -338,7 +351,7 @@ gboolean services_alert_async(svc_action_t *action,
             case PCMK_OCF_OK:
                 return "ok";
             case PCMK_OCF_UNKNOWN_ERROR:
-                return "unknown error";
+                return "error";
             case PCMK_OCF_INVALID_PARAM:
                 return "invalid parameter";
             case PCMK_OCF_UNIMPLEMENT_FEATURE:
