@@ -1,21 +1,16 @@
 /*
- * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2004-2019 the Pacemaker project contributors
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * The version control history for this file may have further details.
  *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * This source code is licensed under the GNU Lesser General Public License
+ * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /**
  * \file
  * \brief Fencing aka. STONITH
@@ -27,10 +22,9 @@
 
 #  include <dlfcn.h>
 #  include <errno.h>
-#  include <stdbool.h>
-
-/* TO-DO: Work out how to drop this requirement */
-#  include <libxml/tree.h>
+#  include <stdbool.h>  // bool
+#  include <stdint.h>   // uint32_t
+#  include <time.h>     // time_t
 
 #  define T_STONITH_NOTIFY_DISCONNECT     "st_notify_disconnect"
 #  define T_STONITH_NOTIFY_FENCE          "st_notify_fence"
@@ -106,11 +100,10 @@ typedef struct stonith_history_s {
     char *action;
     char *origin;
     char *delegate;
-    int completed;
-    int state;
-
-    struct stonith_history_s *next;
     char *client;
+    int state;
+    time_t completed;
+    struct stonith_history_s *next;
 } stonith_history_t;
 
 typedef struct stonith_s stonith_t;
@@ -186,7 +179,7 @@ typedef struct stonith_api_operations_s
      */
     int (*register_device)(
         stonith_t *st, int options, const char *id,
-        const char *namespace, const char *agent, stonith_key_value_t *params);
+        const char *provider, const char *agent, stonith_key_value_t *params);
 
     /*!
      * \brief Remove a fencing level for a specific node.
@@ -220,19 +213,19 @@ typedef struct stonith_api_operations_s
      * \retval negative error code on failure
      */
     int (*metadata)(stonith_t *st, int options,
-            const char *device, const char *namespace, char **output, int timeout);
+            const char *device, const char *provider, char **output, int timeout);
 
     /*!
      * \brief Retrieve a list of installed stonith agents
      *
-     * \note if namespace is not provided, all known agents will be returned
+     * \note if provider is not provided, all known agents will be returned
      * \note list must be freed using stonith_key_value_freeall()
      * \note call_options parameter is not used, it is reserved for future use.
      *
      * \retval num items in list on success
      * \retval negative error code on failure
      */
-    int (*list_agents)(stonith_t *stonith, int call_options, const char *namespace,
+    int (*list_agents)(stonith_t *stonith, int call_options, const char *provider,
             stonith_key_value_t **devices, int timeout);
 
     /*!
@@ -313,18 +306,21 @@ typedef struct stonith_api_operations_s
     int (*remove_notification)(stonith_t *st, const char *event);
 
     /*!
-     * \brief Register a callback to receive the result of an async call id
+     * \brief Register a callback to receive the result of an asynchronous call
      *
-     * \param call_id, The call id to register the callback for.
-     * \param timeout, The default timeout period to wait until this callback expires
-     * \param options, Option flags, st_opt_timeout_updates and st_opt_report_only_success are the
-     *                 only valid options for this function.
-     * \param userdate, A pointer that will be handed back in the callback.
-     * \param callback_name, Unique name given to callback
-     * \param callback, The callback function
+     * \param[in] call_id        The call ID to register callback for
+     * \param[in] timeout        Default time to wait until callback expires
+     * \param[in] options        Bitmask of \c stonith_call_options (respects
+     *                           \c st_opt_timeout_updates and
+     *                           \c st_opt_report_only_success)
+     * \param[in] userdata       Pointer that will be given to callback
+     * \param[in] callback_name  Unique name to identify callback
+     * \param[in] callback       The callback function to register
      *
-     * \retval 0 success
-     * \retval negative error code on failure.
+     * \return \c TRUE on success, \c FALSE if call_id is negative, -errno otherwise
+     *
+     * \todo This function should return \c pcmk_ok on success, and \c call_id
+     *       when negative, but that would break backward compatibility.
      */
     int (*register_callback)(stonith_t *st,
         int call_id,
@@ -342,8 +338,8 @@ typedef struct stonith_api_operations_s
     /*!
      * \brief Remove fencing level for specific node, node regex or attribute
      *
-     * \param[in] st      Stonithd connection to use
-     * \param[in] options Bitmask of stonith_call_options to pass to stonithd
+     * \param[in] st      Fencer connection to use
+     * \param[in] options Bitmask of stonith_call_options to pass to the fencer
      * \param[in] node    If not NULL, target level by this node name
      * \param[in] pattern If not NULL, target by node name using this regex
      * \param[in] attr    If not NULL, target by this node attribute
@@ -362,8 +358,8 @@ typedef struct stonith_api_operations_s
     /*!
      * \brief Register fencing level for specific node, node regex or attribute
      *
-     * \param[in] st          Stonithd connection to use
-     * \param[in] options     Bitmask of stonith_call_options to pass to stonithd
+     * \param[in] st          Fencer connection to use
+     * \param[in] options     Bitmask of stonith_call_options to pass to fencer
      * \param[in] node        If not NULL, target level by this node name
      * \param[in] pattern     If not NULL, target by node name using this regex
      * \param[in] attr        If not NULL, target by this node attribute
@@ -404,26 +400,6 @@ typedef struct stonith_api_operations_s
                     stonith_key_value_t *params, int timeout, char **output,
                     char **error_output);
 
-    /*!
-     * \brief Issue a fencing action against a node with requested fencing delay.
-     *
-     * \note Possible actions are, 'on', 'off', and 'reboot'.
-     *
-     * \param st, stonith connection
-     * \param options, call options
-     * \param node, The target node to fence
-     * \param action, The fencing action to take
-     * \param timeout, The default per device timeout to use with each device
-     *                 capable of fencing the target.
-     * \param delay, Apply a fencing delay. Value -1 means disable also any
-     *               static/random fencing delays from pcmk_delay_base/max
-     *
-     * \retval 0 success
-     * \retval negative error code on failure.
-     */
-    int (*fence_with_delay)(stonith_t *st, int options, const char *node, const char *action,
-                            int timeout, int tolerance, int delay);
-
 } stonith_api_operations_t;
 
 struct stonith_s
@@ -432,7 +408,7 @@ struct stonith_s
 
     int call_id;
     int call_timeout;
-    void *private;
+    void *st_private;
 
     stonith_api_operations_t *cmds;
 };
@@ -455,6 +431,10 @@ void stonith_key_value_freeall(stonith_key_value_t * kvp, int keys, int values);
 
 void stonith_history_free(stonith_history_t *history);
 
+// Convenience functions
+int stonith_api_connect_retry(stonith_t *st, const char *name,
+                              int max_attempts);
+
 /* Basic helpers that allows nodes to be fenced and the history to be
  * queried without mainloop or the caller understanding the full API
  *
@@ -503,19 +483,22 @@ time_t stonith_api_time(uint32_t nodeid, const char *uname, bool in_progress);
 
  */
 
-#  define STONITH_LIBRARY "libstonithd.so.2"
+#  define STONITH_LIBRARY "libstonithd.so.26"
+
+typedef int (*st_api_kick_fn) (int nodeid, const char *uname, int timeout, bool off);
+typedef time_t (*st_api_time_fn) (int nodeid, const char *uname, bool in_progress);
 
 static inline int
 stonith_api_kick_helper(uint32_t nodeid, int timeout, bool off)
 {
     static void *st_library = NULL;
-    static int (*st_kick_fn) (int nodeid, const char *uname, int timeout, bool off) = NULL;
+    static st_api_kick_fn st_kick_fn;
 
     if (st_library == NULL) {
         st_library = dlopen(STONITH_LIBRARY, RTLD_LAZY);
     }
     if (st_library && st_kick_fn == NULL) {
-        st_kick_fn = dlsym(st_library, "stonith_api_kick");
+        st_kick_fn = (st_api_kick_fn) dlsym(st_library, "stonith_api_kick");
     }
     if (st_kick_fn == NULL) {
 #ifdef ELIBACC
@@ -532,13 +515,13 @@ static inline time_t
 stonith_api_time_helper(uint32_t nodeid, bool in_progress)
 {
     static void *st_library = NULL;
-    static time_t(*st_time_fn) (int nodeid, const char *uname, bool in_progress) = NULL;
+    static st_api_time_fn st_time_fn;
 
     if (st_library == NULL) {
         st_library = dlopen(STONITH_LIBRARY, RTLD_LAZY);
     }
     if (st_library && st_time_fn == NULL) {
-        st_time_fn = dlsym(st_library, "stonith_api_time");
+        st_time_fn = (st_api_time_fn) dlsym(st_library, "stonith_api_time");
     }
     if (st_time_fn == NULL) {
         return 0;
@@ -547,4 +530,26 @@ stonith_api_time_helper(uint32_t nodeid, bool in_progress)
     return (*st_time_fn) (nodeid, NULL, in_progress);
 }
 
+/**
+ * Does the given agent describe a stonith resource that can exist?
+ *
+ * \param[in] agent     What is the name of the agent?
+ * \param[in] timeout   Timeout to use when querying.  If 0 is given,
+ *                      use a default of 120.
+ *
+ * \return A boolean
+ */
+bool stonith_agent_exists(const char *agent, int timeout);
+
+/*!
+ * \brief Turn stonith action into a more readable string.
+ *
+ * \param action Stonith action
+ */
+const char *stonith_action_str(const char *action);
+
+#ifdef __cplusplus
+}
+#endif
+
 #endif
