@@ -238,9 +238,7 @@ get_shadow_file(const char *suffix)
                 }
             }
 
-            if ((dir = getenv("TMPDIR")) == NULL) {
-                dir = "/tmp";
-            }
+            dir = crm_get_tmpdir();
             if (home && home[0] == '/') {
                 int rc = 0;
 
@@ -389,7 +387,7 @@ cib_new_variant(void)
     new_cib->cmds->modify = cib_client_modify;
     new_cib->cmds->update = cib_client_update;
     new_cib->cmds->replace = cib_client_replace;
-    new_cib->cmds->delete = cib_client_delete;
+    new_cib->cmds->remove = cib_client_delete;
     new_cib->cmds->erase = cib_client_erase;
 
     new_cib->cmds->delete_absolute = cib_client_delete_absolute;
@@ -469,7 +467,7 @@ cib_client_add_notify_callback(cib_t * cib, const char *event,
     if (list_item != NULL) {
         crm_warn("Callback already present");
         free(new_client);
-        return -ENOTUNIQ;
+        return -EINVAL;
 
     } else {
         cib->notify_list = g_list_append(cib->notify_list, new_client);
