@@ -1,20 +1,10 @@
 /*
- * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2004-2018 Andrew Beekhof <andrew@beekhof.net>
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * This source code is licensed under the GNU Lesser General Public License
+ * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
+
 #include <crm_internal.h>
 
 #ifndef _GNU_SOURCE
@@ -55,8 +45,7 @@ GHashTable *crm_peer_cache = NULL;
  * so it would be a good idea to merge them one day.
  *
  * libcluster provides two avenues for populating the cache:
- * crm_remote_peer_get(), crm_remote_peer_cache_add() and
- * crm_remote_peer_cache_remove() directly manage it,
+ * crm_remote_peer_get() and crm_remote_peer_cache_remove() directly manage it,
  * while crm_remote_peer_cache_refresh() populates it via the CIB.
  */
 GHashTable *crm_remote_peer_cache = NULL;
@@ -127,20 +116,6 @@ crm_remote_peer_get(const char *node_name)
     return node;
 }
 
-/*!
- * \brief Add a node to the remote peer cache
- *
- * \param[in] node_name  Name of remote node
- *
- * \note This is a legacy convenience wrapper for crm_remote_peer_get()
- *       for callers that don't need the cache entry returned.
- */
-void
-crm_remote_peer_cache_add(const char *node_name)
-{
-    CRM_ASSERT(crm_remote_peer_get(node_name) != NULL);
-}
-
 void
 crm_remote_peer_cache_remove(const char *node_name)
 {
@@ -158,7 +133,7 @@ crm_remote_peer_cache_remove(const char *node_name)
  * \return CRM_NODE_LOST if XML_NODE_IN_CLUSTER is false in node_state,
  *         CRM_NODE_MEMBER otherwise
  * \note Unlike most boolean XML attributes, this one defaults to true, for
- *       backward compatibility with older crmd versions that don't set it.
+ *       backward compatibility with older controllers that don't set it.
  */
 static const char *
 remote_state_from_cib(xmlNode *node_state)
@@ -306,14 +281,9 @@ crm_is_peer_active(const crm_node_t * node)
         return FALSE;
     }
 #if SUPPORT_COROSYNC
-    if (is_openais_cluster()) {
+    if (is_corosync_cluster()) {
         return crm_is_corosync_peer_active(node);
     }
-#endif
-#if SUPPORT_HEARTBEAT
-    if (is_heartbeat_cluster()) {
-        return crm_is_heartbeat_peer_active(node);
-    }
 #endif
     crm_err("Unhandled cluster type: %s", name_for_cluster_type(get_cluster_type()));
     return FALSE;
@@ -409,7 +379,6 @@ destroy_crm_node(gpointer data)
 
     crm_trace("Destroying entry for node %u: %s", node->id, node->uname);
 
-    free(node->addr);
     free(node->uname);
     free(node->state);
     free(node->uuid);
@@ -627,12 +596,9 @@ crm_find_peer(unsigned int id, const char *uname)
     } else {
         /* Simple merge */
 
-        /* Only corosync based clusters use nodeid's
-         *
-         * The functions that call crm_update_peer_state() only know nodeid
-         * so 'by_id' is authorative when merging
-         *
-         * Same for crm_update_peer_proc()
+        /* Only corosync-based clusters use node IDs. The functions that call
+         * crm_update_peer_state() and crm_update_peer_proc() only know nodeid,
+         * so 'by_id' is authoritative when merging.
          */
         crm_dump_peer_hash(LOG_DEBUG, __FUNCTION__);
 
@@ -655,11 +621,9 @@ crm_remove_conflicting_peer(crm_node_t *node)
         return 0;
     }
 
-#  if !SUPPORT_PLUGIN
     if (corosync_cmap_has_config("nodelist") != 0) {
         return 0;
     }
-#  endif
 
     g_hash_table_iter_init(&iter, crm_peer_cache);
     while (g_hash_table_iter_next(&iter, NULL, (gpointer *) &existing_node)) {
@@ -753,89 +717,6 @@ crm_get_peer(unsigned int id, const char *uname)
     return node;
 }
 
-/*!
- * \internal
- * \brief Update all of a node's information (process list, state, etc.)
- *
- * \param[in] source      Caller's function name (for log messages)
- *
- * \return NULL if node was reaped from peer caches, pointer to node otherwise
- *
- * \note This function should not be called within a peer cache iteration,
- *       otherwise reaping could invalidate the iterator.
- */
-crm_node_t *
-crm_update_peer(const char *source, unsigned int id, uint64_t born, uint64_t seen, int32_t votes,
-                uint32_t children, const char *uuid, const char *uname, const char *addr,
-                const char *state)
-{
-#if SUPPORT_PLUGIN
-    gboolean addr_changed = FALSE;
-    gboolean votes_changed = FALSE;
-#endif
-    crm_node_t *node = NULL;
-
-    id = get_corosync_id(id, uuid);
-    node = crm_get_peer(id, uname);
-
-    CRM_ASSERT(node != NULL);
-
-    if (node->uuid == NULL) {
-        if (is_openais_cluster()) {
-            /* Yes, overrule whatever was passed in */
-            crm_peer_uuid(node);
-
-        } else if (uuid != NULL) {
-            node->uuid = strdup(uuid);
-        }
-    }
-
-    if (children > 0) {
-        if (crm_update_peer_proc(source, node, children, state) == NULL) {
-            return NULL;
-        }
-    }
-
-    if (state != NULL) {
-        if (crm_update_peer_state(source, node, state, seen) == NULL) {
-            return NULL;
-        }
-    }
-#if SUPPORT_HEARTBEAT
-    if (born != 0) {
-        node->born = born;
-    }
-#endif
-
-#if SUPPORT_PLUGIN
-    /* These were only used by the plugin */
-    if (born != 0) {
-        node->born = born;
-    }
-
-    if (votes > 0 && node->votes != votes) {
-        votes_changed = TRUE;
-        node->votes = votes;
-    }
-
-    if (addr != NULL) {
-        if (node->addr == NULL || crm_str_eq(node->addr, addr, FALSE) == FALSE) {
-            addr_changed = TRUE;
-            free(node->addr);
-            node->addr = strdup(addr);
-        }
-    }
-    if (addr_changed || votes_changed) {
-        crm_info("%s: Node %s: id=%u state=%s addr=%s%s votes=%d%s born=" U64T " seen=" U64T
-                 " proc=%.32x", source, node->uname, node->id, node->state,
-                 node->addr, addr_changed ? " (new)" : "", node->votes,
-                 votes_changed ? " (new)" : "", node->born, node->last_seen, node->processes);
-    }
-#endif
-
-    return node;
-}
-
 /*!
  * \internal
  * \brief Update a node's uname
@@ -878,7 +759,7 @@ crm_update_peer_uname(crm_node_t *node, const char *uname)
     }
 
 #if SUPPORT_COROSYNC
-    if (is_openais_cluster() && !is_set(node->flags, crm_remote_node)) {
+    if (is_corosync_cluster() && !is_set(node->flags, crm_remote_node)) {
         crm_remove_conflicting_peer(node);
     }
 #endif
@@ -926,13 +807,6 @@ crm_update_peer_proc(const char *source, crm_node_t * node, uint32_t flag, const
             set_bit(node->processes, flag);
             changed = TRUE;
         }
-#if SUPPORT_PLUGIN
-    } else if (safe_str_eq(status, CRM_NODE_MEMBER)) {
-        if (flag > 0 && node->processes != flag) {
-            node->processes = flag;
-            changed = TRUE;
-        }
-#endif
 
     } else if (node->processes & flag) {
         clear_bit(node->processes, flag);
@@ -1021,7 +895,7 @@ crm_update_peer_expected(const char *source, crm_node_t * node, const char *expe
  *       within a peer cache iteration if the iterator is supplied.
  */
 static crm_node_t *
-crm_update_peer_state_iter(const char *source, crm_node_t * node, const char *state, uint64_t membership, GHashTableIter *iter)
+crm_update_peer_state_iter(const char *source, crm_node_t * node, const char *state, int membership, GHashTableIter *iter)
 {
     gboolean is_member;
 
@@ -1031,21 +905,22 @@ crm_update_peer_state_iter(const char *source, crm_node_t * node, const char *st
               return NULL);
 
     is_member = safe_str_eq(state, CRM_NODE_MEMBER);
-    if (membership && is_member) {
-        node->last_seen = membership;
+    if (is_member) {
+        node->when_lost = 0;
+        if (membership) {
+            node->last_seen = membership;
+        }
     }
 
     if (state && safe_str_neq(node->state, state)) {
         char *last = node->state;
-        enum crm_status_type status_type = is_set(node->flags, crm_remote_node)?
-                                           crm_status_rstate : crm_status_nstate;
 
         node->state = strdup(state);
         crm_notice("Node %s state is now %s " CRM_XS
                    " nodeid=%u previous=%s source=%s", node->uname, state,
                    node->id, (last? last : "unknown"), source);
         if (crm_status_callback) {
-            crm_status_callback(status_type, node, last);
+            crm_status_callback(crm_status_nstate, node, last);
         }
         free(last);
 
@@ -1087,7 +962,7 @@ crm_update_peer_state_iter(const char *source, crm_node_t * node, const char *st
  *       otherwise reaping could invalidate the iterator.
  */
 crm_node_t *
-crm_update_peer_state(const char *source, crm_node_t * node, const char *state, uint64_t membership)
+crm_update_peer_state(const char *source, crm_node_t * node, const char *state, int membership)
 {
     return crm_update_peer_state_iter(source, node, state, membership, NULL);
 }
