@@ -130,7 +130,8 @@ calculate_xml_digest_v2(xmlNode * source, gboolean do_filter)
                                         crm_trace_nonlog);
     }
     if (digest_cs && digest_cs->targets) {
-        char *trace_file = crm_concat("/tmp/digest", digest, '-');
+        char *trace_file = crm_strdup_printf("%s/digest-%s",
+                                             crm_get_tmpdir(), digest);
 
         crm_trace("Saving %s.%s.%s to %s",
                   crm_element_value(source, XML_ATTR_GENERATION_ADMIN),
@@ -193,8 +194,11 @@ calculate_xml_versioned_digest(xmlNode * input, gboolean sort, gboolean do_filte
                                const char *version)
 {
     /*
+     * @COMPAT digests (on-disk or in diffs/patchsets) created <1.1.4;
+     * removing this affects even full-restart upgrades from old versions
+     *
      * The sorting associated with v1 digest creation accounted for 23% of
-     * the CIB's CPU usage on the server. v2 drops this.
+     * the CIB manager's CPU usage on the server. v2 drops this.
      *
      * The filtering accounts for an additional 2.5% and we may want to
      * remove it in future.
