@@ -17,6 +17,7 @@
 #include <crm/crm.h>
 #include <crm/msg_xml.h>
 #include <crm/common/xml.h>
+#include <crm/common/iso8601_internal.h>
 #include "crmcommon_private.h"
 
 /*
@@ -33,7 +34,7 @@
  * \internal
  * \brief Allocate a new name/value pair
  *
- * \param[in] name   New name
+ * \param[in] name   New name (required)
  * \param[in] value  New value
  *
  * \return Newly allocated name/value pair
@@ -43,11 +44,15 @@
 static pcmk_nvpair_t *
 pcmk__new_nvpair(const char *name, const char *value)
 {
-    pcmk_nvpair_t *nvpair = calloc(1, sizeof(pcmk_nvpair_t));
+    pcmk_nvpair_t *nvpair = NULL;
 
+    CRM_ASSERT(name);
+
+    nvpair = calloc(1, sizeof(pcmk_nvpair_t));
     CRM_ASSERT(nvpair);
+
     nvpair->name = strdup(name);
-    nvpair->value = strdup(value);
+    nvpair->value = value? strdup(value) : NULL;
     return nvpair;
 }
 
@@ -196,6 +201,102 @@ pcmk_nvpairs2xml_attrs(GSList *list, xmlNode *xml)
     g_slist_foreach(list, pcmk__nvpair_add_xml_attr, xml);
 }
 
+// convenience function for name=value strings
+
+/*!
+ * \brief Extract the name and value from an input string formatted as "name=value".
+ * If unable to extract them, they are returned as NULL.
+ *
+ * \param[in]  input The input string, likely from the command line
+ * \param[out] name  Everything before the first '=' in the input string
+ * \param[out] value Everything after the first '=' in the input string
+ *
+ * \return 2 if both name and value could be extracted, 1 if only one could, and
+ *         and error code otherwise
+ */
+int
+pcmk_scan_nvpair(const char *input, char **name, char **value)
+{
+#ifdef SSCANF_HAS_M
+    *name = NULL;
+    *value = NULL;
+    if (sscanf(input, "%m[^=]=%m[^\n]", name, value) <= 0) {
+        return -pcmk_err_bad_nvpair;
+    }
+#else
+    char *sep = NULL;
+    *name = NULL;
+    *value = NULL;
+
+    sep = strstr(optarg, "=");
+    if (sep == NULL) {
+        return -pcmk_err_bad_nvpair;
+    }
+
+    *name = strndup(input, sep-input);
+
+    if (*name == NULL) {
+        return -ENOMEM;
+    }
+
+    /* If the last char in optarg is =, the user gave no
+     * value for the option.  Leave it as NULL.
+     */
+    if (*(sep+1) != '\0') {
+        *value = strdup(sep+1);
+
+        if (*value == NULL) {
+            return -ENOMEM;
+        }
+    }
+#endif
+
+    if (*name != NULL && *value != NULL) {
+        return 2;
+    } else if (*name != NULL || *value != NULL) {
+        return 1;
+    } else {
+        return -pcmk_err_bad_nvpair;
+    }
+}
+
+/*!
+ * \internal
+ * \brief Format a name/value pair.
+ *
+ * Units can optionally be provided for the value.  Note that unlike most
+ * formatting functions, this one returns the formatted string.  It is
+ * assumed that the most common use of this function will be to build up
+ * a string to be output as part of other functions.
+ *
+ * \note The caller is responsible for freeing the return value after use.
+ *
+ * \param[in]     name  The name of the nvpair.
+ * \param[in]     value The value of the nvpair.
+ * \param[in]     units Optional units for the value, or NULL.
+ */
+char *
+pcmk_format_nvpair(const char *name, const char *value, const char *units) {
+    return crm_strdup_printf("%s=\"%s%s\"", name, value, units ? units : "");
+}
+
+/*!
+ * \internal
+ * \brief Format a name/time pair.
+ *
+ * See pcmk_format_nvpair() for more details.
+ *
+ * \note The caller is responsible for freeing the return value after use.
+ *
+ * \param[in]     name       The name for the time.
+ * \param[in]     epoch_time The time to format.
+ */
+char *
+pcmk_format_named_time(const char *name, time_t epoch_time) {
+    const char *now_str = crm_now_string(&epoch_time);
+    return crm_strdup_printf("%s=\"%s\"", name, now_str ? now_str : "");
+}
+
 // XML attribute handling
 
 /*!
@@ -246,7 +347,7 @@ crm_xml_add(xmlNode *node, const char *name, const char *value)
         return NULL;
     }
 
-    attr = xmlSetProp(node, (const xmlChar *)name, (const xmlChar *)value);
+    attr = xmlSetProp(node, (pcmkXmlStr) name, (pcmkXmlStr) value);
     if (dirty) {
         pcmk__mark_xml_attr_dirty(attr);
     }
@@ -296,7 +397,7 @@ crm_xml_replace(xmlNode *node, const char *name, const char *value)
         }
     }
 
-    attr = xmlSetProp(node, (const xmlChar *)name, (const xmlChar *)value);
+    attr = xmlSetProp(node, (pcmkXmlStr) name, (pcmkXmlStr) value);
     if (dirty) {
         pcmk__mark_xml_attr_dirty(attr);
     }
@@ -348,6 +449,64 @@ crm_xml_add_ms(xmlNode *node, const char *name, guint ms)
     return added;
 }
 
+// Maximum size of null-terminated string representation of 64-bit integer
+// -9223372036854775808
+#define LLSTRSIZE 21
+
+/*!
+ * \brief Create an XML attribute with specified name and long long int value
+ *
+ * This is like \c crm_xml_add() but taking a long long int value. It is a
+ * useful equivalent for defined types like time_t, etc.
+ *
+ * \param[in,out] xml    XML node to modify
+ * \param[in]     name   Attribute name to set
+ * \param[in]     value  Attribute value to set
+ *
+ * \return New value as string on success, \c NULL otherwise
+ * \note This does nothing if xml or name are \c NULL or empty.
+ *       This does not support greater than 64-bit values.
+ */
+const char *
+crm_xml_add_ll(xmlNode *xml, const char *name, long long value)
+{
+    char s[LLSTRSIZE] = { '\0', };
+
+    if (snprintf(s, LLSTRSIZE, "%lld", (long long) value) == LLSTRSIZE) {
+        return NULL;
+    }
+    return crm_xml_add(xml, name, s);
+}
+
+/*!
+ * \brief Create XML attributes for seconds and microseconds
+ *
+ * This is like \c crm_xml_add() but taking a struct timeval.
+ *
+ * \param[in,out] xml        XML node to modify
+ * \param[in]     name_sec   Name of XML attribute for seconds
+ * \param[in]     name_usec  Name of XML attribute for microseconds (or NULL)
+ * \param[in]     value      Time value to set
+ *
+ * \return New seconds value as string on success, \c NULL otherwise
+ * \note This does nothing if xml, name_sec, or value is \c NULL.
+ */
+const char *
+crm_xml_add_timeval(xmlNode *xml, const char *name_sec, const char *name_usec,
+                    const struct timeval *value)
+{
+    const char *added = NULL;
+
+    if (xml && name_sec && value) {
+        added = crm_xml_add_ll(xml, name_sec, (long long) value->tv_sec);
+        if (added && name_usec) {
+            // Any error is ignored (we successfully added seconds)
+            crm_xml_add_ll(xml, name_usec, (long long) value->tv_usec);
+        }
+    }
+    return added;
+}
+
 /*!
  * \brief Retrieve the value of an XML attribute
  *
@@ -374,7 +533,7 @@ crm_element_value(const xmlNode *data, const char *name)
     /* The first argument to xmlHasProp() has always been const,
      * but libxml2 <2.9.2 didn't declare that, so cast it
      */
-    attr = xmlHasProp((xmlNode *) data, (const xmlChar *)name);
+    attr = xmlHasProp((xmlNode *) data, (pcmkXmlStr) name);
     if (!attr || !attr->children) {
         return NULL;
     }
@@ -384,12 +543,13 @@ crm_element_value(const xmlNode *data, const char *name)
 /*!
  * \brief Retrieve the integer value of an XML attribute
  *
- * This is like \c crm_element_value() but returning the value as an integer.
+ * This is like \c crm_element_value() but getting the value as an integer.
  *
  * \param[in] data   XML node to check
  * \param[in] name   Attribute name to check
+ * \param[in] dest   Where to store element value
  *
- * \return Integer value of specified attribute on success, -1 otherwise
+ * \return 0 on success, -1 otherwise
  */
 int
 crm_element_value_int(const xmlNode *data, const char *name, int *dest)
@@ -399,8 +559,39 @@ crm_element_value_int(const xmlNode *data, const char *name, int *dest)
     CRM_CHECK(dest != NULL, return -1);
     value = crm_element_value(data, name);
     if (value) {
+        errno = 0;
+        *dest = crm_parse_int(value, NULL);
+        if (errno == 0) {
+            return 0;
+        }
+    }
+    return -1;
+}
+
+/*!
+ * \brief Retrieve the long long integer value of an XML attribute
+ *
+ * This is like \c crm_element_value() but getting the value as a long long int.
+ *
+ * \param[in] data   XML node to check
+ * \param[in] name   Attribute name to check
+ * \param[in] dest   Where to store element value
+ *
+ * \return 0 on success, -1 otherwise
+ */
+int
+crm_element_value_ll(const xmlNode *data, const char *name, long long *dest)
+{
+    const char *value = NULL;
+
+    CRM_CHECK(dest != NULL, return -1);
+    value = crm_element_value(data, name);
+    if (value) {
+        errno = 0;
         *dest = crm_int_helper(value, NULL);
-        return 0;
+        if (errno == 0) {
+            return 0;
+        }
     }
     return -1;
 }
@@ -427,6 +618,33 @@ crm_element_value_ms(const xmlNode *data, const char *name, guint *dest)
     return errno? -1 : 0;
 }
 
+/*!
+ * \brief Retrieve the seconds-since-epoch value of an XML attribute
+ *
+ * This is like \c crm_element_value() but returning the value as a time_t.
+ *
+ * \param[in]  xml    XML node to check
+ * \param[in]  name   Attribute name to check
+ * \param[out] dest   Where to store attribute value
+ *
+ * \return \c pcmk_ok on success, -1 otherwise
+ */
+int
+crm_element_value_epoch(const xmlNode *xml, const char *name, time_t *dest)
+{
+    long long value_ll = 0;
+
+    if (crm_element_value_ll(xml, name, &value_ll) < 0) {
+        return -1;
+    }
+
+    /* Unfortunately, we can't do any bounds checking, since time_t has neither
+     * standardized bounds nor constants defined for them.
+     */
+    *dest = (time_t) value_ll;
+    return pcmk_ok;
+}
+
 /*!
  * \brief Retrieve the value of XML second/microsecond attributes as time
  *
@@ -444,7 +662,6 @@ int
 crm_element_value_timeval(const xmlNode *xml, const char *name_sec,
                           const char *name_usec, struct timeval *dest)
 {
-    const char *value_s = NULL;
     long long value_i = 0;
 
     CRM_CHECK(dest != NULL, return -EINVAL);
@@ -452,29 +669,29 @@ crm_element_value_timeval(const xmlNode *xml, const char *name_sec,
     dest->tv_usec = 0;
 
     if (xml == NULL) {
-        return 0;
+        return pcmk_ok;
     }
 
+    /* Unfortunately, we can't do any bounds checking, since there are no
+     * constants provided for the bounds of time_t and suseconds_t, and
+     * calculating them isn't worth the effort. If there are XML values
+     * beyond the native sizes, there will probably be worse problems anyway.
+     */
+
     // Parse seconds
-    value_s = crm_element_value(xml, name_sec);
-    if (value_s) {
-        value_i = crm_parse_ll(value_s, NULL);
-        if (errno) {
-            return -errno;
-        }
-        dest->tv_sec = (time_t) value_i;
+    errno = 0;
+    if (crm_element_value_ll(xml, name_sec, &value_i) < 0) {
+        return -errno;
     }
+    dest->tv_sec = (time_t) value_i;
 
     // Parse microseconds
-    value_s = crm_element_value(xml, name_usec);
-    if (value_s) {
-        value_i = crm_parse_ll(value_s, NULL);
-        if (errno) {
-            return -errno;
-        }
-        dest->tv_usec = (suseconds_t) value_i;
+    if (crm_element_value_ll(xml, name_usec, &value_i) < 0) {
+        return -errno;
     }
-    return 0;
+    dest->tv_usec = (suseconds_t) value_i;
+
+    return pcmk_ok;
 }
 
 /*!
