@@ -1,5 +1,7 @@
 /*
- * Copyright 2004-2018 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2004-2019 the Pacemaker project contributors
+ *
+ * The version control history for this file may have further details.
  *
  * This source code is licensed under the GNU Lesser General Public License
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
@@ -132,13 +134,21 @@ generate_notify_key(const char *rsc_id, const char *notify_type, const char *op_
                              rsc_id, notify_type, op_type);
 }
 
-char *
-generate_transition_magic(const char *transition_key, int op_status, int op_rc)
-{
-    CRM_CHECK(transition_key != NULL, return NULL);
-    return crm_strdup_printf("%d:%d;%s", op_status, op_rc, transition_key);
-}
-
+/*!
+ * \brief Parse a transition magic string into its constituent parts
+ *
+ * \param[in]  magic          Magic string to parse (must be non-NULL)
+ * \param[out] uuid           If non-NULL, where to store copy of parsed UUID
+ * \param[out] transition_id  If non-NULL, where to store parsed transition ID
+ * \param[out] action_id      If non-NULL, where to store parsed action ID
+ * \param[out] op_status      If non-NULL, where to store parsed result status
+ * \param[out] op_rc          If non-NULL, where to store parsed actual rc
+ * \param[out] target_rc      If non-NULL, where to stored parsed target rc
+ *
+ * \return TRUE if key was valid, FALSE otherwise
+ * \note If uuid is supplied and this returns TRUE, the caller is responsible
+ *       for freeing the memory for *uuid using free().
+ */
 gboolean
 decode_transition_magic(const char *magic, char **uuid, int *transition_id, int *action_id,
                         int *op_status, int *op_rc, int *target_rc)
@@ -146,21 +156,36 @@ decode_transition_magic(const char *magic, char **uuid, int *transition_id, int
     int res = 0;
     char *key = NULL;
     gboolean result = TRUE;
+    int local_op_status = -1;
+    int local_op_rc = -1;
 
     CRM_CHECK(magic != NULL, return FALSE);
-    CRM_CHECK(op_rc != NULL, return FALSE);
-    CRM_CHECK(op_status != NULL, return FALSE);
 
-    key = calloc(1, strlen(magic) + 1);
-    res = sscanf(magic, "%d:%d;%s", op_status, op_rc, key);
-    if (res != 3) {
-        crm_warn("Only found %d items in: '%s'", res, magic);
-        free(key);
-        return FALSE;
+#ifdef SSCANF_HAS_M
+    res = sscanf(magic, "%d:%d;%ms", &local_op_status, &local_op_rc, &key);
+#else
+    key = calloc(1, strlen(magic) - 3); // magic must have >=4 other characters
+    CRM_ASSERT(key);
+    res = sscanf(magic, "%d:%d;%s", &local_op_status, &local_op_rc, key);
+#endif
+    if (res == EOF) {
+        crm_err("Could not decode transition information '%s': %s",
+                magic, pcmk_strerror(errno));
+        result = FALSE;
+    } else if (res < 3) {
+        crm_warn("Transition information '%s' incomplete (%d of 3 expected items)",
+                 magic, res);
+        result = FALSE;
+    } else {
+        if (op_status) {
+            *op_status = local_op_status;
+        }
+        if (op_rc) {
+            *op_rc = local_op_rc;
+        }
+        result = decode_transition_key(key, uuid, transition_id, action_id,
+                                       target_rc);
     }
-
-    CRM_CHECK(decode_transition_key(key, uuid, transition_id, action_id, target_rc), result = FALSE);
-
     free(key);
     return result;
 }
@@ -173,28 +198,63 @@ generate_transition_key(int transition_id, int action_id, int target_rc, const c
                              action_id, transition_id, target_rc, 36, node);
 }
 
+/*!
+ * \brief Parse a transition key into its constituent parts
+ *
+ * \param[in]  key            Transition key to parse (must be non-NULL)
+ * \param[out] uuid           If non-NULL, where to store copy of parsed UUID
+ * \param[out] transition_id  If non-NULL, where to store parsed transition ID
+ * \param[out] action_id      If non-NULL, where to store parsed action ID
+ * \param[out] target_rc      If non-NULL, where to stored parsed target rc
+ *
+ * \return TRUE if key was valid, FALSE otherwise
+ * \note If uuid is supplied and this returns TRUE, the caller is responsible
+ *       for freeing the memory for *uuid using free().
+ */
 gboolean
 decode_transition_key(const char *key, char **uuid, int *transition_id, int *action_id,
                       int *target_rc)
 {
-    CRM_CHECK(uuid != NULL, return FALSE);
-    CRM_CHECK(target_rc != NULL, return FALSE);
-    CRM_CHECK(action_id != NULL, return FALSE);
-    CRM_CHECK(transition_id != NULL, return FALSE);
+    int local_transition_id = -1;
+    int local_action_id = -1;
+    int local_target_rc = -1;
+    char local_uuid[37] = { '\0' };
 
-    *uuid = calloc(37, sizeof(char));
-    if (sscanf(key, "%d:%d:%d:%36s",
-               action_id, transition_id, target_rc, *uuid) != 4) {
-        crm_err("Invalid transition key '%s'", key);
-        free(*uuid);
+    // Initialize any supplied output arguments
+    if (uuid) {
         *uuid = NULL;
-        *target_rc = -1;
-        *action_id = -1;
+    }
+    if (transition_id) {
         *transition_id = -1;
+    }
+    if (action_id) {
+        *action_id = -1;
+    }
+    if (target_rc) {
+        *target_rc = -1;
+    }
+
+    CRM_CHECK(key != NULL, return FALSE);
+    if (sscanf(key, "%d:%d:%d:%36s", &local_action_id, &local_transition_id,
+               &local_target_rc, local_uuid) != 4) {
+        crm_err("Invalid transition key '%s'", key);
         return FALSE;
     }
-    if (strlen(*uuid) != 36) {
-        crm_warn("Invalid UUID '%s' in transition key '%s'", *uuid, key);
+    if (strlen(local_uuid) != 36) {
+        crm_warn("Invalid UUID '%s' in transition key '%s'", local_uuid, key);
+    }
+    if (uuid) {
+        *uuid = strdup(local_uuid);
+        CRM_ASSERT(*uuid);
+    }
+    if (transition_id) {
+        *transition_id = local_transition_id;
+    }
+    if (action_id) {
+        *action_id = local_action_id;
+    }
+    if (target_rc) {
+        *target_rc = local_target_rc;
     }
     return TRUE;
 }
@@ -268,54 +328,13 @@ filter_action_parameters(xmlNode * param_set, const char *version)
     free(key);
 }
 
-#define FAKE_TE_ID	"xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx"
-static void
-append_digest(lrmd_event_data_t * op, xmlNode * update, const char *version, const char *magic,
-              int level)
-{
-    /* this will enable us to later determine that the
-     *   resource's parameters have changed and we should force
-     *   a restart
-     */
-    char *digest = NULL;
-    xmlNode *args_xml = NULL;
-
-    if (op->params == NULL) {
-        return;
-    }
-
-    args_xml = create_xml_node(NULL, XML_TAG_PARAMS);
-    g_hash_table_foreach(op->params, hash2field, args_xml);
-    filter_action_parameters(args_xml, version);
-    digest = calculate_operation_digest(args_xml, version);
-
-#if 0
-    if (level < get_crm_log_level()
-        && op->interval_ms == 0 && crm_str_eq(op->op_type, CRMD_ACTION_START, TRUE)) {
-        char *digest_source = dump_xml_unformatted(args_xml);
-
-        do_crm_log(level, "Calculated digest %s for %s (%s). Source: %s\n",
-                   digest, ID(update), magic, digest_source);
-        free(digest_source);
-    }
-#endif
-    crm_xml_add(update, XML_LRM_ATTR_OP_DIGEST, digest);
-
-    free_xml(args_xml);
-    free(digest);
-}
-
 int
 rsc_op_expected_rc(lrmd_event_data_t * op)
 {
     int rc = 0;
 
     if (op && op->user_data) {
-        int dummy = 0;
-        char *uuid = NULL;
-
-        decode_transition_key(op->user_data, &uuid, &dummy, &dummy, &rc);
-        free(uuid);
+        decode_transition_key(op->user_data, NULL, NULL, NULL, &rc);
     }
     return rc;
 }
@@ -332,6 +351,8 @@ did_rsc_op_fail(lrmd_event_data_t * op, int target_rc)
         case PCMK_LRM_OP_NOTSUPPORTED:
         case PCMK_LRM_OP_TIMEOUT:
         case PCMK_LRM_OP_ERROR:
+        case PCMK_LRM_OP_NOT_CONNECTED:
+        case PCMK_LRM_OP_INVALID:
             return TRUE;
             break;
 
@@ -373,164 +394,6 @@ crm_create_op_xml(xmlNode *parent, const char *prefix, const char *task,
     return xml_op;
 }
 
-xmlNode *
-create_operation_update(xmlNode * parent, lrmd_event_data_t * op, const char * caller_version,
-                        int target_rc, const char * node, const char * origin, int level)
-{
-    char *key = NULL;
-    char *magic = NULL;
-    char *op_id = NULL;
-    char *op_id_additional = NULL;
-    char *local_user_data = NULL;
-    const char *exit_reason = NULL;
-
-    xmlNode *xml_op = NULL;
-    const char *task = NULL;
-
-    CRM_CHECK(op != NULL, return NULL);
-    do_crm_log(level, "%s: Updating resource %s after %s op %s (interval=%u)",
-               origin, op->rsc_id, op->op_type, services_lrm_status_str(op->op_status),
-               op->interval_ms);
-
-    crm_trace("DC version: %s", caller_version);
-
-    task = op->op_type;
-
-    /* Record a successful reload as a start, and a failed reload as a monitor,
-     * to make life easier for the scheduler when determining the current state.
-     */
-    if (crm_str_eq(task, "reload", TRUE)) {
-        if (op->op_status == PCMK_LRM_OP_DONE) {
-            task = CRMD_ACTION_START;
-        } else {
-            task = CRMD_ACTION_STATUS;
-        }
-    }
-
-    key = generate_op_key(op->rsc_id, task, op->interval_ms);
-    if (crm_str_eq(task, CRMD_ACTION_NOTIFY, TRUE)) {
-        const char *n_type = crm_meta_value(op->params, "notify_type");
-        const char *n_task = crm_meta_value(op->params, "notify_operation");
-
-        CRM_LOG_ASSERT(n_type != NULL);
-        CRM_LOG_ASSERT(n_task != NULL);
-        op_id = generate_notify_key(op->rsc_id, n_type, n_task);
-
-        if (op->op_status != PCMK_LRM_OP_PENDING) {
-            /* Ignore notify errors.
-             *
-             * @TODO It might be better to keep the correct result here, and
-             * ignore it in process_graph_event().
-             */
-            op->op_status = PCMK_LRM_OP_DONE;
-            op->rc = 0;
-        }
-
-    } else if (did_rsc_op_fail(op, target_rc)) {
-        op_id = generate_op_key(op->rsc_id, "last_failure", 0);
-        if (op->interval_ms == 0) {
-            // Ensure 'last' gets updated, in case record-pending is true
-            op_id_additional = generate_op_key(op->rsc_id, "last", 0);
-        }
-        exit_reason = op->exit_reason;
-
-    } else if (op->interval_ms > 0) {
-        op_id = strdup(key);
-
-    } else {
-        op_id = generate_op_key(op->rsc_id, "last", 0);
-    }
-
-  again:
-    xml_op = find_entity(parent, XML_LRM_TAG_RSC_OP, op_id);
-    if (xml_op == NULL) {
-        xml_op = create_xml_node(parent, XML_LRM_TAG_RSC_OP);
-    }
-
-    if (op->user_data == NULL) {
-        crm_debug("Generating fake transition key for: " CRM_OP_FMT " %d from %s",
-                  op->rsc_id, op->op_type, op->interval_ms,
-                  op->call_id, origin);
-        local_user_data = generate_transition_key(-1, op->call_id, target_rc, FAKE_TE_ID);
-        op->user_data = local_user_data;
-    }
-
-    if(magic == NULL) {
-        magic = generate_transition_magic(op->user_data, op->op_status, op->rc);
-    }
-
-    crm_xml_add(xml_op, XML_ATTR_ID, op_id);
-    crm_xml_add(xml_op, XML_LRM_ATTR_TASK_KEY, key);
-    crm_xml_add(xml_op, XML_LRM_ATTR_TASK, task);
-    crm_xml_add(xml_op, XML_ATTR_ORIGIN, origin);
-    crm_xml_add(xml_op, XML_ATTR_CRM_VERSION, caller_version);
-    crm_xml_add(xml_op, XML_ATTR_TRANSITION_KEY, op->user_data);
-    crm_xml_add(xml_op, XML_ATTR_TRANSITION_MAGIC, magic);
-    crm_xml_add(xml_op, XML_LRM_ATTR_EXIT_REASON, exit_reason == NULL ? "" : exit_reason);
-    crm_xml_add(xml_op, XML_LRM_ATTR_TARGET, node); /* For context during triage */
-
-    crm_xml_add_int(xml_op, XML_LRM_ATTR_CALLID, op->call_id);
-    crm_xml_add_int(xml_op, XML_LRM_ATTR_RC, op->rc);
-    crm_xml_add_int(xml_op, XML_LRM_ATTR_OPSTATUS, op->op_status);
-    crm_xml_add_ms(xml_op, XML_LRM_ATTR_INTERVAL_MS, op->interval_ms);
-
-    if (compare_version("2.1", caller_version) <= 0) {
-        if (op->t_run || op->t_rcchange || op->exec_time || op->queue_time) {
-            crm_trace("Timing data (" CRM_OP_FMT "): last=%u change=%u exec=%u queue=%u",
-                      op->rsc_id, op->op_type, op->interval_ms,
-                      op->t_run, op->t_rcchange, op->exec_time, op->queue_time);
-
-            if (op->interval_ms == 0) {
-                /* The values are the same for non-recurring ops */
-                crm_xml_add_int(xml_op, XML_RSC_OP_LAST_RUN, op->t_run);
-                crm_xml_add_int(xml_op, XML_RSC_OP_LAST_CHANGE, op->t_run);
-
-            } else if(op->t_rcchange) {
-                /* last-run is not accurate for recurring ops */
-                crm_xml_add_int(xml_op, XML_RSC_OP_LAST_CHANGE, op->t_rcchange);
-
-            } else {
-                /* ...but is better than nothing otherwise */
-                crm_xml_add_int(xml_op, XML_RSC_OP_LAST_CHANGE, op->t_run);
-            }
-
-            crm_xml_add_int(xml_op, XML_RSC_OP_T_EXEC, op->exec_time);
-            crm_xml_add_int(xml_op, XML_RSC_OP_T_QUEUE, op->queue_time);
-        }
-    }
-
-    if (crm_str_eq(op->op_type, CRMD_ACTION_MIGRATE, TRUE)
-        || crm_str_eq(op->op_type, CRMD_ACTION_MIGRATED, TRUE)) {
-        /*
-         * Record migrate_source and migrate_target always for migrate ops.
-         */
-        const char *name = XML_LRM_ATTR_MIGRATE_SOURCE;
-
-        crm_xml_add(xml_op, name, crm_meta_value(op->params, name));
-
-        name = XML_LRM_ATTR_MIGRATE_TARGET;
-        crm_xml_add(xml_op, name, crm_meta_value(op->params, name));
-    }
-
-    append_digest(op, xml_op, caller_version, magic, LOG_DEBUG);
-
-    if (op_id_additional) {
-        free(op_id);
-        op_id = op_id_additional;
-        op_id_additional = NULL;
-        goto again;
-    }
-
-    if (local_user_data) {
-        free(local_user_data);
-        op->user_data = NULL;
-    }
-    free(magic);
-    free(op_id);
-    free(key);
-    return xml_op;
-}
-
 /*!
  * \brief Check whether an operation requires resource agent meta-data
  *
