@@ -1,21 +1,10 @@
 /*
- * Copyright (c) 2008 Andrew Beekhof
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- * 
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * Copyright 2008-2018 Andrew Beekhof <andrew@beekhof.net>
  *
+ * This source code is licensed under the GNU Lesser General Public License
+ * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
+
 #include <crm_internal.h>
 #include <crm/crm.h>
 
@@ -30,11 +19,11 @@
 #include <netinet/ip.h>
 #include <netinet/tcp.h>
 #include <netdb.h>
-
 #include <stdlib.h>
 #include <errno.h>
-#include <glib.h>
+#include <inttypes.h>  /* X32T ~ PRIx32 */
 
+#include <glib.h>
 #include <bzlib.h>
 
 #include <crm/common/ipcs.h>
@@ -119,7 +108,8 @@ crm_remote_header(crm_remote_t * remote)
 
         CRM_LOG_ASSERT(endian == ENDIAN_LOCAL);
         if(endian != ENDIAN_LOCAL) {
-            crm_err("Invalid message detected, endian mismatch: %lx is neither %lx nor the swab'd %lx",
+            crm_err("Invalid message detected, endian mismatch: %" X32T
+                    " is neither %" X32T " nor the swab'd %" X32T,
                     ENDIAN_LOCAL, header->endian, endian);
             return NULL;
         }
@@ -236,13 +226,10 @@ pcmk__new_tls_session(int csock, unsigned int conn_type,
                       gnutls_credentials_type_t cred_type, void *credentials)
 {
     int rc = GNUTLS_E_SUCCESS;
-#  ifdef HAVE_GNUTLS_PRIORITY_SET_DIRECT
     const char *prio_base = NULL;
     char *prio = NULL;
-#  endif
     gnutls_session_t *session = NULL;
 
-#  ifdef HAVE_GNUTLS_PRIORITY_SET_DIRECT
     /* Determine list of acceptable ciphers, etc. Pacemaker always adds the
      * values required for its functionality.
      *
@@ -256,7 +243,6 @@ pcmk__new_tls_session(int csock, unsigned int conn_type,
     }
     prio = crm_strdup_printf("%s:%s", prio_base,
                              (cred_type == GNUTLS_CRD_ANON)? "+ANON-DH" : "+DHE-PSK:+PSK");
-#  endif
 
     session = gnutls_malloc(sizeof(gnutls_session_t));
     if (session == NULL) {
@@ -269,7 +255,6 @@ pcmk__new_tls_session(int csock, unsigned int conn_type,
         goto error;
     }
 
-#  ifdef HAVE_GNUTLS_PRIORITY_SET_DIRECT
     /* @TODO On the server side, it would be more efficient to cache the
      * priority with gnutls_priority_init2() and set it with
      * gnutls_priority_set() for all sessions.
@@ -281,10 +266,6 @@ pcmk__new_tls_session(int csock, unsigned int conn_type,
     if (conn_type == GNUTLS_CLIENT) {
         pcmk__set_minimum_dh_bits(session);
     }
-#  else
-    gnutls_set_default_priority(*session);
-    gnutls_kx_set_priority(*session, (cred_type == GNUTLS_CRD_ANON)? anon_tls_kx_order : psk_tls_kx_order);
-#  endif
 
     gnutls_transport_set_ptr(*session,
                              (gnutls_transport_ptr_t) GINT_TO_POINTER(csock));
@@ -293,28 +274,16 @@ pcmk__new_tls_session(int csock, unsigned int conn_type,
     if (rc != GNUTLS_E_SUCCESS) {
         goto error;
     }
-#  ifdef HAVE_GNUTLS_PRIORITY_SET_DIRECT
     free(prio);
-#  endif
     return session;
 
 error:
-    {
-#  ifdef HAVE_GNUTLS_PRIORITY_SET_DIRECT
-        const char *prio_s = prio;
-#  else
-        const char *prio_s = "default";
-#  endif
-
-        crm_err("Could not initialize %s TLS %s session: %s "
-                CRM_XS " rc=%d priority='%s'",
-                (cred_type == GNUTLS_CRD_ANON)? "anonymous" : "PSK",
-                (conn_type == GNUTLS_SERVER)? "server" : "client",
-                gnutls_strerror(rc), rc, prio_s);
-    }
-#  ifdef HAVE_GNUTLS_PRIORITY_SET_DIRECT
+    crm_err("Could not initialize %s TLS %s session: %s "
+            CRM_XS " rc=%d priority='%s'",
+            (cred_type == GNUTLS_CRD_ANON)? "anonymous" : "PSK",
+            (conn_type == GNUTLS_SERVER)? "server" : "client",
+            gnutls_strerror(rc), rc, prio);
     free(prio);
-#  endif
     if (session != NULL) {
         gnutls_free(session);
     }
@@ -431,8 +400,9 @@ crm_send_tls(gnutls_session_t * session, const char *buf, size_t len)
                       (unsigned long long) len);
 
         } else if (rc < 0) {
-            crm_err("Connection terminated: %s " CRM_XS " rc=%d",
-                    gnutls_strerror(rc), rc);
+            // Caller can log as error if necessary
+            crm_info("TLS connection terminated: %s " CRM_XS " rc=%d",
+                     gnutls_strerror(rc), rc);
             rc = -ECONNABORTED;
             break;
 
@@ -475,7 +445,9 @@ crm_send_plaintext(int sock, const char *buf, size_t len)
                 crm_trace("Retry");
                 goto retry;
             default:
-                crm_perror(LOG_ERR, "Could only write %d of the remaining %d bytes", rc, (int)len);
+                crm_perror(LOG_INFO,
+                           "Could only write %d of the remaining %llu bytes",
+                           rc, (unsigned long long) len);
                 break;
         }
 
@@ -497,22 +469,19 @@ crm_send_plaintext(int sock, const char *buf, size_t len)
 static int
 crm_remote_sendv(crm_remote_t * remote, struct iovec * iov, int iovs)
 {
-    int lpc = 0;
-    int rc = -ESOCKTNOSUPPORT;
-
-    for(; lpc < iovs; lpc++) {
+    int rc = 0;
 
+    for (int lpc = 0; (lpc < iovs) && (rc >= 0); lpc++) {
 #ifdef HAVE_GNUTLS_GNUTLS_H
         if (remote->tls_session) {
             rc = crm_send_tls(remote->tls_session, iov[lpc].iov_base, iov[lpc].iov_len);
-        } else if (remote->tcp_socket) {
-#else
-        if (remote->tcp_socket) {
+            continue;
+        }
 #endif
+        if (remote->tcp_socket) {
             rc = crm_send_plaintext(remote->tcp_socket, iov[lpc].iov_base, iov[lpc].iov_len);
-
         } else {
-            crm_err("Unsupported connection type");
+            rc = -ESOCKTNOSUPPORT;
         }
     }
     return rc;
@@ -597,7 +566,8 @@ crm_remote_parse_buffer(crm_remote_t * remote)
             return NULL;
 
         } else if (rc != BZ_OK) {
-            crm_err("Decompression failed: %s (%d)", bz2_strerror(rc), rc);
+            crm_err("Decompression failed: %s " CRM_XS " bzerror=%d",
+                    bz2_strerror(rc), rc);
             free(uncompressed);
             return NULL;
         }
