@@ -1,19 +1,10 @@
 /*
- * Copyright (C) 2004-2016 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2004-2019 the Pacemaker project contributors
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * The version control history for this file may have further details.
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * This source code is licensed under the GNU Lesser General Public License
+ * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
  */
 
 #include <crm_internal.h>
@@ -25,13 +16,12 @@
 #include <sys/stat.h>
 #include <stdarg.h>
 
-#if HAVE_LIBXML2
-#  include <libxml/relaxng.h>
-#endif
+#include <libxml/relaxng.h>
 
 #if HAVE_LIBXSLT
 #  include <libxslt/xslt.h>
 #  include <libxslt/transform.h>
+#  include <libxslt/security.h>
 #  include <libxslt/xsltutils.h>
 #endif
 
@@ -59,22 +49,23 @@ typedef struct {
 
 enum schema_validator_e {
     schema_validator_none,
-    schema_validator_dtd,
     schema_validator_rng
 };
 
 struct schema_s {
     char *name;
-    char *location;
     char *transform;
     void *cache;
     enum schema_validator_e validator;
     int after_transform;
     schema_version_t version;
+    char *transform_enter;
+    bool transform_onleave;
 };
 
 static struct schema_s *known_schemas = NULL;
 static int xml_schema_max = 0;
+static bool silent_logging = FALSE;
 
 static void
 xml_log(int priority, const char *fmt, ...)
@@ -86,15 +77,17 @@ xml_log(int priority, const char *fmt, ...)
     va_list ap;
 
     va_start(ap, fmt);
-    /* XXX should not this enable dechunking as well? */
-    CRM_XML_LOG_BASE(priority, FALSE, 0, NULL, fmt, ap);
+    if (silent_logging == FALSE) {
+        /* XXX should not this enable dechunking as well? */
+        CRM_XML_LOG_BASE(priority, FALSE, 0, NULL, fmt, ap);
+    }
     va_end(ap);
 }
 
 static int
 xml_latest_schema_index(void)
 {
-    return xml_schema_max - 4;
+    return xml_schema_max - 3; // index from 0, ignore "pacemaker-next"/"none"
 }
 
 static int
@@ -124,31 +117,6 @@ xml_latest_schema(void)
     return get_schema_name(xml_latest_schema_index());
 }
 
-static const char *
-get_schema_root(void)
-{
-    static const char *base = NULL;
-
-    if (base == NULL) {
-        base = getenv("PCMK_schema_directory");
-    }
-    if (base == NULL || strlen(base) == 0) {
-        base = CRM_DTD_DIRECTORY;
-    }
-    return base;
-}
-
-static char *
-get_schema_path(const char *name, const char *file)
-{
-    const char *base = get_schema_root();
-
-    if (file) {
-        return crm_strdup_printf("%s/%s", base, file);
-    }
-    return crm_strdup_printf("%s/%s.rng", base, name);
-}
-
 static inline bool
 version_from_filename(const char *filename, schema_version_t *version)
 {
@@ -172,10 +140,6 @@ schema_filter(const struct dirent *a)
     } else if (!version_from_filename(a->d_name, &version)) {
         /* crm_trace("%s - wrong format", a->d_name); */
 
-    } else if (strcmp(a->d_name, "pacemaker-1.1.rng") == 0) {
-        /* "-1.1" was used for what later became "-next" */
-        /* crm_trace("%s - hack", a->d_name); */
-
     } else {
         /* crm_debug("%s - candidate", a->d_name); */
         rc = 1;
@@ -206,9 +170,17 @@ schema_sort(const struct dirent **a, const struct dirent **b)
     return 0;
 }
 
+/*!
+ * \internal
+ * \brief Add given schema + auxiliary data to internal bookkeeping.
+ *
+ * \note When providing \p version, should not be called directly but
+ *       through \c add_schema_by_version.
+ */
 static void
 add_schema(enum schema_validator_e validator, const schema_version_t *version,
-           const char *name, const char *location, const char *transform,
+           const char *name, const char *transform,
+           const char *transform_enter, bool transform_onleave,
            int after_transform)
 {
     int last = xml_schema_max;
@@ -230,164 +202,235 @@ add_schema(enum schema_validator_e validator, const schema_version_t *version,
     }
     if (have_version) {
         known_schemas[last].name = schema_strdup_printf("pacemaker-", *version, "");
-        known_schemas[last].location = crm_strdup_printf("%s.rng",
-                                                         known_schemas[last].name);
     } else {
         CRM_ASSERT(name);
-        CRM_ASSERT(location);
         schema_scanf(name, "%*[^-]-", known_schemas[last].version, "");
         known_schemas[last].name = strdup(name);
-        known_schemas[last].location = strdup(location);
     }
 
     if (transform) {
         known_schemas[last].transform = strdup(transform);
     }
+    if (transform_enter) {
+        known_schemas[last].transform_enter = strdup(transform_enter);
+    }
+    known_schemas[last].transform_onleave = transform_onleave;
     if (after_transform == 0) {
         after_transform = xml_schema_max;  /* upgrade is a one-way */
     }
     known_schemas[last].after_transform = after_transform;
 
     if (known_schemas[last].after_transform < 0) {
-        crm_debug("Added supported schema %d: %s (%s)",
-                  last, known_schemas[last].name, known_schemas[last].location);
+        crm_debug("Added supported schema %d: %s",
+                  last, known_schemas[last].name);
 
     } else if (known_schemas[last].transform) {
-        crm_debug("Added supported schema %d: %s (%s upgrades to %d with %s)",
-                  last, known_schemas[last].name, known_schemas[last].location,
+        crm_debug("Added supported schema %d: %s (upgrades to %d with %s.xsl)",
+                  last, known_schemas[last].name,
                   known_schemas[last].after_transform,
                   known_schemas[last].transform);
 
     } else {
-        crm_debug("Added supported schema %d: %s (%s upgrades to %d)",
-                  last, known_schemas[last].name, known_schemas[last].location,
+        crm_debug("Added supported schema %d: %s (upgrades to %d)",
+                  last, known_schemas[last].name,
                   known_schemas[last].after_transform);
     }
 }
 
+/*!
+ * \internal
+ * \brief Add version-specified schema + auxiliary data to internal bookkeeping.
+ * \return \c -ENOENT when no upgrade schema associated, \c pcmk_ok otherwise.
+ *
+ * \note There's no reliance on the particular order of schemas entering here.
+ *
+ * \par A bit of theory
+ * We track 3 XSLT stylesheets that differ per usage:
+ * - "upgrade":
+ *   . sparsely spread over the sequence of all available schemas,
+ *     as they are only relevant when major version of the schema
+ *     is getting bumped -- in that case, it MUST be set
+ *   . name convention:  upgrade-X.Y.xsl
+ * - "upgrade-enter":
+ *   . may only accompany "upgrade" occurrence, but doesn't need to
+ *     be present anytime such one is, i.e., it MAY not be set when
+ *     "upgrade" is
+ *   . name convention:  upgrade-X.Y-enter.xsl,
+ *     when not present: upgrade-enter.xsl
+ * - "upgrade-leave":
+ *   . like "upgrade-enter", but SHOULD be present whenever
+ *     "upgrade-enter" is (and vice versa, but that's only
+ *     to prevent confusion based on observing the files,
+ *     it would get ignored regardless)
+ *   . name convention:  (see "upgrade-enter")
+ */
+static int
+add_schema_by_version(const schema_version_t *version, int next,
+                      bool transform_expected)
+{
+    bool transform_onleave = FALSE;
+    int rc = pcmk_ok;
+    struct stat s;
+    char *xslt = NULL,
+         *transform_upgrade = NULL,
+         *transform_enter = NULL;
+
+    /* prologue for further transform_expected handling */
+    if (transform_expected) {
+        /* check if there's suitable "upgrade" stylesheet */
+        transform_upgrade = schema_strdup_printf("upgrade-", *version, );
+        xslt = pcmk__xml_artefact_path(pcmk__xml_artefact_ns_legacy_xslt,
+                                       transform_upgrade);
+    }
+
+    if (!transform_expected) {
+        /* jump directly to the end */
+
+    } else if (stat(xslt, &s) == 0) {
+        /* perhaps there's also a targeted "upgrade-enter" stylesheet */
+        transform_enter = schema_strdup_printf("upgrade-", *version, "-enter");
+        free(xslt);
+        xslt = pcmk__xml_artefact_path(pcmk__xml_artefact_ns_legacy_xslt,
+                                       transform_enter);
+        if (stat(xslt, &s) != 0) {
+            /* or initially, at least a generic one */
+            crm_debug("Upgrade-enter transform %s.xsl not found", xslt);
+            free(xslt);
+            free(transform_enter);
+            transform_enter = strdup("upgrade-enter");
+            xslt = pcmk__xml_artefact_path(pcmk__xml_artefact_ns_legacy_xslt,
+                                           transform_enter);
+            if (stat(xslt, &s) != 0) {
+                crm_debug("Upgrade-enter transform %s.xsl not found, either", xslt);
+                free(xslt);
+                xslt = NULL;
+            }
+        }
+        /* xslt contains full path to "upgrade-enter" stylesheet */
+        if (xslt != NULL) {
+            /* then there should be "upgrade-leave" counterpart (enter->leave) */
+            memcpy(strrchr(xslt, '-') + 1, "leave", sizeof("leave") - 1);
+            transform_onleave = (stat(xslt, &s) == 0);
+            free(xslt);
+        } else {
+            free(transform_enter);
+            transform_enter = NULL;
+        }
+
+    } else {
+        crm_err("Upgrade transform %s not found", xslt);
+        free(xslt);
+        free(transform_upgrade);
+        transform_upgrade = NULL;
+        next = -1;
+        rc = -ENOENT;
+    }
+
+    add_schema(schema_validator_rng, version, NULL,
+               transform_upgrade, transform_enter, transform_onleave, next);
+
+    free(transform_upgrade);
+    free(transform_enter);
+
+    return rc;
+}
+
+static int
+wrap_libxslt(bool finalize)
+{
+    static xsltSecurityPrefsPtr secprefs;
+    int ret = 0;
+
+    /* security framework preferences */
+    if (!finalize) {
+        CRM_ASSERT(secprefs == NULL);
+        secprefs = xsltNewSecurityPrefs();
+        ret = xsltSetSecurityPrefs(secprefs, XSLT_SECPREF_WRITE_FILE,
+                                   xsltSecurityForbid)
+              | xsltSetSecurityPrefs(secprefs, XSLT_SECPREF_CREATE_DIRECTORY,
+                                     xsltSecurityForbid)
+              | xsltSetSecurityPrefs(secprefs, XSLT_SECPREF_READ_NETWORK,
+                                     xsltSecurityForbid)
+              | xsltSetSecurityPrefs(secprefs, XSLT_SECPREF_WRITE_NETWORK,
+                                     xsltSecurityForbid);
+        if (ret != 0) {
+            return -1;
+        }
+    } else {
+        xsltFreeSecurityPrefs(secprefs);
+        secprefs = NULL;
+    }
+
+    /* cleanup only */
+    if (finalize) {
+        xsltCleanupGlobals();
+    }
+
+    return ret;
+}
+
 /*!
  * \internal
  * \brief Load pacemaker schemas into cache
+ *
+ * \note This currently also serves as an entry point for the
+ *       generic initialization of the libxslt library.
  */
 void
 crm_schema_init(void)
 {
     int lpc, max;
-    const char *base = get_schema_root();
+    char *base = pcmk__xml_artefact_root(pcmk__xml_artefact_ns_legacy_rng);
     struct dirent **namelist = NULL;
     const schema_version_t zero = SCHEMA_ZERO;
 
+    wrap_libxslt(false);
+
     max = scandir(base, &namelist, schema_filter, schema_sort);
-
-    add_schema(schema_validator_dtd, &zero, "pacemaker-0.6",
-               "crm.dtd", "upgrade06.xsl", 3);
-
-    add_schema(schema_validator_dtd, &zero, "transitional-0.6",
-               "crm-transitional.dtd", "upgrade06.xsl", 3);
-
-    add_schema(schema_validator_rng, &zero, "pacemaker-0.7",
-               "pacemaker-1.0.rng", NULL, 0);
-
     if (max < 0) {
         crm_notice("scandir(%s) failed: %s (%d)", base, strerror(errno), errno);
+        free(base);
 
     } else {
+        free(base);
         for (lpc = 0; lpc < max; lpc++) {
+            bool transform_expected = FALSE;
             int next = 0;
             schema_version_t version = SCHEMA_ZERO;
-            char *transform = NULL;
 
             if (!version_from_filename(namelist[lpc]->d_name, &version)) {
                 // Shouldn't be possible, but makes static analysis happy
                 crm_err("Skipping schema '%s': could not parse version",
                         namelist[lpc]->d_name);
-                free(namelist[lpc]);
                 continue;
             }
             if ((lpc + 1) < max) {
                 schema_version_t next_version = SCHEMA_ZERO;
 
                 if (version_from_filename(namelist[lpc+1]->d_name, &next_version)
-                    && (version.v[0] < next_version.v[0])) {
-
-                    struct stat s;
-                    char *xslt = NULL;
-
-                    transform = schema_strdup_printf("upgrade-", version, ".xsl");
-                    xslt = get_schema_path(NULL, transform);
-                    if (stat(xslt, &s) != 0) {
-                        crm_err("Transform %s not found", xslt);
-                        free(xslt);
-                        add_schema(schema_validator_rng, &version, NULL, NULL,
-                                   NULL, -1);
-                        break;
-                    } else {
-                        free(xslt);
-                    }
+                        && (version.v[0] < next_version.v[0])) {
+                    transform_expected = TRUE;
                 }
 
             } else {
                 next = -1;
             }
-            add_schema(schema_validator_rng, &version, NULL, NULL, transform,
-                       next);
-            free(namelist[lpc]);
-            free(transform);
+            if (add_schema_by_version(&version, next, transform_expected)
+                    == -ENOENT) {
+                break;
+            }
         }
-    }
 
-    /* 1.1 was the old name for -next */
-    add_schema(schema_validator_rng, &zero, "pacemaker-1.1",
-               "pacemaker-next.rng", NULL, 0);
+        for (lpc = 0; lpc < max; lpc++) {
+            free(namelist[lpc]);
+        }
+        free(namelist);
+    }
 
     add_schema(schema_validator_rng, &zero, "pacemaker-next",
-               "pacemaker-next.rng", NULL, -1);
+               NULL, NULL, FALSE, -1);
 
-    add_schema(schema_validator_none, &zero, "none",
-               "N/A", NULL, -1);
-    free(namelist);
-}
-
-static gboolean
-validate_with_dtd(xmlDocPtr doc, gboolean to_logs, const char *dtd_file)
-{
-    gboolean valid = TRUE;
-
-    xmlDtdPtr dtd = NULL;
-    xmlValidCtxtPtr cvp = NULL;
-
-    CRM_CHECK(doc != NULL, return FALSE);
-    CRM_CHECK(dtd_file != NULL, return FALSE);
-
-    dtd = xmlParseDTD(NULL, (const xmlChar *)dtd_file);
-    if (dtd == NULL) {
-        crm_err("Could not locate/parse DTD: %s", dtd_file);
-        return TRUE;
-    }
-
-    cvp = xmlNewValidCtxt();
-    if (cvp) {
-        if (to_logs) {
-            cvp->userData = (void *)LOG_ERR;
-            cvp->error = (xmlValidityErrorFunc) xml_log;
-            cvp->warning = (xmlValidityWarningFunc) xml_log;
-        } else {
-            cvp->userData = (void *)stderr;
-            cvp->error = (xmlValidityErrorFunc) fprintf;
-            cvp->warning = (xmlValidityWarningFunc) fprintf;
-        }
-
-        if (!xmlValidateDtd(cvp, doc, dtd)) {
-            valid = FALSE;
-        }
-        xmlFreeValidCtxt(cvp);
-
-    } else {
-        crm_err("Internal error: No valid context");
-    }
-
-    xmlFreeDtd(dtd);
-    return valid;
+    add_schema(schema_validator_none, &zero, "none", NULL, NULL, FALSE, -1);
 }
 
 #if 0
@@ -431,7 +474,7 @@ validate_with_relaxng(xmlDocPtr doc, gboolean to_logs, const char *relaxng_file,
         ctx = *cached_ctx;
 
     } else {
-        crm_info("Creating RNG parser context");
+        crm_debug("Creating RNG parser context");
         ctx = calloc(1, sizeof(relaxng_ctx_cache_t));
 
         xmlLoadExtDtdDefaultValue = 1;
@@ -517,8 +560,7 @@ crm_schema_cleanup(void)
     for (lpc = 0; lpc < xml_schema_max; lpc++) {
 
         switch (known_schemas[lpc].validator) {
-            case schema_validator_none:
-            case schema_validator_dtd: // not cached
+            case schema_validator_none: // not cached
                 break;
             case schema_validator_rng: // cached
                 ctx = (relaxng_ctx_cache_t *) known_schemas[lpc].cache;
@@ -539,11 +581,13 @@ crm_schema_cleanup(void)
                 break;
         }
         free(known_schemas[lpc].name);
-        free(known_schemas[lpc].location);
         free(known_schemas[lpc].transform);
+        free(known_schemas[lpc].transform_enter);
     }
     free(known_schemas);
     known_schemas = NULL;
+
+    wrap_libxslt(true);
 }
 
 static gboolean
@@ -563,15 +607,12 @@ validate_with(xmlNode *xml, int method, gboolean to_logs)
 
     CRM_CHECK(xml != NULL, return FALSE);
     doc = getDocPtr(xml);
-    file = get_schema_path(known_schemas[method].name,
-                           known_schemas[method].location);
+    file = pcmk__xml_artefact_path(pcmk__xml_artefact_ns_legacy_rng,
+                                   known_schemas[method].name);
 
     crm_trace("Validating with: %s (type=%d)",
               crm_str(file), known_schemas[method].validator);
     switch (known_schemas[method].validator) {
-        case schema_validator_dtd:
-            valid = validate_with_dtd(doc, to_logs, file);
-            break;
         case schema_validator_rng:
             valid =
                 validate_with_relaxng(doc, to_logs, file,
@@ -587,6 +628,16 @@ validate_with(xmlNode *xml, int method, gboolean to_logs)
     return valid;
 }
 
+static bool
+validate_with_silent(xmlNode *xml, int method)
+{
+    bool rc, sl_backup = silent_logging;
+    silent_logging = TRUE;
+    rc = validate_with(xml, method, TRUE);
+    silent_logging = sl_backup;
+    return rc;
+}
+
 static void
 dump_file(const char *filename)
 {
@@ -625,9 +676,9 @@ validate_xml_verbose(xmlNode *xml_blob)
     xmlDoc *doc = NULL;
     xmlNode *xml = NULL;
     gboolean rc = FALSE;
-    char *filename = strdup(CRM_STATE_DIR "/cib-invalid.XXXXXX");
+    char *filename = NULL;
 
-    CRM_CHECK(filename != NULL, return FALSE);
+    filename = crm_strdup_printf("%s/cib-invalid.XXXXXX", crm_get_tmpdir());
 
     umask(S_IWGRP | S_IWOTH | S_IROTH);
     fd = mkstemp(filename);
@@ -659,14 +710,6 @@ validate_xml(xmlNode *xml_blob, const char *validation, gboolean to_logs)
         int lpc = 0;
         bool valid = FALSE;
 
-        /* @COMPAT pre-1.0 configs */
-        validation = crm_element_value(xml_blob, "ignore_dtd");
-        if (crm_is_true(validation)) {
-            crm_xml_add(xml_blob, XML_ATTR_VALIDATION, "none");
-            return TRUE;
-        }
-
-        /* Work it out */
         for (lpc = 0; lpc < xml_schema_max; lpc++) {
             if (validate_with(xml_blob, lpc, FALSE)) {
                 valid = TRUE;
@@ -699,16 +742,163 @@ static void
 cib_upgrade_err(void *ctx, const char *fmt, ...)
 G_GNUC_PRINTF(2, 3);
 
+/* With this arrangement, an attempt to identify the message severity
+   as explicitly signalled directly from XSLT is performed in rather
+   a smart way (no reliance on formatting string + arguments being
+   always specified as ["%s", purposeful_string], as it can also be
+   ["%s: %s", some_prefix, purposeful_string] etc. so every argument
+   pertaining %s specifier is investigated), and if such a mark found,
+   the respective level is determined and, when the messages are to go
+   to the native logs, the mark itself gets dropped
+   (by the means of string shift).
+
+   NOTE: whether the native logging is the right sink is decided per
+         the ctx parameter -- NULL denotes this case, otherwise it
+         carries a pointer to the numeric expression of the desired
+         target logging level (messages with higher level will be
+         suppressed)
+
+   NOTE: on some architectures, this string shift may not have any
+         effect, but that's an acceptable tradeoff
+
+   The logging level for not explicitly designated messages
+   (suspicious, likely internal errors or some runaways) is
+   LOG_WARNING.
+ */
 static void
 cib_upgrade_err(void *ctx, const char *fmt, ...)
 {
-    va_list ap;
+    va_list ap, aq;
+    char *arg_cur;
+
+    bool found = FALSE;
+    const char *fmt_iter = fmt;
+    uint8_t msg_log_level = LOG_WARNING;  /* default for runaway messages */
+    const unsigned * log_level = (const unsigned *) ctx;
+    enum {
+        escan_seennothing,
+        escan_seenpercent,
+    } scan_state = escan_seennothing;
 
     va_start(ap, fmt);
-    CRM_XML_LOG_BASE(LOG_WARNING, TRUE, 0, "CIB upgrade: ", fmt, ap);
+    va_copy(aq, ap);
+
+    while (!found && *fmt_iter != '\0') {
+        /* while casing schema borrowed from libqb:qb_vsnprintf_serialize */
+        switch (*fmt_iter++) {
+        case '%':
+            if (scan_state == escan_seennothing) {
+                scan_state = escan_seenpercent;
+            } else if (scan_state == escan_seenpercent) {
+                scan_state = escan_seennothing;
+            }
+            break;
+        case 's':
+            if (scan_state == escan_seenpercent) {
+                scan_state = escan_seennothing;
+                arg_cur = va_arg(aq, char *);
+                if (arg_cur != NULL) {
+                    switch (arg_cur[0]) {
+                    case 'W':
+                        if (!strncmp(arg_cur, "WARNING: ",
+                                     sizeof("WARNING: ") - 1)) {
+                            msg_log_level = LOG_WARNING;
+                        }
+                        if (ctx == NULL) {
+                            memmove(arg_cur, arg_cur + sizeof("WARNING: ") - 1,
+                                    strlen(arg_cur + sizeof("WARNING: ") - 1) + 1);
+                        }
+                        found = TRUE;
+                        break;
+                    case 'I':
+                        if (!strncmp(arg_cur, "INFO: ",
+                                     sizeof("INFO: ") - 1)) {
+                            msg_log_level = LOG_INFO;
+                        }
+                        if (ctx == NULL) {
+                            memmove(arg_cur, arg_cur + sizeof("INFO: ") - 1,
+                                    strlen(arg_cur + sizeof("INFO: ") - 1) + 1);
+                        }
+                        found = TRUE;
+                        break;
+                    case 'D':
+                        if (!strncmp(arg_cur, "DEBUG: ",
+                                     sizeof("DEBUG: ") - 1)) {
+                            msg_log_level = LOG_DEBUG;
+                        }
+                        if (ctx == NULL) {
+                            memmove(arg_cur, arg_cur + sizeof("DEBUG: ") - 1,
+                                    strlen(arg_cur + sizeof("DEBUG: ") - 1) + 1);
+                        }
+                        found = TRUE;
+                        break;
+                    }
+                }
+            }
+            break;
+        case '#': case '-': case ' ': case '+': case '\'': case 'I': case '.':
+        case '0': case '1': case '2': case '3': case '4':
+        case '5': case '6': case '7': case '8': case '9':
+        case '*':
+            break;
+        case 'l':
+        case 'z':
+        case 't':
+        case 'j':
+        case 'd': case 'i':
+        case 'o':
+        case 'u':
+        case 'x': case 'X':
+        case 'e': case 'E':
+        case 'f': case 'F':
+        case 'g': case 'G':
+        case 'a': case 'A':
+        case 'c':
+        case 'p':
+            if (scan_state == escan_seenpercent) {
+                (void) va_arg(aq, void *);  /* skip forward */
+                scan_state = escan_seennothing;
+            }
+            break;
+        default:
+            scan_state = escan_seennothing;
+            break;
+        }
+    }
+
+    if (log_level != NULL) {
+        /* intention of the following offset is:
+           cibadmin -V -> start showing INFO labelled messages */
+        if (*log_level + 4 >= msg_log_level) {
+            vfprintf(stderr, fmt, ap);
+        }
+    } else {
+        CRM_XML_LOG_BASE(msg_log_level, TRUE, 0, "CIB upgrade: ", fmt, ap);
+    }
+
+    va_end(aq);
     va_end(ap);
 }
 
+
+/* Denotes temporary emergency fix for "xmldiff'ing not text-node-ready";
+   proper fix is most likely to teach __xml_diff_object and friends to
+   deal with XML_TEXT_NODE (and more?), i.e., those nodes currently
+   missing "_private" field (implicitly as NULL) which clashes with
+   unchecked accesses (e.g. in __xml_offset) -- the outcome may be that
+   those unexpected XML nodes will simply be ignored for the purpose of
+   diff'ing, or it may be made more robust, or per the user's preference
+   (which then may be exposed as crm_diff switch).
+
+   Said XML_TEXT_NODE may appear unexpectedly due to how upgrade-2.10.xsl
+   is arranged.
+
+   The emergency fix is simple: reparse XSLT output with blank-ignoring
+   parser. */
+#ifndef PCMK_SCHEMAS_EMERGENCY_XSLT
+#define PCMK_SCHEMAS_EMERGENCY_XSLT 1
+#endif
+
 static xmlNode *
 apply_transformation(xmlNode *xml, const char *transform, gboolean to_logs)
 {
@@ -717,10 +907,16 @@ apply_transformation(xmlNode *xml, const char *transform, gboolean to_logs)
     xmlDocPtr res = NULL;
     xmlDocPtr doc = NULL;
     xsltStylesheet *xslt = NULL;
+#if PCMK_SCHEMAS_EMERGENCY_XSLT != 0
+    xmlChar *emergency_result;
+    int emergency_txt_len;
+    int emergency_res;
+#endif
 
     CRM_CHECK(xml != NULL, return FALSE);
     doc = getDocPtr(xml);
-    xform = get_schema_path(NULL, transform);
+    xform = pcmk__xml_artefact_path(pcmk__xml_artefact_ns_legacy_xslt,
+                                    transform);
 
     xmlLoadExtDtdDefaultValue = 1;
     xmlSubstituteEntitiesDefault(1);
@@ -729,10 +925,10 @@ apply_transformation(xmlNode *xml, const char *transform, gboolean to_logs)
     if (to_logs) {
         xsltSetGenericErrorFunc(NULL, cib_upgrade_err);
     } else {
-        xsltSetGenericErrorFunc((void *) stderr, (xmlGenericErrorFunc) fprintf);
+        xsltSetGenericErrorFunc(&crm_log_level, cib_upgrade_err);
     }
 
-    xslt = xsltParseStylesheetFile((const xmlChar *)xform);
+    xslt = xsltParseStylesheetFile((pcmkXmlStr) xform);
     CRM_CHECK(xslt != NULL, goto cleanup);
 
     res = xsltApplyStylesheet(xslt, doc, NULL);
@@ -740,7 +936,17 @@ apply_transformation(xmlNode *xml, const char *transform, gboolean to_logs)
 
     xsltSetGenericErrorFunc(NULL, NULL);  /* restore default one */
 
+
+#if PCMK_SCHEMAS_EMERGENCY_XSLT != 0
+    emergency_res = xsltSaveResultToString(&emergency_result,
+                                           &emergency_txt_len, res, xslt);
+    xmlFreeDoc(res);
+    CRM_CHECK(emergency_res == 0, goto cleanup);
+    out = string2xml((const char *) emergency_result);
+    free(emergency_result);
+#else
     out = xmlDocGetRootElement(res);
+#endif
 
   cleanup:
     if (xslt) {
@@ -751,7 +957,66 @@ apply_transformation(xmlNode *xml, const char *transform, gboolean to_logs)
 
     return out;
 }
-#endif
+
+/*!
+ * \internal
+ * \brief Possibly full enter->upgrade->leave trip per internal bookkeeping.
+ *
+ * \note Only emits warnings about enter/leave phases in case of issues.
+ */
+static xmlNode *
+apply_upgrade(xmlNode *xml, const struct schema_s *schema, gboolean to_logs)
+{
+    bool transform_onleave = schema->transform_onleave;
+    char *transform_leave;
+    xmlNode *upgrade = NULL,
+            *final = NULL;
+
+    if (schema->transform_enter) {
+        crm_debug("Upgrading %s-style configuration, pre-upgrade phase with %s.xsl",
+                  schema->name, schema->transform_enter);
+        upgrade = apply_transformation(xml, schema->transform_enter, to_logs);
+        if (upgrade == NULL) {
+            crm_warn("Upgrade-enter transformation %s.xsl failed",
+                     schema->transform_enter);
+            transform_onleave = FALSE;
+        }
+    }
+    if (upgrade == NULL) {
+        upgrade = xml;
+    }
+
+    crm_debug("Upgrading %s-style configuration, main phase with %s.xsl",
+              schema->name, schema->transform);
+    final = apply_transformation(upgrade, schema->transform, to_logs);
+    if (upgrade != xml) {
+        free_xml(upgrade);
+        upgrade = NULL;
+    }
+
+    if (final != NULL && transform_onleave) {
+        upgrade = final;
+        /* following condition ensured in add_schema_by_version */
+        CRM_ASSERT(schema->transform_enter != NULL);
+        transform_leave = strdup(schema->transform_enter);
+        /* enter -> leave */
+        memcpy(strrchr(transform_leave, '-') + 1, "leave", sizeof("leave") - 1);
+        crm_debug("Upgrading %s-style configuration, post-upgrade phase with %s.xsl",
+                  schema->name, transform_leave);
+        final = apply_transformation(upgrade, transform_leave, to_logs);
+        if (final == NULL) {
+            crm_warn("Upgrade-leave transformation %s.xsl failed", transform_leave);
+            final = upgrade;
+        } else {
+            free_xml(upgrade);
+        }
+        free(transform_leave);
+    }
+
+    return final;
+}
+
+#endif  /* HAVE_LIBXSLT */
 
 const char *
 get_schema_name(int version)
@@ -865,27 +1130,33 @@ update_validation(xmlNode **xml_blob, int *best, int max, gboolean transform,
                           known_schemas[lpc].name, lpc, next, max);
                 break;
 
-            } else if (known_schemas[lpc].transform == NULL) {
+            } else if (known_schemas[lpc].transform == NULL
+                       /* possibly avoid transforming when readily valid
+                          (in general more restricted when crossing the major
+                          version boundary, as X.0 "transitional" version is
+                          expected to be more strict than it's successors that
+                          may re-allow constructs from previous major line) */
+                       || validate_with_silent(xml, next)) {
                 crm_debug("%s-style configuration is also valid for %s",
                            known_schemas[lpc].name, known_schemas[next].name);
 
                 lpc = next;
 
             } else {
-                crm_debug("Upgrading %s-style configuration to %s with %s",
+                crm_debug("Upgrading %s-style configuration to %s with %s.xsl",
                            known_schemas[lpc].name, known_schemas[next].name,
-                           known_schemas[lpc].transform ? known_schemas[lpc].transform : "no-op");
+                           known_schemas[lpc].transform);
 
 #if HAVE_LIBXSLT
-                upgrade = apply_transformation(xml, known_schemas[lpc].transform, to_logs);
+                upgrade = apply_upgrade(xml, &known_schemas[lpc], to_logs);
 #endif
                 if (upgrade == NULL) {
-                    crm_err("Transformation %s failed",
+                    crm_err("Transformation %s.xsl failed",
                             known_schemas[lpc].transform);
                     rc = -pcmk_err_transform_failed;
 
                 } else if (validate_with(upgrade, next, to_logs)) {
-                    crm_info("Transformation %s successful",
+                    crm_info("Transformation %s.xsl successful",
                              known_schemas[lpc].transform);
                     lpc = next;
                     *best = next;
@@ -894,7 +1165,7 @@ update_validation(xmlNode **xml_blob, int *best, int max, gboolean transform,
                     rc = pcmk_ok;
 
                 } else {
-                    crm_err("Transformation %s did not produce a valid configuration",
+                    crm_err("Transformation %s.xsl did not produce a valid configuration",
                             known_schemas[lpc].transform);
                     crm_log_xml_info(upgrade, "transform:bad");
                     free_xml(upgrade);
