@@ -1,5 +1,7 @@
 /*
- * Copyright 2004-2018 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2004-2019 the Pacemaker project contributors
+ *
+ * The version control history for this file may have further details.
  *
  * This source code is licensed under the GNU Lesser General Public License
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
@@ -17,44 +19,48 @@ void populate_hash(xmlNode * nvpair_list, GHashTable * hash, const char **attrs,
 
 resource_object_functions_t resource_class_functions[] = {
     {
-     native_unpack,
-     native_find_rsc,
-     native_parameter,
-     native_print,
-     native_active,
-     native_resource_state,
-     native_location,
-     native_free
+         native_unpack,
+         native_find_rsc,
+         native_parameter,
+         native_print,
+         native_active,
+         native_resource_state,
+         native_location,
+         native_free,
+         pe__count_common,
     },
     {
-     group_unpack,
-     native_find_rsc,
-     native_parameter,
-     group_print,
-     group_active,
-     group_resource_state,
-     native_location,
-     group_free
+         group_unpack,
+         native_find_rsc,
+         native_parameter,
+         group_print,
+         group_active,
+         group_resource_state,
+         native_location,
+         group_free,
+         pe__count_common,
     },
     {
-     clone_unpack,
-     native_find_rsc,
-     native_parameter,
-     clone_print,
-     clone_active,
-     clone_resource_state,
-     native_location,
-     clone_free
+         clone_unpack,
+         native_find_rsc,
+         native_parameter,
+         clone_print,
+         clone_active,
+         clone_resource_state,
+         native_location,
+         clone_free,
+         pe__count_common,
     },
     {
-     container_unpack,
-     native_find_rsc,
-     native_parameter,
-     container_print,
-     container_active,
-     container_resource_state,
-     native_location,
-     container_free
+         pe__unpack_bundle,
+         native_find_rsc,
+         native_parameter,
+         pe__print_bundle,
+         pe__bundle_active,
+         pe__bundle_resource_state,
+         native_location,
+         pe__free_bundle,
+         pe__count_bundle,
     }
 };
 
@@ -108,8 +114,8 @@ get_meta_attributes(GHashTable * meta_hash, resource_t * rsc,
         }
     }
 
-    unpack_instance_attributes(data_set->input, rsc->xml, XML_TAG_META_SETS, node_hash,
-                               meta_hash, NULL, FALSE, data_set->now);
+    pe__unpack_dataset_nvpairs(rsc->xml, XML_TAG_META_SETS, node_hash,
+                               meta_hash, NULL, FALSE, data_set);
 
     /* set anything else based on the parent */
     if (rsc->parent != NULL) {
@@ -117,8 +123,8 @@ get_meta_attributes(GHashTable * meta_hash, resource_t * rsc,
     }
 
     /* and finally check the defaults */
-    unpack_instance_attributes(data_set->input, data_set->rsc_defaults, XML_TAG_META_SETS,
-                               node_hash, meta_hash, NULL, FALSE, data_set->now);
+    pe__unpack_dataset_nvpairs(data_set->rsc_defaults, XML_TAG_META_SETS,
+                               node_hash, meta_hash, NULL, FALSE, data_set);
 }
 
 void
@@ -131,8 +137,8 @@ get_rsc_attributes(GHashTable * meta_hash, resource_t * rsc,
         node_hash = node->details->attrs;
     }
 
-    unpack_instance_attributes(data_set->input, rsc->xml, XML_TAG_ATTR_SETS, node_hash,
-                               meta_hash, NULL, FALSE, data_set->now);
+    pe__unpack_dataset_nvpairs(rsc->xml, XML_TAG_ATTR_SETS, node_hash,
+                               meta_hash, NULL, FALSE, data_set);
 
     /* set anything else based on the parent */
     if (rsc->parent != NULL) {
@@ -140,8 +146,8 @@ get_rsc_attributes(GHashTable * meta_hash, resource_t * rsc,
 
     } else {
         /* and finally check the defaults */
-        unpack_instance_attributes(data_set->input, data_set->rsc_defaults, XML_TAG_ATTR_SETS,
-                                   node_hash, meta_hash, NULL, FALSE, data_set->now);
+        pe__unpack_dataset_nvpairs(data_set->rsc_defaults, XML_TAG_ATTR_SETS,
+                                   node_hash, meta_hash, NULL, FALSE, data_set);
     }
 }
 
@@ -157,7 +163,7 @@ pe_get_versioned_attributes(xmlNode * meta_hash, resource_t * rsc,
     }
 
     pe_unpack_versioned_attributes(data_set->input, rsc->xml, XML_TAG_ATTR_SETS, node_hash,
-                                   meta_hash, data_set->now);
+                                   meta_hash, data_set->now, NULL);
 
     /* set anything else based on the parent */
     if (rsc->parent != NULL) {
@@ -166,7 +172,7 @@ pe_get_versioned_attributes(xmlNode * meta_hash, resource_t * rsc,
     } else {
         /* and finally check the defaults */
         pe_unpack_versioned_attributes(data_set->input, data_set->rsc_defaults, XML_TAG_ATTR_SETS,
-                                       node_hash, meta_hash, data_set->now);
+                                       node_hash, meta_hash, data_set->now, NULL);
     }
 }
 #endif
@@ -244,7 +250,7 @@ unpack_template(xmlNode * xml_obj, xmlNode ** expanded_xml, pe_working_set_t * d
 
     template_ops = find_xml_node(new_xml, "operations", FALSE);
 
-    for (child_xml = __xml_first_child(xml_obj); child_xml != NULL;
+    for (child_xml = __xml_first_child_element(xml_obj); child_xml != NULL;
          child_xml = __xml_next_element(child_xml)) {
         xmlNode *new_child = NULL;
 
@@ -261,13 +267,17 @@ unpack_template(xmlNode * xml_obj, xmlNode ** expanded_xml, pe_working_set_t * d
                                                          g_str_equal, free,
                                                          NULL);
 
-        for (op = __xml_first_child(rsc_ops); op != NULL; op = __xml_next_element(op)) {
+        for (op = __xml_first_child_element(rsc_ops); op != NULL;
+             op = __xml_next_element(op)) {
+
             char *key = template_op_key(op);
 
             g_hash_table_insert(rsc_ops_hash, key, op);
         }
 
-        for (op = __xml_first_child(template_ops); op != NULL; op = __xml_next_element(op)) {
+        for (op = __xml_first_child_element(template_ops); op != NULL;
+             op = __xml_next_element(op)) {
+
             char *key = template_op_key(op);
 
             if (g_hash_table_lookup(rsc_ops_hash, key) == NULL) {
@@ -364,8 +374,8 @@ common_unpack(xmlNode * xml_obj, resource_t ** rsc,
     const char *value = NULL;
     const char *rclass = NULL; /* Look for this after any templates have been expanded */
     const char *id = crm_element_value(xml_obj, XML_ATTR_ID);
-    int container_remote_node = 0;
-    int baremetal_remote_node = 0;
+    bool guest_node = FALSE;
+    bool remote_node = FALSE;
     bool has_versioned_params = FALSE;
 
     crm_log_xml_trace(xml_obj, "Processing resource input...");
@@ -473,9 +483,9 @@ common_unpack(xmlNode * xml_obj, resource_t ** rsc,
     if (xml_contains_remote_node((*rsc)->xml)) {
         (*rsc)->is_remote_node = TRUE;
         if (g_hash_table_lookup((*rsc)->meta, XML_RSC_ATTR_CONTAINER)) {
-            container_remote_node = 1;
+            guest_node = TRUE;
         } else {
-            baremetal_remote_node = 1;
+            remote_node = TRUE;
         }
     }
 
@@ -487,8 +497,8 @@ common_unpack(xmlNode * xml_obj, resource_t ** rsc,
         pe_rsc_trace((*rsc), "Migration is disabled for resources with versioned parameters");
     } else if (crm_is_true(value)) {
         set_bit((*rsc)->flags, pe_rsc_allow_migrate);
-    } else if ((value == NULL) && baremetal_remote_node && !has_versioned_params) {
-        /* by default, we want baremetal remote-nodes to be able
+    } else if ((value == NULL) && remote_node && !has_versioned_params) {
+        /* By default, we want remote nodes to be able
          * to float around the cluster without having to stop all the
          * resources within the remote-node before moving. Allowing
          * migration support enables this feature. If this ever causes
@@ -573,6 +583,16 @@ common_unpack(xmlNode * xml_obj, resource_t ** rsc,
     value = g_hash_table_lookup((*rsc)->meta, XML_RSC_ATTR_FAIL_STICKINESS);
     if (value != NULL && safe_str_neq("default", value)) {
         (*rsc)->migration_threshold = char2score(value);
+        if ((*rsc)->migration_threshold < 0) {
+            /* @TODO We use 1 here to preserve previous behavior, but this
+             * should probably use the default (INFINITY) or 0 (to disable)
+             * instead.
+             */
+            pe_warn_once(pe_wo_neg_threshold,
+                         XML_RSC_ATTR_FAIL_STICKINESS
+                         " must be non-negative, using 1 instead");
+            (*rsc)->migration_threshold = 1;
+        }
     }
 
     if (safe_str_eq(rclass, PCMK_RESOURCE_CLASS_STONITH)) {
@@ -649,11 +669,11 @@ common_unpack(xmlNode * xml_obj, resource_t ** rsc,
     pe_rsc_trace((*rsc), "\tRequired to start: %s%s", value, isdefault?" (default)":"");
     value = g_hash_table_lookup((*rsc)->meta, XML_RSC_ATTR_FAIL_TIMEOUT);
     if (value != NULL) {
-        /* call crm_get_msec() and convert back to seconds */
-        (*rsc)->failure_timeout = (crm_get_msec(value) / 1000);
+        // Stored as seconds
+        (*rsc)->failure_timeout = (int) (crm_parse_interval_spec(value) / 1000);
     }
 
-    if (baremetal_remote_node) {
+    if (remote_node) {
         value = g_hash_table_lookup((*rsc)->parameters, XML_REMOTE_ATTR_RECONNECT_INTERVAL);
         if (value) {
             /* reconnect delay works by setting failure_timeout and preventing the
@@ -676,7 +696,7 @@ common_unpack(xmlNode * xml_obj, resource_t ** rsc,
     if (is_set(data_set->flags, pe_flag_symmetric_cluster)) {
         // This tag must stay exactly the same because it is tested elsewhere
         resource_location(*rsc, NULL, 0, "symmetric_default", data_set);
-    } else if (container_remote_node) {
+    } else if (guest_node) {
         /* remote resources tied to a container resource must always be allowed
          * to opt-in to the cluster. Whether the connection resource is actually
          * allowed to be placed on a node is dependent on the container resource */
@@ -688,8 +708,8 @@ common_unpack(xmlNode * xml_obj, resource_t ** rsc,
 
     (*rsc)->utilization = crm_str_table_new();
 
-    unpack_instance_attributes(data_set->input, (*rsc)->xml, XML_TAG_UTILIZATION, NULL,
-                               (*rsc)->utilization, NULL, FALSE, data_set->now);
+    pe__unpack_dataset_nvpairs((*rsc)->xml, XML_TAG_UTILIZATION, NULL,
+                               (*rsc)->utilization, NULL, FALSE, data_set);
 
 /* 	data_set->resources = g_list_append(data_set->resources, (*rsc)); */
 
@@ -924,3 +944,23 @@ pe__find_active_requires(const pe_resource_t *rsc, unsigned int *count)
     }
     return pe__find_active_on(rsc, count, NULL);
 }
+
+void
+pe__count_common(pe_resource_t *rsc)
+{
+    if (rsc->children != NULL) {
+        for (GList *item = rsc->children; item != NULL; item = item->next) {
+            ((pe_resource_t *) item->data)->fns->count(item->data);
+        }
+
+    } else if (is_not_set(rsc->flags, pe_rsc_orphan)
+               || (rsc->role > RSC_ROLE_STOPPED)) {
+        rsc->cluster->ninstances++;
+        if (pe__resource_is_disabled(rsc)) {
+            rsc->cluster->disabled_resources++;
+        }
+        if (is_set(rsc->flags, pe_rsc_block)) {
+            rsc->cluster->blocked_resources++;
+        }
+    }
+}
