@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2017 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2008-2019 the Pacemaker project contributors
  *
  * This source code is licensed under the GNU Lesser General Public License
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
@@ -23,9 +23,10 @@ is_matched_failure(const char *rsc_id, xmlNode *conf_op_xml,
 {
     gboolean matched = FALSE;
     const char *conf_op_name = NULL;
-    int conf_op_interval = 0;
     const char *lrm_op_task = NULL;
-    int lrm_op_interval = 0;
+    const char *conf_op_interval_spec = NULL;
+    guint conf_op_interval_ms = 0;
+    guint lrm_op_interval_ms = 0;
     const char *lrm_op_id = NULL;
     char *last_failure_key = NULL;
 
@@ -33,13 +34,19 @@ is_matched_failure(const char *rsc_id, xmlNode *conf_op_xml,
         return FALSE;
     }
 
+    // Get name and interval from configured op
     conf_op_name = crm_element_value(conf_op_xml, "name");
-    conf_op_interval = crm_get_msec(crm_element_value(conf_op_xml, "interval"));
-    lrm_op_task = crm_element_value(lrm_op_xml, XML_LRM_ATTR_TASK);
-    crm_element_value_int(lrm_op_xml, XML_LRM_ATTR_INTERVAL, &lrm_op_interval);
+    conf_op_interval_spec = crm_element_value(conf_op_xml,
+                                              XML_LRM_ATTR_INTERVAL);
+    conf_op_interval_ms = crm_parse_interval_spec(conf_op_interval_spec);
 
-    if (safe_str_eq(conf_op_name, lrm_op_task) == FALSE
-        || conf_op_interval != lrm_op_interval) {
+    // Get name and interval from op history entry
+    lrm_op_task = crm_element_value(lrm_op_xml, XML_LRM_ATTR_TASK);
+    crm_element_value_ms(lrm_op_xml, XML_LRM_ATTR_INTERVAL_MS,
+                         &lrm_op_interval_ms);
+
+    if ((conf_op_interval_ms != lrm_op_interval_ms)
+        || safe_str_neq(conf_op_name, lrm_op_task)) {
         return FALSE;
     }
 
@@ -51,11 +58,11 @@ is_matched_failure(const char *rsc_id, xmlNode *conf_op_xml,
 
     } else {
         char *expected_op_key = generate_op_key(rsc_id, conf_op_name,
-                                                conf_op_interval);
+                                                conf_op_interval_ms);
 
         if (safe_str_eq(expected_op_key, lrm_op_id)) {
             int rc = 0;
-            int target_rc = get_target_rc(lrm_op_xml);
+            int target_rc = pe__target_rc_from_xml(lrm_op_xml);
 
             crm_element_value_int(lrm_op_xml, XML_LRM_ATTR_RC, &rc);
             if (rc != target_rc) {
@@ -74,21 +81,25 @@ block_failure(node_t *node, resource_t *rsc, xmlNode *xml_op,
               pe_working_set_t *data_set)
 {
     char *xml_name = clone_strip(rsc->id);
+
+    /* @TODO This xpath search occurs after template expansion, but it is unable
+     * to properly detect on-fail in id-ref, operation meta-attributes, or
+     * op_defaults, or evaluate rules.
+     *
+     * Also, on-fail defaults to block (in unpack_operation()) for stop actions
+     * when stonith is disabled.
+     *
+     * Ideally, we'd unpack the operation before this point, and pass in a
+     * meta-attributes table that takes all that into consideration.
+     */
     char *xpath = crm_strdup_printf("//primitive[@id='%s']//op[@on-fail='block']",
                                     xml_name);
+
     xmlXPathObject *xpathObj = xpath_search(rsc->xml, xpath);
     gboolean should_block = FALSE;
 
     free(xpath);
 
-#if 0
-    /* A good idea? */
-    if (rsc->container == NULL && is_not_set(data_set->flags, pe_flag_stonith_enabled)) {
-        /* In this case, stop on-fail defaults to block in unpack_operation() */
-        return TRUE;
-    }
-#endif
-
     if (xpathObj) {
         int max = numXpathResults(xpathObj);
         int lpc = 0;
@@ -104,18 +115,21 @@ block_failure(node_t *node, resource_t *rsc, xmlNode *xml_op,
 
             } else {
                 const char *conf_op_name = NULL;
-                int conf_op_interval = 0;
+                const char *conf_op_interval_spec = NULL;
+                guint conf_op_interval_ms = 0;
                 char *lrm_op_xpath = NULL;
                 xmlXPathObject *lrm_op_xpathObj = NULL;
 
+                // Get name and interval from configured op
                 conf_op_name = crm_element_value(pref, "name");
-                conf_op_interval = crm_get_msec(crm_element_value(pref, "interval"));
+                conf_op_interval_spec = crm_element_value(pref, XML_LRM_ATTR_INTERVAL);
+                conf_op_interval_ms = crm_parse_interval_spec(conf_op_interval_spec);
 
                 lrm_op_xpath = crm_strdup_printf("//node_state[@uname='%s']"
                                                "//lrm_resource[@id='%s']"
-                                               "/lrm_rsc_op[@operation='%s'][@interval='%d']",
+                                               "/lrm_rsc_op[@operation='%s'][@interval='%u']",
                                                node->details->uname, xml_name,
-                                               conf_op_name, conf_op_interval);
+                                               conf_op_name, conf_op_interval_ms);
                 lrm_op_xpathObj = xpath_search(data_set->input, lrm_op_xpath);
 
                 free(lrm_op_xpath);
@@ -268,7 +282,7 @@ pe_get_failcount(node_t *node, resource_t *rsc, time_t *last_failure,
         && block_failure(node, rsc, xml_op, data_set)) {
 
         pe_warn("Ignoring failure timeout %d for %s because it conflicts with on-fail=block",
-                rsc->id, rsc->failure_timeout);
+                rsc->failure_timeout, rsc->id);
         rsc->failure_timeout = 0;
     }
 
@@ -285,7 +299,19 @@ pe_get_failcount(node_t *node, resource_t *rsc, time_t *last_failure,
         }
     }
 
-    if (is_set(flags, pe_fc_fillers) && rsc->fillers) {
+    /* We never want the fail counts of a bundle container's fillers to
+     * count towards the container's fail count.
+     *
+     * Most importantly, a Pacemaker Remote connection to a bundle container
+     * is a filler of the container, but can reside on a different node than the
+     * container itself. Counting its fail count on its node towards the
+     * container's fail count on that node could lead to attempting to stop the
+     * container on the wrong node.
+     */
+
+    if (is_set(flags, pe_fc_fillers) && rsc->fillers
+        && !pe_rsc_is_bundled(rsc)) {
+
         GListPtr gIter = NULL;
 
         for (gIter = rsc->fillers; gIter != NULL; gIter = gIter->next) {
@@ -319,3 +345,31 @@ pe_get_failcount(node_t *node, resource_t *rsc, time_t *last_failure,
 
     return failcount;
 }
+
+/*!
+ * \brief Schedule a controller operation to clear a fail count
+ *
+ * \param[in] rsc       Resource with failure
+ * \param[in] node      Node failure occurred on
+ * \param[in] reason    Readable description why needed (for logging)
+ * \param[in] data_set  Working set for cluster
+ *
+ * \return Scheduled action
+ */
+pe_action_t *
+pe__clear_failcount(pe_resource_t *rsc, pe_node_t *node,
+                    const char *reason, pe_working_set_t *data_set)
+{
+    char *key = NULL;
+    action_t *clear = NULL;
+
+    CRM_CHECK(rsc && node && reason && data_set, return NULL);
+
+    key = generate_op_key(rsc->id, CRM_OP_CLEAR_FAILCOUNT, 0);
+    clear = custom_action(rsc, key, CRM_OP_CLEAR_FAILCOUNT, node, FALSE, TRUE,
+                          data_set);
+    add_hash_param(clear->meta, XML_ATTR_TE_NOWAIT, XML_BOOLEAN_TRUE);
+    crm_notice("Clearing failure of %s on %s because %s " CRM_XS " %s",
+               rsc->id, node->details->uname, reason, clear->uuid);
+    return clear;
+}
