@@ -15,6 +15,7 @@
 #include <glib.h>
 
 #include <crm/pengine/rules.h>
+#include <crm/pengine/rules_internal.h>
 #include <crm/pengine/internal.h>
 
 #include <sys/types.h>
@@ -23,38 +24,51 @@
 
 CRM_TRACE_INIT_DATA(pe_rules);
 
-crm_time_t *parse_xml_duration(crm_time_t * start, xmlNode * duration_spec);
-
-gboolean test_date_expression(xmlNode * time_expr, crm_time_t * now);
-gboolean cron_range_satisfied(crm_time_t * now, xmlNode * cron_spec);
-gboolean test_attr_expression(xmlNode * expr, GHashTable * hash, crm_time_t * now);
-gboolean pe_test_attr_expression_full(xmlNode * expr, GHashTable * hash, crm_time_t * now, pe_match_data_t * match_data);
-gboolean test_role_expression(xmlNode * expr, enum rsc_role_e role, crm_time_t * now);
-
+/*!
+ * \brief Evaluate any rules contained by given XML element
+ *
+ * \param[in]  xml          XML element to check for rules
+ * \param[in]  node_hash    Node attributes to use when evaluating expressions
+ * \param[in]  now          Time to use when evaluating expressions
+ * \param[out] next_change  If not NULL, set to when evaluation will change
+ *
+ * \return TRUE if no rules, or any of rules present is in effect, else FALSE
+ */
 gboolean
-test_ruleset(xmlNode * ruleset, GHashTable * node_hash, crm_time_t * now)
+pe_evaluate_rules(xmlNode *ruleset, GHashTable *node_hash, crm_time_t *now,
+                  crm_time_t *next_change)
 {
+    // If there are no rules, pass by default
     gboolean ruleset_default = TRUE;
-    xmlNode *rule = NULL;
 
-    for (rule = __xml_first_child_element(ruleset); rule != NULL;
-         rule = __xml_next_element(rule)) {
+    for (xmlNode *rule = first_named_child(ruleset, XML_TAG_RULE);
+         rule != NULL; rule = crm_next_same_xml(rule)) {
 
-        if (crm_str_eq((const char *)rule->name, XML_TAG_RULE, TRUE)) {
-            ruleset_default = FALSE;
-            if (test_rule(rule, node_hash, RSC_ROLE_UNKNOWN, now)) {
-                return TRUE;
-            }
+        ruleset_default = FALSE;
+        if (pe_test_rule(rule, node_hash, RSC_ROLE_UNKNOWN, now, next_change,
+                         NULL)) {
+            /* Only the deprecated "lifetime" element of location constraints
+             * may contain more than one rule at the top level -- the schema
+             * limits a block of nvpairs to a single top-level rule. So, this
+             * effectively means that a lifetime is active if any rule it
+             * contains is active.
+             */
+            return TRUE;
         }
     }
-
     return ruleset_default;
 }
 
+gboolean
+test_ruleset(xmlNode *ruleset, GHashTable *node_hash, crm_time_t *now)
+{
+    return pe_evaluate_rules(ruleset, node_hash, now, NULL);
+}
+
 gboolean
 test_rule(xmlNode * rule, GHashTable * node_hash, enum rsc_role_e role, crm_time_t * now)
 {
-    return pe_test_rule_full(rule, node_hash, role, now, NULL);
+    return pe_test_rule(rule, node_hash, role, now, NULL, NULL);
 }
 
 gboolean
@@ -65,11 +79,13 @@ pe_test_rule_re(xmlNode * rule, GHashTable * node_hash, enum rsc_role_e role, cr
                                     .params = NULL,
                                     .meta = NULL,
                                  };
-    return pe_test_rule_full(rule, node_hash, role, now, &match_data);
+    return pe_test_rule(rule, node_hash, role, now, NULL, &match_data);
 }
 
 gboolean
-pe_test_rule_full(xmlNode * rule, GHashTable * node_hash, enum rsc_role_e role, crm_time_t * now, pe_match_data_t * match_data)
+pe_test_rule(xmlNode *rule, GHashTable *node_hash, enum rsc_role_e role,
+             crm_time_t *now, crm_time_t *next_change,
+             pe_match_data_t *match_data)
 {
     xmlNode *expr = NULL;
     gboolean test = TRUE;
@@ -89,7 +105,8 @@ pe_test_rule_full(xmlNode * rule, GHashTable * node_hash, enum rsc_role_e role,
     for (expr = __xml_first_child_element(rule); expr != NULL;
          expr = __xml_next_element(expr)) {
 
-        test = pe_test_expression_full(expr, node_hash, role, now, match_data);
+        test = pe_test_expression(expr, node_hash, role, now, next_change,
+                                  match_data);
         empty = FALSE;
 
         if (test && do_and == FALSE) {
@@ -110,10 +127,17 @@ pe_test_rule_full(xmlNode * rule, GHashTable * node_hash, enum rsc_role_e role,
     return passed;
 }
 
+gboolean
+pe_test_rule_full(xmlNode *rule, GHashTable *node_hash, enum rsc_role_e role,
+                  crm_time_t *now, pe_match_data_t *match_data)
+{
+    return pe_test_rule(rule, node_hash, role, now, NULL, match_data);
+}
+
 gboolean
 test_expression(xmlNode * expr, GHashTable * node_hash, enum rsc_role_e role, crm_time_t * now)
 {
-    return pe_test_expression_full(expr, node_hash, role, now, NULL);
+    return pe_test_expression(expr, node_hash, role, now, NULL, NULL);
 }
 
 gboolean
@@ -124,18 +148,37 @@ pe_test_expression_re(xmlNode * expr, GHashTable * node_hash, enum rsc_role_e ro
                                     .params = NULL,
                                     .meta = NULL,
                                  };
-    return pe_test_expression_full(expr, node_hash, role, now, &match_data);
+    return pe_test_expression(expr, node_hash, role, now, NULL, &match_data);
 }
 
+/*!
+ * \brief Evaluate one rule subelement (pass/fail)
+ *
+ * A rule element may contain another rule, a node attribute expression, or a
+ * date expression. Given any one of those, evaluate it and return whether it
+ * passed.
+ *
+ * \param[in]  expr         Rule subelement XML
+ * \param[in]  node_hash    Node attributes to use when evaluating expression
+ * \param[in]  role         Resource role to use when evaluating expression
+ * \param[in]  now          Time to use when evaluating expression
+ * \param[out] next_change  If not NULL, set to when evaluation will change
+ * \param[in]  match_data   If not NULL, resource back-references and params
+ *
+ * \return TRUE if expression is in effect under given conditions, else FALSE
+ */
 gboolean
-pe_test_expression_full(xmlNode * expr, GHashTable * node_hash, enum rsc_role_e role, crm_time_t * now, pe_match_data_t * match_data)
+pe_test_expression(xmlNode *expr, GHashTable *node_hash, enum rsc_role_e role,
+                   crm_time_t *now, crm_time_t *next_change,
+                   pe_match_data_t *match_data)
 {
     gboolean accept = FALSE;
     const char *uname = NULL;
 
     switch (find_expression_type(expr)) {
         case nested_rule:
-            accept = pe_test_rule_full(expr, node_hash, role, now, match_data);
+            accept = pe_test_rule(expr, node_hash, role, now, next_change,
+                                  match_data);
             break;
         case attr_expr:
         case loc_expr:
@@ -143,24 +186,24 @@ pe_test_expression_full(xmlNode * expr, GHashTable * node_hash, enum rsc_role_e
              * no node to compare with
              */
             if (node_hash != NULL) {
-                accept = pe_test_attr_expression_full(expr, node_hash, now, match_data);
+                accept = pe_test_attr_expression(expr, node_hash, now, match_data);
             }
             break;
 
         case time_expr:
-            accept = test_date_expression(expr, now);
+            accept = pe_test_date_expression(expr, now, next_change);
             break;
 
         case role_expr:
-            accept = test_role_expression(expr, role, now);
+            accept = pe_test_role_expression(expr, role, now);
             break;
 
-#ifdef ENABLE_VERSIONED_ATTRS
+#if ENABLE_VERSIONED_ATTRS
         case version_expr:
             if (node_hash && g_hash_table_lookup_extended(node_hash,
                                                           CRM_ATTR_RA_VERSION,
                                                           NULL, NULL)) {
-                accept = test_attr_expression(expr, node_hash, now);
+                accept = pe_test_attr_expression(expr, node_hash, now, NULL);
             } else {
                 // we are going to test it when we have ra-version
                 accept = TRUE;
@@ -181,6 +224,14 @@ pe_test_expression_full(xmlNode * expr, GHashTable * node_hash, enum rsc_role_e
     return accept;
 }
 
+gboolean
+pe_test_expression_full(xmlNode *expr, GHashTable *node_hash,
+                        enum rsc_role_e role, crm_time_t *now,
+                        pe_match_data_t *match_data)
+{
+    return pe_test_expression(expr, node_hash, role, now, NULL, match_data);
+}
+
 enum expression_type
 find_expression_type(xmlNode * expr)
 {
@@ -207,7 +258,7 @@ find_expression_type(xmlNode * expr)
     } else if (safe_str_eq(attr, CRM_ATTR_ROLE)) {
         return role_expr;
 
-#ifdef ENABLE_VERSIONED_ATTRS
+#if ENABLE_VERSIONED_ATTRS
     } else if (safe_str_eq(attr, CRM_ATTR_RA_VERSION)) {
         return version_expr;
 #endif
@@ -217,7 +268,7 @@ find_expression_type(xmlNode * expr)
 }
 
 gboolean
-test_role_expression(xmlNode * expr, enum rsc_role_e role, crm_time_t * now)
+pe_test_role_expression(xmlNode * expr, enum rsc_role_e role, crm_time_t * now)
 {
     gboolean accept = FALSE;
     const char *op = NULL;
@@ -246,7 +297,7 @@ test_role_expression(xmlNode * expr, enum rsc_role_e role, crm_time_t * now)
         }
 
     } else if (safe_str_eq(op, "ne")) {
-        /* we will only test "ne" wtih master/slave roles style */
+        // Test "ne" only with promotable clone roles
         if (role < RSC_ROLE_SLAVE && role > RSC_ROLE_UNKNOWN) {
             accept = FALSE;
 
@@ -258,13 +309,8 @@ test_role_expression(xmlNode * expr, enum rsc_role_e role, crm_time_t * now)
 }
 
 gboolean
-test_attr_expression(xmlNode * expr, GHashTable * hash, crm_time_t * now)
-{
-    return pe_test_attr_expression_full(expr, hash, now, NULL);
-}
-
-gboolean
-pe_test_attr_expression_full(xmlNode * expr, GHashTable * hash, crm_time_t * now, pe_match_data_t * match_data)
+pe_test_attr_expression(xmlNode *expr, GHashTable *hash, crm_time_t *now,
+                        pe_match_data_t *match_data)
 {
     gboolean accept = FALSE;
     gboolean attr_allocated = FALSE;
@@ -455,9 +501,7 @@ phase_of_the_moon(crm_time_t * now)
 static gboolean
 decodeNVpair(const char *srcstring, char separator, char **name, char **value)
 {
-    int lpc = 0;
-    int len = 0;
-    const char *temp = NULL;
+    const char *seploc = NULL;
 
     CRM_ASSERT(name != NULL && value != NULL);
     *name = NULL;
@@ -465,46 +509,15 @@ decodeNVpair(const char *srcstring, char separator, char **name, char **value)
 
     crm_trace("Attempting to decode: [%s]", srcstring);
     if (srcstring != NULL) {
-        len = strlen(srcstring);
-        while (lpc <= len) {
-            if (srcstring[lpc] == separator) {
-                *name = calloc(1, lpc + 1);
-                if (*name == NULL) {
-                    break;      /* and return FALSE */
-                }
-                memcpy(*name, srcstring, lpc);
-                (*name)[lpc] = '\0';
-
-/* this sucks but as the strtok manpage says..
- * it *is* a bug
- */
-                len = len - lpc;
-                len--;
-                if (len <= 0) {
-                    *value = NULL;
-                } else {
-
-                    *value = calloc(1, len + 1);
-                    if (*value == NULL) {
-                        break;  /* and return FALSE */
-                    }
-                    temp = srcstring + lpc + 1;
-                    memcpy(*value, temp, len);
-                    (*value)[len] = '\0';
-                }
-                return TRUE;
+        seploc = strchr(srcstring, separator);
+        if (seploc) {
+            *name = strndup(srcstring, seploc - srcstring);
+            if (*(seploc + 1)) {
+                *value = strdup(seploc + 1);
             }
-            lpc++;
+            return TRUE;
         }
     }
-
-    if (*name != NULL) {
-        free(*name);
-        *name = NULL;
-    }
-    *name = NULL;
-    *value = NULL;
-
     return FALSE;
 }
 
@@ -537,7 +550,7 @@ decodeNVpair(const char *srcstring, char separator, char **name, char **value)
     }
 
 gboolean
-cron_range_satisfied(crm_time_t * now, xmlNode * cron_spec)
+pe_cron_range_satisfied(crm_time_t * now, xmlNode * cron_spec)
 {
     const char *value = NULL;
     char *value_low = NULL;
@@ -585,7 +598,7 @@ cron_range_satisfied(crm_time_t * now, xmlNode * cron_spec)
     }
 
 crm_time_t *
-parse_xml_duration(crm_time_t * start, xmlNode * duration_spec)
+pe_parse_xml_duration(crm_time_t * start, xmlNode * duration_spec)
 {
     crm_time_t *end = NULL;
     const char *value = NULL;
@@ -604,8 +617,55 @@ parse_xml_duration(crm_time_t * start, xmlNode * duration_spec)
     return end;
 }
 
+/*!
+ * \internal
+ * \brief Test a date expression (pass/fail) for a specific time
+ *
+ * \param[in]  time_expr    date_expression XML
+ * \param[in]  now          Time for which to evaluate expression
+ * \param[out] next_change  If not NULL, set to when evaluation will change
+ *
+ * \return TRUE if date expression is in effect at given time, FALSE otherwise
+ */
 gboolean
-test_date_expression(xmlNode * time_expr, crm_time_t * now)
+pe_test_date_expression(xmlNode *time_expr, crm_time_t *now,
+                        crm_time_t *next_change)
+{
+    switch (pe_eval_date_expression(time_expr, now, next_change)) {
+        case pe_date_within_range:
+        case pe_date_op_satisfied:
+            return TRUE;
+
+        default:
+            return FALSE;
+    }
+}
+
+// Set next_change to t if t is earlier
+static void
+crm_time_set_if_earlier(crm_time_t *next_change, crm_time_t *t)
+{
+    if ((next_change != NULL) && (t != NULL)) {
+        if (!crm_time_is_defined(next_change)
+            || (crm_time_compare(t, next_change) < 0)) {
+            crm_time_set(next_change, t);
+        }
+    }
+}
+
+/*!
+ * \internal
+ * \brief Evaluate a date expression for a specific time
+ *
+ * \param[in]  time_expr    date_expression XML
+ * \param[in]  now          Time for which to evaluate expression
+ * \param[out] next_change  If not NULL, set to when evaluation will change
+ *
+ * \return Evaluation result
+ */
+pe_eval_date_result_t
+pe_eval_date_expression(xmlNode *time_expr, crm_time_t *now,
+                        crm_time_t *next_change)
 {
     crm_time_t *start = NULL;
     crm_time_t *end = NULL;
@@ -615,7 +675,8 @@ test_date_expression(xmlNode * time_expr, crm_time_t * now)
     xmlNode *duration_spec = NULL;
     xmlNode *date_spec = NULL;
 
-    gboolean passed = FALSE;
+    // "undetermined" will also be returned for parsing errors
+    pe_eval_date_result_t rc = pe_date_result_undetermined;
 
     crm_trace("Testing expression: %s", ID(time_expr));
 
@@ -632,46 +693,66 @@ test_date_expression(xmlNode * time_expr, crm_time_t * now)
     }
 
     if (start != NULL && end == NULL && duration_spec != NULL) {
-        end = parse_xml_duration(start, duration_spec);
-    }
-    if (op == NULL) {
-        op = "in_range";
+        end = pe_parse_xml_duration(start, duration_spec);
     }
 
-    if (safe_str_eq(op, "date_spec") || safe_str_eq(op, "in_range")) {
-        if (start != NULL && crm_time_compare(start, now) > 0) {
-            passed = FALSE;
-        } else if (end != NULL && crm_time_compare(end, now) < 0) {
-            passed = FALSE;
-        } else if (safe_str_eq(op, "in_range")) {
-            passed = TRUE;
+    if ((op == NULL) || safe_str_eq(op, "in_range")) {
+        if ((start == NULL) && (end == NULL)) {
+            // in_range requires at least one of start or end
+        } else if ((start != NULL) && (crm_time_compare(now, start) < 0)) {
+            rc = pe_date_before_range;
+            crm_time_set_if_earlier(next_change, start);
+        } else if ((end != NULL) && (crm_time_compare(now, end) > 0)) {
+            rc = pe_date_after_range;
         } else {
-            passed = cron_range_satisfied(now, date_spec);
+            rc = pe_date_within_range;
+            if (end && next_change) {
+                // Evaluation doesn't change until second after end
+                crm_time_add_seconds(end, 1);
+                crm_time_set_if_earlier(next_change, end);
+            }
         }
 
-    } else if (safe_str_eq(op, "gt") && crm_time_compare(start, now) < 0) {
-        passed = TRUE;
+    } else if (safe_str_eq(op, "date_spec")) {
+        rc = pe_cron_range_satisfied(now, date_spec) ? pe_date_op_satisfied
+                                                     : pe_date_op_unsatisfied;
+        // @TODO set next_change appropriately
 
-    } else if (safe_str_eq(op, "lt") && crm_time_compare(end, now) > 0) {
-        passed = TRUE;
+    } else if (safe_str_eq(op, "gt")) {
+        if (start == NULL) {
+            // gt requires start
+        } else if (crm_time_compare(now, start) > 0) {
+            rc = pe_date_within_range;
+        } else {
+            rc = pe_date_before_range;
 
-    } else if (safe_str_eq(op, "eq") && crm_time_compare(start, now) == 0) {
-        passed = TRUE;
+            // Evaluation doesn't change until second after start
+            crm_time_add_seconds(start, 1);
+            crm_time_set_if_earlier(next_change, start);
+        }
 
-    } else if (safe_str_eq(op, "neq") && crm_time_compare(start, now) != 0) {
-        passed = TRUE;
+    } else if (safe_str_eq(op, "lt")) {
+        if (end == NULL) {
+            // lt requires end
+        } else if (crm_time_compare(now, end) < 0) {
+            rc = pe_date_within_range;
+            crm_time_set_if_earlier(next_change, end);
+        } else {
+            rc = pe_date_after_range;
+        }
     }
 
     crm_time_free(start);
     crm_time_free(end);
-    return passed;
+    return rc;
 }
 
+// Information about a block of nvpair elements
 typedef struct sorted_set_s {
-    int score;
-    const char *name;
-    const char *special_name;
-    xmlNode *attr_set;
+    int score;                  // This block's score for sorting
+    const char *name;           // This block's ID
+    const char *special_name;   // ID that should sort first
+    xmlNode *attr_set;          // This block
 } sorted_set_t;
 
 static gint
@@ -759,7 +840,7 @@ populate_hash(xmlNode * nvpair_list, GHashTable * hash, gboolean overwrite, xmlN
     }
 }
 
-#ifdef ENABLE_VERSIONED_ATTRS
+#if ENABLE_VERSIONED_ATTRS
 static xmlNode*
 get_versioned_rule(xmlNode * attr_set)
 {
@@ -823,6 +904,7 @@ typedef struct unpack_data_s {
     GHashTable *node_hash;
     void *hash;
     crm_time_t *now;
+    crm_time_t *next_change;
     xmlNode *top;
 } unpack_data_t;
 
@@ -832,11 +914,12 @@ unpack_attr_set(gpointer data, gpointer user_data)
     sorted_set_t *pair = data;
     unpack_data_t *unpack_data = user_data;
 
-    if (test_ruleset(pair->attr_set, unpack_data->node_hash, unpack_data->now) == FALSE) {
+    if (!pe_evaluate_rules(pair->attr_set, unpack_data->node_hash,
+                           unpack_data->now, unpack_data->next_change)) {
         return;
     }
 
-#ifdef ENABLE_VERSIONED_ATTRS
+#if ENABLE_VERSIONED_ATTRS
     if (get_versioned_rule(pair->attr_set) && !(unpack_data->node_hash &&
         g_hash_table_lookup_extended(unpack_data->node_hash,
                                      CRM_ATTR_RA_VERSION, NULL, NULL))) {
@@ -849,25 +932,34 @@ unpack_attr_set(gpointer data, gpointer user_data)
     populate_hash(pair->attr_set, unpack_data->hash, unpack_data->overwrite, unpack_data->top);
 }
 
-#ifdef ENABLE_VERSIONED_ATTRS
+#if ENABLE_VERSIONED_ATTRS
 static void
 unpack_versioned_attr_set(gpointer data, gpointer user_data)
 {
     sorted_set_t *pair = data;
     unpack_data_t *unpack_data = user_data;
 
-    if (test_ruleset(pair->attr_set, unpack_data->node_hash, unpack_data->now) == FALSE) {
-        return;
+    if (pe_evaluate_rules(pair->attr_set, unpack_data->node_hash,
+                          unpack_data->now, unpack_data->next_change)) {
+        add_versioned_attributes(pair->attr_set, unpack_data->hash);
     }
-
-    add_versioned_attributes(pair->attr_set, unpack_data->hash);
 }
 #endif
 
-static GListPtr
-make_pairs_and_populate_data(xmlNode * top, xmlNode * xml_obj, const char *set_name,
-                             GHashTable * node_hash, void * hash, const char *always_first,
-                             gboolean overwrite, crm_time_t * now, unpack_data_t * data)
+/*!
+ * \internal
+ * \brief Create a sorted list of nvpair blocks
+ *
+ * \param[in]  top           XML document root (used to expand id-ref's)
+ * \param[in]  xml_obj       XML element containing blocks of nvpair elements
+ * \param[in]  set_name      If not NULL, only get blocks of this element type
+ * \param[in]  always_first  If not NULL, sort block with this ID as first
+ *
+ * \return List of sorted_set_t entries for nvpair blocks
+ */
+static GList *
+make_pairs(xmlNode *top, xmlNode *xml_obj, const char *set_name,
+           const char *always_first)
 {
     GListPtr unsorted = NULL;
     const char *score = NULL;
@@ -902,50 +994,90 @@ make_pairs_and_populate_data(xmlNode * top, xmlNode * xml_obj, const char *set_n
             unsorted = g_list_prepend(unsorted, pair);
         }
     }
-
-    if (pair != NULL) {
-        data->hash = hash;
-        data->node_hash = node_hash;
-        data->now = now;
-        data->overwrite = overwrite;
-        data->top = top;
-    }
-
-    if (unsorted) {
-        return g_list_sort(unsorted, sort_pairs);
-    }
-
-    return NULL;
+    return g_list_sort(unsorted, sort_pairs);
 }
 
-void
-unpack_instance_attributes(xmlNode * top, xmlNode * xml_obj, const char *set_name,
-                           GHashTable * node_hash, GHashTable * hash, const char *always_first,
-                           gboolean overwrite, crm_time_t * now)
+/*!
+ * \internal
+ * \brief Extract nvpair blocks contained by an XML element into a hash table
+ *
+ * \param[in]  top           XML document root (used to expand id-ref's)
+ * \param[in]  xml_obj       XML element containing blocks of nvpair elements
+ * \param[in]  set_name      If not NULL, only use blocks of this element type
+ * \param[in]  node_hash     Node attributes to use when evaluating rules
+ * \param[out] hash          Where to store extracted name/value pairs
+ * \param[in]  always_first  If not NULL, process block with this ID first
+ * \param[in]  overwrite     Whether to replace existing values with same name
+ * \param[in]  now           Time to use when evaluating rules
+ * \param[out] next_change   If not NULL, set to when rule evaluation will change
+ * \param[in]  unpack_func   Function to call to unpack each block
+ */
+static void
+unpack_nvpair_blocks(xmlNode *top, xmlNode *xml_obj, const char *set_name,
+                     GHashTable *node_hash, void *hash,
+                     const char *always_first, gboolean overwrite,
+                     crm_time_t *now, crm_time_t *next_change,
+                     GFunc unpack_func)
 {
-    unpack_data_t data;
-    GListPtr pairs = make_pairs_and_populate_data(top, xml_obj, set_name, node_hash, hash,
-                                                  always_first, overwrite, now, &data);
+    GList *pairs = make_pairs(top, xml_obj, set_name, always_first);
 
     if (pairs) {
-        g_list_foreach(pairs, unpack_attr_set, &data);
+        unpack_data_t data = {
+            .hash = hash,
+            .node_hash = node_hash,
+            .now = now,
+            .overwrite = overwrite,
+            .next_change = next_change,
+            .top = top,
+        };
+
+        g_list_foreach(pairs, unpack_func, &data);
         g_list_free_full(pairs, free);
     }
 }
 
-#ifdef ENABLE_VERSIONED_ATTRS
+/*!
+ * \brief Extract nvpair blocks contained by an XML element into a hash table
+ *
+ * \param[in]  top           XML document root (used to expand id-ref's)
+ * \param[in]  xml_obj       XML element containing blocks of nvpair elements
+ * \param[in]  set_name      Element name to identify nvpair blocks
+ * \param[in]  node_hash     Node attributes to use when evaluating rules
+ * \param[out] hash          Where to store extracted name/value pairs
+ * \param[in]  always_first  If not NULL, process block with this ID first
+ * \param[in]  overwrite     Whether to replace existing values with same name
+ * \param[in]  now           Time to use when evaluating rules
+ * \param[out] next_change   If not NULL, set to when rule evaluation will change
+ */
 void
-pe_unpack_versioned_attributes(xmlNode * top, xmlNode * xml_obj, const char *set_name,
-                               GHashTable * node_hash, xmlNode * hash, crm_time_t * now)
+pe_unpack_nvpairs(xmlNode *top, xmlNode *xml_obj, const char *set_name,
+                  GHashTable *node_hash, GHashTable *hash,
+                  const char *always_first, gboolean overwrite,
+                  crm_time_t *now, crm_time_t *next_change)
 {
-    unpack_data_t data;
-    GListPtr pairs = make_pairs_and_populate_data(top, xml_obj, set_name, node_hash, hash,
-                                                  NULL, FALSE, now, &data);
+    unpack_nvpair_blocks(top, xml_obj, set_name, node_hash, hash, always_first,
+                         overwrite, now, next_change, unpack_attr_set);
+}
 
-    if (pairs) {
-        g_list_foreach(pairs, unpack_versioned_attr_set, &data);
-        g_list_free_full(pairs, free);
-    }
+void
+unpack_instance_attributes(xmlNode *top, xmlNode *xml_obj, const char *set_name,
+                           GHashTable *node_hash, GHashTable *hash,
+                           const char *always_first, gboolean overwrite,
+                           crm_time_t *now)
+{
+    unpack_nvpair_blocks(top, xml_obj, set_name, node_hash, hash, always_first,
+                         overwrite, now, NULL, unpack_attr_set);
+}
+
+#if ENABLE_VERSIONED_ATTRS
+void
+pe_unpack_versioned_attributes(xmlNode *top, xmlNode *xml_obj,
+                               const char *set_name, GHashTable *node_hash,
+                               xmlNode *hash, crm_time_t *now,
+                               crm_time_t *next_change)
+{
+    unpack_nvpair_blocks(top, xml_obj, set_name, node_hash, hash, NULL, FALSE,
+                         now, next_change, unpack_versioned_attr_set);
 }
 #endif
 
@@ -1006,7 +1138,7 @@ pe_expand_re_matches(const char *string, pe_re_match_data_t *match_data)
     return result;
 }
 
-#ifdef ENABLE_VERSIONED_ATTRS
+#if ENABLE_VERSIONED_ATTRS
 GHashTable*
 pe_unpack_versioned_parameters(xmlNode *versioned_params, const char *ra_version)
 {
@@ -1019,8 +1151,9 @@ pe_unpack_versioned_parameters(xmlNode *versioned_params, const char *ra_version
         if (attr_set) {
             g_hash_table_insert(node_hash, strdup(CRM_ATTR_RA_VERSION),
                                 strdup(ra_version));
-            unpack_instance_attributes(NULL, versioned_params, crm_element_name(attr_set),
-                                       node_hash, hash, NULL, FALSE, NULL);
+            pe_unpack_nvpairs(NULL, versioned_params,
+                              crm_element_name(attr_set), node_hash, hash, NULL,
+                              FALSE, NULL, NULL);
         }
 
         g_hash_table_destroy(node_hash);
