@@ -121,6 +121,13 @@ cluster_status(pe_working_set_t * data_set)
         unpack_status(cib_status, data_set);
     }
 
+    if (is_not_set(data_set->flags, pe_flag_no_counts)) {
+        for (GList *item = data_set->resources; item != NULL;
+             item = item->next) {
+            ((pe_resource_t *) (item->data))->fns->count(item->data);
+        }
+    }
+
     set_bit(data_set->flags, pe_flag_have_status);
     return TRUE;
 }
@@ -184,8 +191,8 @@ pe_free_nodes(GListPtr nodes)
         /* This is called after pe_free_resources(), which means that we can't
          * use node->details->uname for Pacemaker Remote nodes.
          */
-        crm_trace("Freeing node %s", (is_remote_node(node)?
-                  "(Pacemaker Remote)" : node->details->uname));
+        crm_trace("Freeing node %s", (pe__is_guest_or_remote_node(node)?
+                  "(guest or remote)" : node->details->uname));
 
         if (node->details->attrs != NULL) {
             g_hash_table_destroy(node->details->attrs);
@@ -354,6 +361,9 @@ set_working_set_defaults(pe_working_set_t * data_set)
     set_bit(data_set->flags, pe_flag_stop_rsc_orphans);
     set_bit(data_set->flags, pe_flag_symmetric_cluster);
     set_bit(data_set->flags, pe_flag_stop_action_orphans);
+#ifdef DEFAULT_CONCURRENT_FENCING_TRUE
+    set_bit(data_set->flags, pe_flag_concurrent_fencing);
+#endif
 }
 
 resource_t *
