@@ -12,9 +12,7 @@
 #endif
 
 #include <sys/types.h>
-#include <sys/stat.h>
 #include <stdio.h>
-
 #include <errno.h>
 #include <unistd.h>
 #include <dirent.h>
@@ -23,8 +21,10 @@
 #include <crm/crm.h>
 #include <crm/common/mainloop.h>
 #include <crm/services.h>
+#include <crm/stonith-ng.h>
 #include <crm/msg_xml.h>
 #include "services_private.h"
+#include "services_lsb.h"
 
 #if SUPPORT_UPSTART
 #  include <upstart.h>
@@ -48,32 +48,6 @@ static GList *inflight_ops = NULL;
 
 static void handle_blocked_ops(void);
 
-svc_action_t *
-services_action_create(const char *name, const char *action, int interval, int timeout)
-{
-    return resources_action_create(name, PCMK_RESOURCE_CLASS_LSB, NULL, name,
-                                   action, interval, timeout, NULL, 0);
-}
-
-static char *
-services__lsb_agent_path(const char *agent)
-{
-    return (*agent == '/')? strdup(agent)
-        : crm_strdup_printf("%s/%s", LSB_ROOT_DIR, agent);
-}
-
-static bool
-services__lsb_agent_exists(const char *agent)
-{
-    bool rc = FALSE;
-    struct stat st;
-    char *path = services__lsb_agent_path(agent);
-
-    rc = (stat(path, &st) == 0);
-    free(path);
-    return rc;
-}
-
 /*!
  * \brief Find first service class that can provide a specified agent
  *
@@ -88,11 +62,6 @@ services__lsb_agent_exists(const char *agent)
 const char *
 resources_find_service_class(const char *agent)
 {
-    /* Priority is:
-     * - lsb
-     * - systemd
-     * - upstart
-     */
     if (services__lsb_agent_exists(agent)) {
         return PCMK_RESOURCE_CLASS_LSB;
     }
@@ -171,10 +140,26 @@ expand_resource_class(const char *rsc, const char *standard, const char *agent)
     return expanded_class;
 }
 
+/*!
+ * \brief Duplicate a file path, inserting a prefix if not absolute
+ *
+ * \param[in] filename  File path to duplicate
+ * \param[in] dirname   If filename is not absolute, prefix to add
+ *
+ * \return Newly allocated memory with full path
+ */
+static char *
+dup_file_path(const char *filename, const char *dirname)
+{
+    return (*filename == '/')? strdup(filename)
+           : crm_strdup_printf("%s/%s", dirname, filename);
+}
+
 svc_action_t *
-resources_action_create(const char *name, const char *standard, const char *provider,
-                        const char *agent, const char *action, int interval, int timeout,
-                        GHashTable * params, enum svc_action_flags flags)
+resources_action_create(const char *name, const char *standard,
+                        const char *provider, const char *agent,
+                        const char *action, guint interval_ms, int timeout,
+                        GHashTable *params, enum svc_action_flags flags)
 {
     svc_action_t *op = NULL;
     uint32_t ra_caps = 0;
@@ -217,13 +202,13 @@ resources_action_create(const char *name, const char *standard, const char *prov
     op = calloc(1, sizeof(svc_action_t));
     op->opaque = calloc(1, sizeof(svc_action_private_t));
     op->rsc = strdup(name);
-    op->interval = interval;
+    op->interval_ms = interval_ms;
     op->timeout = timeout;
     op->standard = expand_resource_class(name, standard, agent);
     op->agent = strdup(agent);
     op->sequence = ++operations;
     op->flags = flags;
-    op->id = generate_op_key(name, action, interval);
+    op->id = generate_op_key(name, action, interval_ms);
 
     if (is_set(ra_caps, pcmk_ra_cap_status) && safe_str_eq(action, "monitor")) {
         op->action = strdup("status");
@@ -241,10 +226,8 @@ resources_action_create(const char *name, const char *standard, const char *prov
     }
 
     if (strcasecmp(op->standard, PCMK_RESOURCE_CLASS_OCF) == 0) {
-        if (asprintf(&op->opaque->exec, "%s/resource.d/%s/%s", OCF_ROOT_DIR, provider, agent) == -1) {
-            crm_err("Internal error: cannot create agent path");
-            goto return_error;
-        }
+        op->opaque->exec = crm_strdup_printf("%s/resource.d/%s/%s",
+                                             OCF_ROOT_DIR, provider, agent);
         op->opaque->args[0] = strdup(op->opaque->exec);
         op->opaque->args[1] = strdup(op->action);
 
@@ -253,48 +236,6 @@ resources_action_create(const char *name, const char *standard, const char *prov
         op->opaque->args[0] = strdup(op->opaque->exec);
         op->opaque->args[1] = strdup(op->action);
 
-#if SUPPORT_HEARTBEAT
-    } else if (strcasecmp(op->standard, PCMK_RESOURCE_CLASS_HB) == 0) {
-        int index;
-        int param_num;
-        char buf_tmp[20];
-        void *value_tmp;
-
-        if (op->agent[0] == '/') {
-            /* if given an absolute path, use that instead
-             * of tacking on the HB_RA_DIR path to the front */
-            op->opaque->exec = strdup(op->agent);
-        } else if (asprintf(&op->opaque->exec, "%s/%s", HB_RA_DIR, op->agent) == -1) {
-            crm_err("Internal error: cannot create agent path");
-            goto return_error;
-        }
-        op->opaque->args[0] = strdup(op->opaque->exec);
-
-        /* The "heartbeat" agent class only has positional arguments,
-         * which we keyed by their decimal position number. */
-        param_num = 1;
-        if (op->params) {
-            for (index = 1; index <= MAX_ARGC - 3; index++ ) {
-                snprintf(buf_tmp, sizeof(buf_tmp), "%d", index);
-                value_tmp = g_hash_table_lookup(op->params, buf_tmp);
-                if (value_tmp == NULL) {
-                    /* maybe: strdup("") ??
-                     * But the old lrmd did simply continue as well. */
-                    continue;
-                }
-                op->opaque->args[param_num++] = strdup(value_tmp);
-            }
-
-            // Heartbeat actions don't need to keep the parameters
-            g_hash_table_destroy(op->params);
-            op->params = NULL;
-        }
-
-        /* Add operation code as the last argument, */
-        /* and the terminating NULL pointer */
-        op->opaque->args[param_num++] = strdup(op->action);
-        op->opaque->args[param_num] = NULL;
-#endif
 #if SUPPORT_SYSTEMD
     } else if (strcasecmp(op->standard, PCMK_RESOURCE_CLASS_SYSTEMD) == 0) {
         op->opaque->exec = strdup("systemd-dbus");
@@ -305,19 +246,10 @@ resources_action_create(const char *name, const char *standard, const char *prov
 #endif
 #if SUPPORT_NAGIOS
     } else if (strcasecmp(op->standard, PCMK_RESOURCE_CLASS_NAGIOS) == 0) {
-        if (op->agent[0] == '/') {
-            /* if given an absolute path, use that instead
-             * of tacking on the NAGIOS_PLUGIN_DIR path to the front */
-            op->opaque->exec = strdup(op->agent);
-
-        } else if (asprintf(&op->opaque->exec, "%s/%s", NAGIOS_PLUGIN_DIR, op->agent) == -1) {
-            crm_err("Internal error: cannot create agent path");
-            goto return_error;
-        }
-
+        op->opaque->exec = dup_file_path(op->agent, NAGIOS_PLUGIN_DIR);
         op->opaque->args[0] = strdup(op->opaque->exec);
 
-        if (safe_str_eq(op->action, "monitor") && op->interval == 0) {
+        if (safe_str_eq(op->action, "monitor") && (op->interval_ms == 0)) {
             /* Invoke --version for a nagios probe */
             op->opaque->args[1] = strdup("--version");
 
@@ -332,21 +264,12 @@ resources_action_create(const char *name, const char *standard, const char *prov
 
             while (g_hash_table_iter_next(&iter, (gpointer *) & key, (gpointer *) & value) &&
                    index <= args_size - 3) {
-                int len = 3;
-                char *long_opt = NULL;
 
                 if (safe_str_eq(key, XML_ATTR_CRM_VERSION) || strstr(key, CRM_META "_")) {
                     continue;
                 }
-
-                len += strlen(key);
-                long_opt = calloc(1, len);
-                sprintf(long_opt, "--%s", key);
-                long_opt[len - 1] = 0;
-
-                op->opaque->args[index] = long_opt;
-                op->opaque->args[index + 1] = strdup(value);
-                index += 2;
+                op->opaque->args[index++] = crm_strdup_printf("--%s", key);
+                op->opaque->args[index++] = strdup(value);
             }
         }
 
@@ -608,17 +531,17 @@ cancel_recurring_action(svc_action_t * op)
 /*!
  * \brief Cancel a recurring action
  *
- * \param[in] name      Name of resource that operation is for
- * \param[in] action    Name of operation to cancel
- * \param[in] interval  Interval of operation to cancel
+ * \param[in] name         Name of resource that operation is for
+ * \param[in] action       Name of operation to cancel
+ * \param[in] interval_ms  Interval of operation to cancel
  *
  * \return TRUE if action was successfully cancelled, FALSE otherwise
  */
 gboolean
-services_action_cancel(const char *name, const char *action, int interval)
+services_action_cancel(const char *name, const char *action, guint interval_ms)
 {
     gboolean cancelled = FALSE;
-    char *id = generate_op_key(name, action, interval);
+    char *id = generate_op_key(name, action, interval_ms);
     svc_action_t *op = NULL;
 
     /* We can only cancel a recurring action */
@@ -682,10 +605,10 @@ done:
 }
 
 gboolean
-services_action_kick(const char *name, const char *action, int interval /* ms */)
+services_action_kick(const char *name, const char *action, guint interval_ms)
 {
     svc_action_t * op = NULL;
-    char *id = generate_op_key(name, action, interval);
+    char *id = generate_op_key(name, action, interval_ms);
 
     init_recurring_actions();
     op = g_hash_table_lookup(recurring_actions, id);
@@ -817,7 +740,7 @@ services_action_async_fork_notify(svc_action_t * op,
         op->opaque->fork_callback = action_fork_callback;
     }
 
-    if (op->interval > 0) {
+    if (op->interval_ms > 0) {
         init_recurring_actions();
         if (handle_duplicate_recurring(op) == TRUE) {
             /* entry rescheduled, dup freed */
@@ -902,225 +825,6 @@ handle_blocked_ops(void)
     processing_blocked_ops = FALSE;
 }
 
-#define lsb_metadata_template  \
-    "<?xml version='1.0'?>\n"                                           \
-    "<!DOCTYPE resource-agent SYSTEM 'ra-api-1.dtd'>\n"                 \
-    "<resource-agent name='%s' version='" PCMK_DEFAULT_AGENT_VERSION "'>\n" \
-    "  <version>1.0</version>\n"                                        \
-    "  <longdesc lang='en'>\n"                                          \
-    "%s"                                                                \
-    "  </longdesc>\n"                                                   \
-    "  <shortdesc lang='en'>%s</shortdesc>\n"                           \
-    "  <parameters>\n"                                                  \
-    "  </parameters>\n"                                                 \
-    "  <actions>\n"                                                     \
-    "    <action name='meta-data'    timeout='5' />\n"                  \
-    "    <action name='start'        timeout='15' />\n"                 \
-    "    <action name='stop'         timeout='15' />\n"                 \
-    "    <action name='status'       timeout='15' />\n"                 \
-    "    <action name='restart'      timeout='15' />\n"                 \
-    "    <action name='force-reload' timeout='15' />\n"                 \
-    "    <action name='monitor'      timeout='15' interval='15' />\n"   \
-    "  </actions>\n"                                                    \
-    "  <special tag='LSB'>\n"                                           \
-    "    <Provides>%s</Provides>\n"                                     \
-    "    <Required-Start>%s</Required-Start>\n"                         \
-    "    <Required-Stop>%s</Required-Stop>\n"                           \
-    "    <Should-Start>%s</Should-Start>\n"                             \
-    "    <Should-Stop>%s</Should-Stop>\n"                               \
-    "    <Default-Start>%s</Default-Start>\n"                           \
-    "    <Default-Stop>%s</Default-Stop>\n"                             \
-    "  </special>\n"                                                    \
-    "</resource-agent>\n"
-
-/* See "Comment Conventions for Init Scripts" in the LSB core specification at:
- * http://refspecs.linuxfoundation.org/lsb.shtml
- */
-#define LSB_INITSCRIPT_INFOBEGIN_TAG "### BEGIN INIT INFO"
-#define LSB_INITSCRIPT_INFOEND_TAG "### END INIT INFO"
-#define PROVIDES    "# Provides:"
-#define REQ_START   "# Required-Start:"
-#define REQ_STOP    "# Required-Stop:"
-#define SHLD_START  "# Should-Start:"
-#define SHLD_STOP   "# Should-Stop:"
-#define DFLT_START  "# Default-Start:"
-#define DFLT_STOP   "# Default-Stop:"
-#define SHORT_DSCR  "# Short-Description:"
-#define DESCRIPTION "# Description:"
-
-#define lsb_meta_helper_free_value(m)           \
-    do {                                        \
-        if ((m) != NULL) {                      \
-            xmlFree(m);                         \
-            (m) = NULL;                         \
-        }                                       \
-    } while(0)
-
-/*!
- * \internal
- * \brief Grab an LSB header value
- *
- * \param[in]     line    Line read from LSB init script
- * \param[in,out] value   If not set, will be set to XML-safe copy of value
- * \param[in]     prefix  Set value if line starts with this pattern
- *
- * \return TRUE if value was set, FALSE otherwise
- */
-static inline gboolean
-lsb_meta_helper_get_value(const char *line, char **value, const char *prefix)
-{
-    if (!*value && crm_starts_with(line, prefix)) {
-        *value = (char *)xmlEncodeEntitiesReentrant(NULL, BAD_CAST line+strlen(prefix));
-        return TRUE;
-    }
-    return FALSE;
-}
-
-#define DESC_MAX 2048
-
-static int
-lsb_get_metadata(const char *type, char **output)
-{
-    char ra_pathname[PATH_MAX] = { 0, };
-    FILE *fp = NULL;
-    char buffer[1024] = { 0, };
-    char *provides = NULL;
-    char *req_start = NULL;
-    char *req_stop = NULL;
-    char *shld_start = NULL;
-    char *shld_stop = NULL;
-    char *dflt_start = NULL;
-    char *dflt_stop = NULL;
-    char *s_dscrpt = NULL;
-    char *xml_l_dscrpt = NULL;
-    int offset = 0;
-    bool in_header = FALSE;
-    char description[DESC_MAX] = { 0, };
-
-    if (type[0] == '/') {
-        snprintf(ra_pathname, sizeof(ra_pathname), "%s", type);
-    } else {
-        snprintf(ra_pathname, sizeof(ra_pathname), "%s/%s",
-                 LSB_ROOT_DIR, type);
-    }
-
-    crm_trace("Looking into %s", ra_pathname);
-    fp = fopen(ra_pathname, "r");
-    if (fp == NULL) {
-        return -errno;
-    }
-
-    /* Enter into the LSB-compliant comment block */
-    while (fgets(buffer, sizeof(buffer), fp)) {
-
-        // Ignore lines up to and including the block delimiter
-        if (crm_starts_with(buffer, LSB_INITSCRIPT_INFOBEGIN_TAG)) {
-            in_header = TRUE;
-            continue;
-        }
-        if (!in_header) {
-            continue;
-        }
-
-        /* Assume each of the following eight arguments contain one line */
-        if (lsb_meta_helper_get_value(buffer, &provides, PROVIDES)) {
-            continue;
-        }
-        if (lsb_meta_helper_get_value(buffer, &req_start, REQ_START)) {
-            continue;
-        }
-        if (lsb_meta_helper_get_value(buffer, &req_stop, REQ_STOP)) {
-            continue;
-        }
-        if (lsb_meta_helper_get_value(buffer, &shld_start, SHLD_START)) {
-            continue;
-        }
-        if (lsb_meta_helper_get_value(buffer, &shld_stop, SHLD_STOP)) {
-            continue;
-        }
-        if (lsb_meta_helper_get_value(buffer, &dflt_start, DFLT_START)) {
-            continue;
-        }
-        if (lsb_meta_helper_get_value(buffer, &dflt_stop, DFLT_STOP)) {
-            continue;
-        }
-        if (lsb_meta_helper_get_value(buffer, &s_dscrpt, SHORT_DSCR)) {
-            continue;
-        }
-
-        /* Long description may cross multiple lines */
-        if ((offset == 0) // haven't already found long description
-            && crm_starts_with(buffer, DESCRIPTION)) {
-            bool processed_line = TRUE;
-
-            // Get remainder of description line itself
-            offset += snprintf(description, DESC_MAX, "%s",
-                               buffer + strlen(DESCRIPTION));
-
-            // Read any continuation lines of the description
-            buffer[0] = '\0';
-            while (fgets(buffer, sizeof(buffer), fp)) {
-                if (crm_starts_with(buffer, "#  ")
-                    || crm_starts_with(buffer, "#\t")) {
-                    /* '#' followed by a tab or more than one space indicates a
-                     * continuation of the long description.
-                     */
-                    offset += snprintf(description + offset, DESC_MAX - offset,
-                                       "%s", buffer + 1);
-                } else {
-                    /* This line is not part of the long description,
-                     * so continue with normal processing.
-                     */
-                    processed_line = FALSE;
-                    break;
-                }
-            }
-
-            // Make long description safe to use in XML
-            xml_l_dscrpt = (char *)xmlEncodeEntitiesReentrant(NULL, BAD_CAST(description));
-
-            if (processed_line) {
-                // We grabbed the line into the long description
-                continue;
-            }
-        }
-
-        // Stop if we leave the header block
-        if (crm_starts_with(buffer, LSB_INITSCRIPT_INFOEND_TAG)) {
-            break;
-        }
-        if (buffer[0] != '#') {
-            break;
-        }
-    }
-    fclose(fp);
-
-    *output = crm_strdup_printf(lsb_metadata_template, type,
-                                (xml_l_dscrpt? xml_l_dscrpt : type),
-                                (s_dscrpt? s_dscrpt : type),
-                                (provides? provides : ""),
-                                (req_start? req_start : ""),
-                                (req_stop? req_stop : ""),
-                                (shld_start? shld_start : ""),
-                                (shld_stop? shld_stop : ""),
-                                (dflt_start? dflt_start : ""),
-                                (dflt_stop? dflt_stop : ""));
-
-    lsb_meta_helper_free_value(xml_l_dscrpt);
-    lsb_meta_helper_free_value(s_dscrpt);
-    lsb_meta_helper_free_value(provides);
-    lsb_meta_helper_free_value(req_start);
-    lsb_meta_helper_free_value(req_stop);
-    lsb_meta_helper_free_value(shld_start);
-    lsb_meta_helper_free_value(shld_stop);
-    lsb_meta_helper_free_value(dflt_start);
-    lsb_meta_helper_free_value(dflt_stop);
-
-    crm_trace("Created fake metadata: %llu",
-              (unsigned long long) strlen(*output));
-    return pcmk_ok;
-}
-
 #if SUPPORT_NAGIOS
 static int
 nagios_get_metadata(const char *type, char **output)
@@ -1172,94 +876,6 @@ nagios_get_metadata(const char *type, char **output)
 }
 #endif
 
-#if SUPPORT_HEARTBEAT
-/* strictly speaking, support for class=heartbeat style scripts
- * does not require "heartbeat support" to be enabled.
- * But since those scripts are part of the "heartbeat" package usually,
- * and are very unlikely to be present in any other deployment,
- * I leave it inside this ifdef.
- *
- * Yes, I know, these are legacy and should die,
- * or at least be rewritten to be a proper OCF style agent.
- * But they exist, and custom scripts following these rules do, too.
- *
- * Taken from the old "glue" lrmd, see
- * http://hg.linux-ha.org/glue/file/0a7add1d9996/lib/plugins/lrm/raexechb.c#l49
- * http://hg.linux-ha.org/glue/file/0a7add1d9996/lib/plugins/lrm/raexechb.c#l393
- */
-
-static const char hb_metadata_template[] =
-    "<?xml version='1.0'?>\n"
-    "<!DOCTYPE resource-agent SYSTEM 'ra-api-1.dtd'>\n"
-    "<resource-agent name='%s' version='" PCMK_DEFAULT_AGENT_VERSION "'>\n"
-    "<version>1.0</version>\n"
-    "<longdesc lang='en'>\n"
-    "%s"
-    "</longdesc>\n"
-    "<shortdesc lang='en'>%s</shortdesc>\n"
-    "<parameters>\n"
-    "<parameter name='1' unique='1' required='0'>\n"
-    "<longdesc lang='en'>\n"
-    "This argument will be passed as the first argument to the "
-    "heartbeat resource agent (assuming it supports one)\n"
-    "</longdesc>\n"
-    "<shortdesc lang='en'>argv[1]</shortdesc>\n"
-    "<content type='string' default=' ' />\n"
-    "</parameter>\n"
-    "<parameter name='2' unique='1' required='0'>\n"
-    "<longdesc lang='en'>\n"
-    "This argument will be passed as the second argument to the "
-    "heartbeat resource agent (assuming it supports one)\n"
-    "</longdesc>\n"
-    "<shortdesc lang='en'>argv[2]</shortdesc>\n"
-    "<content type='string' default=' ' />\n"
-    "</parameter>\n"
-    "<parameter name='3' unique='1' required='0'>\n"
-    "<longdesc lang='en'>\n"
-    "This argument will be passed as the third argument to the "
-    "heartbeat resource agent (assuming it supports one)\n"
-    "</longdesc>\n"
-    "<shortdesc lang='en'>argv[3]</shortdesc>\n"
-    "<content type='string' default=' ' />\n"
-    "</parameter>\n"
-    "<parameter name='4' unique='1' required='0'>\n"
-    "<longdesc lang='en'>\n"
-    "This argument will be passed as the fourth argument to the "
-    "heartbeat resource agent (assuming it supports one)\n"
-    "</longdesc>\n"
-    "<shortdesc lang='en'>argv[4]</shortdesc>\n"
-    "<content type='string' default=' ' />\n"
-    "</parameter>\n"
-    "<parameter name='5' unique='1' required='0'>\n"
-    "<longdesc lang='en'>\n"
-    "This argument will be passed as the fifth argument to the "
-    "heartbeat resource agent (assuming it supports one)\n"
-    "</longdesc>\n"
-    "<shortdesc lang='en'>argv[5]</shortdesc>\n"
-    "<content type='string' default=' ' />\n"
-    "</parameter>\n"
-    "</parameters>\n"
-    "<actions>\n"
-    "<action name='start'   timeout='15' />\n"
-    "<action name='stop'    timeout='15' />\n"
-    "<action name='status'  timeout='15' />\n"
-    "<action name='monitor' timeout='15' interval='15' start-delay='15' />\n"
-    "<action name='meta-data'  timeout='5' />\n"
-    "</actions>\n"
-    "<special tag='heartbeat'>\n"
-    "</special>\n"
-    "</resource-agent>\n";
-
-static int
-heartbeat_get_metadata(const char *type, char **output)
-{
-    *output = crm_strdup_printf(hb_metadata_template, type, type, type);
-    crm_trace("Created fake metadata: %llu",
-              (unsigned long long) strlen(*output));
-    return pcmk_ok;
-}
-#endif
-
 static gboolean
 action_get_metadata(svc_action_t *op)
 {
@@ -1287,7 +903,7 @@ action_get_metadata(svc_action_t *op)
     }
 
     if (safe_str_eq(class, PCMK_RESOURCE_CLASS_LSB)) {
-        return (lsb_get_metadata(op->agent, &op->stdout_data) >= 0);
+        return (services__get_lsb_metadata(op->agent, &op->stdout_data) >= 0);
     }
 
 #if SUPPORT_NAGIOS
@@ -1296,12 +912,6 @@ action_get_metadata(svc_action_t *op)
     }
 #endif
 
-#if SUPPORT_HEARTBEAT
-    if (safe_str_eq(class, PCMK_RESOURCE_CLASS_HB)) {
-        return (heartbeat_get_metadata(op->agent, &op->stdout_data) >= 0);
-    }
-#endif
-
     return action_exec_helper(op);
 }
 
@@ -1329,8 +939,8 @@ services_action_sync(svc_action_t * op)
     } else {
         rc = action_exec_helper(op);
     }
-    crm_trace(" > %s_%s_%d: %s = %d",
-              op->rsc, op->action, op->interval, op->opaque->exec, op->rc);
+    crm_trace(" > " CRM_OP_FMT ": %s = %d",
+              op->rsc, op->action, op->interval_ms, op->opaque->exec, op->rc);
     if (op->stdout_data) {
         crm_trace(" >  stdout: %s", op->stdout_data);
     }
@@ -1346,20 +956,6 @@ get_directory_list(const char *root, gboolean files, gboolean executable)
     return services_os_get_directory_list(root, files, executable);
 }
 
-GList *
-services_list(void)
-{
-    return resources_list_agents(PCMK_RESOURCE_CLASS_LSB, NULL);
-}
-
-#if SUPPORT_HEARTBEAT
-static GList *
-resources_os_list_hb_agents(void)
-{
-    return services_os_get_directory_list(HB_RA_DIR, TRUE, TRUE);
-}
-#endif
-
 GList *
 resources_list_standards(void)
 {
@@ -1397,10 +993,6 @@ resources_list_standards(void)
     }
 #endif
 
-#if SUPPORT_HEARTBEAT
-    standards = g_list_append(standards, strdup(PCMK_RESOURCE_CLASS_HB));
-#endif
-
     return standards;
 }
 
@@ -1422,7 +1014,7 @@ resources_list_agents(const char *standard, const char *provider)
 
         GList *tmp1;
         GList *tmp2;
-        GList *result = resources_os_list_lsb_agents();
+        GList *result = services__list_lsb_agents();
 
         if (standard == NULL) {
             tmp1 = result;
@@ -1452,11 +1044,7 @@ resources_list_agents(const char *standard, const char *provider)
     } else if (strcasecmp(standard, PCMK_RESOURCE_CLASS_OCF) == 0) {
         return resources_os_list_ocf_agents(provider);
     } else if (strcasecmp(standard, PCMK_RESOURCE_CLASS_LSB) == 0) {
-        return resources_os_list_lsb_agents();
-#if SUPPORT_HEARTBEAT
-    } else if (strcasecmp(standard, PCMK_RESOURCE_CLASS_HB) == 0) {
-        return resources_os_list_hb_agents();
-#endif
+        return services__list_lsb_agents();
 #if SUPPORT_SYSTEMD
     } else if (strcasecmp(standard, PCMK_RESOURCE_CLASS_SYSTEMD) == 0) {
         return systemd_unit_listall();
@@ -1473,3 +1061,90 @@ resources_list_agents(const char *standard, const char *provider)
 
     return NULL;
 }
+
+gboolean
+resources_agent_exists(const char *standard, const char *provider, const char *agent)
+{
+    GList *standards = NULL;
+    GList *providers = NULL;
+    GListPtr iter = NULL;
+    gboolean rc = FALSE;
+    gboolean has_providers = FALSE;
+
+    standards = resources_list_standards();
+    for (iter = standards; iter != NULL; iter = iter->next) {
+        if (crm_str_eq(iter->data, standard, TRUE)) {
+            rc = TRUE;
+            break;
+        }
+    }
+
+    if (rc == FALSE) {
+        goto done;
+    }
+
+    rc = FALSE;
+
+    has_providers = is_set(pcmk_get_ra_caps(standard), pcmk_ra_cap_provider);
+    if (has_providers == TRUE && provider != NULL) {
+        providers = resources_list_providers(standard);
+        for (iter = providers; iter != NULL; iter = iter->next) {
+            if (crm_str_eq(iter->data, provider, TRUE)) {
+                rc = TRUE;
+                break;
+            }
+        }
+    } else if (has_providers == FALSE && provider == NULL) {
+        rc = TRUE;
+    }
+
+    if (rc == FALSE) {
+        goto done;
+    }
+
+    if (safe_str_eq(standard, PCMK_RESOURCE_CLASS_SERVICE)) {
+        if (services__lsb_agent_exists(agent)) {
+            rc = TRUE;
+#if SUPPORT_SYSTEMD
+        } else if (systemd_unit_exists(agent)) {
+            rc = TRUE;
+#endif
+
+#if SUPPORT_UPSTART
+        } else if (upstart_job_exists(agent)) {
+            rc = TRUE;
+#endif
+        } else {
+            rc = FALSE;
+        }
+
+    } else if (safe_str_eq(standard, PCMK_RESOURCE_CLASS_OCF)) {
+        rc = services__ocf_agent_exists(provider, agent);
+
+    } else if (safe_str_eq(standard, PCMK_RESOURCE_CLASS_LSB)) {
+        rc = services__lsb_agent_exists(agent);
+
+#if SUPPORT_SYSTEMD
+    } else if (safe_str_eq(standard, PCMK_RESOURCE_CLASS_SYSTEMD)) {
+        rc = systemd_unit_exists(agent);
+#endif
+
+#if SUPPORT_UPSTART
+    } else if (safe_str_eq(standard, PCMK_RESOURCE_CLASS_UPSTART)) {
+        rc = upstart_job_exists(agent);
+#endif
+
+#if SUPPORT_NAGIOS
+    } else if (safe_str_eq(standard, PCMK_RESOURCE_CLASS_NAGIOS)) {
+        rc = services__nagios_agent_exists(agent);
+#endif
+
+    } else {
+        rc = FALSE;
+    }
+
+done:
+    g_list_free(standards);
+    g_list_free(providers);
+    return rc;
+}
