@@ -160,6 +160,25 @@ set_ocf_env_with_prefix(gpointer key, gpointer value, gpointer user_data)
     set_ocf_env(buffer, value, user_data);
 }
 
+static void
+set_alert_env(gpointer key, gpointer value, gpointer user_data)
+{
+    int rc;
+
+    if (value != NULL) {
+        rc = setenv(key, value, 1);
+    } else {
+        rc = unsetenv(key);
+    }
+
+    if (rc < 0) {
+        crm_perror(LOG_ERR, "setenv %s=%s",
+                  (char*)key, (value? (char*)value : ""));
+    } else {
+        crm_trace("setenv %s=%s", (char*)key, (value? (char*)value : ""));
+    }
+}
+
 /*!
  * \internal
  * \brief Add environment variables suitable for an action
@@ -169,12 +188,20 @@ set_ocf_env_with_prefix(gpointer key, gpointer value, gpointer user_data)
 static void
 add_action_env_vars(const svc_action_t *op)
 {
-    if (safe_str_eq(op->standard, PCMK_RESOURCE_CLASS_OCF) == FALSE) {
-        return;
+    void (*env_setter)(gpointer, gpointer, gpointer) = NULL;
+    if (op->agent == NULL) {
+        env_setter = set_alert_env;  /* we deal with alert handler */
+
+    } else if (safe_str_eq(op->standard, PCMK_RESOURCE_CLASS_OCF)) {
+        env_setter = set_ocf_env_with_prefix;
     }
 
-    if (op->params) {
-        g_hash_table_foreach(op->params, set_ocf_env_with_prefix, NULL);
+    if (env_setter != NULL && op->params != NULL) {
+        g_hash_table_foreach(op->params, env_setter, NULL);
+    }
+
+    if (env_setter == NULL || env_setter == set_alert_env) {
+        return;
     }
 
     set_ocf_env("OCF_RA_VERSION_MAJOR", "1", NULL);
@@ -196,6 +223,39 @@ add_action_env_vars(const svc_action_t *op)
     }
 }
 
+static void
+pipe_in_single_parameter(gpointer key, gpointer value, gpointer user_data)
+{
+    svc_action_t *op = user_data;
+    char *buffer = crm_strdup_printf("%s=%s\n", (char *)key, (char *) value);
+    int ret, total = 0, len = strlen(buffer);
+
+    do {
+        errno = 0;
+        ret = write(op->opaque->stdin_fd, buffer + total, len - total);
+        if (ret > 0) {
+            total += ret;
+        }
+
+    } while ((errno == EINTR) && (total < len));
+    free(buffer);
+}
+
+/*!
+ * \internal
+ * \brief Pipe parameters in via stdin for action
+ *
+ * \param[in] op  Action to use
+ */
+static void
+pipe_in_action_stdin_parameters(const svc_action_t *op)
+{
+    crm_debug("sending args");
+    if (op->params) {
+        g_hash_table_foreach(op->params, pipe_in_single_parameter, (gpointer) op);
+    }
+}
+
 gboolean
 recurring_action_timer(gpointer data)
 {
@@ -285,6 +345,10 @@ operation_finished(mainloop_child_t * p, pid_t pid, int core, int signo, int exi
         op->opaque->stdout_gsource = NULL;
     }
 
+    if (op->opaque->stdin_fd >= 0) {
+        close(op->opaque->stdin_fd);
+    }
+
     if (signo) {
         if (mainloop_child_timeout(p)) {
             crm_warn("%s - timed out after %dms", prefix, op->timeout);
@@ -380,8 +444,6 @@ services_handle_exec_error(svc_action_t * op, int error)
 static void
 action_launch_child(svc_action_t *op)
 {
-    int lpc;
-
     /* SIGPIPE is ignored (which is different from signal blocking) by the gnutls library.
      * Depending on the libqb version in use, libqb may set SIGPIPE to be ignored as well. 
      * We do not want this to be inherited by the child process. By resetting this the signal
@@ -411,10 +473,7 @@ action_launch_child(svc_action_t *op)
      */
     setpgid(0, 0);
 
-    // Close all file descriptors except stdin/stdout/stderr
-    for (lpc = getdtablesize() - 1; lpc > STDERR_FILENO; lpc--) {
-        close(lpc);
-    }
+    pcmk__close_fds_in_child(false);
 
 #if SUPPORT_CIBSECRETS
     if (replace_secret_params(op->rsc, op->params) < 0) {
@@ -488,9 +547,12 @@ action_synced_wait(svc_action_t * op, sigset_t *mask)
     int wait_rc = 0;
 
 #ifdef HAVE_SYS_SIGNALFD_H
-    sfd = signalfd(-1, mask, SFD_NONBLOCK);
-    if (sfd < 0) {
-        crm_perror(LOG_ERR, "signalfd() failed");
+    // mask is always non-NULL in practice, but this makes static analysis happy
+    if (mask) {
+        sfd = signalfd(-1, mask, SFD_NONBLOCK);
+        if (sfd < 0) {
+            crm_perror(LOG_ERR, "signalfd() failed");
+        }
     }
 #else
     sfd = sigchld_pipe[0];
@@ -616,6 +678,9 @@ action_synced_wait(svc_action_t * op, sigset_t *mask)
 
     close(op->opaque->stdout_fd);
     close(op->opaque->stderr_fd);
+    if (op->opaque->stdin_fd >= 0) {
+        close(op->opaque->stdin_fd);
+    }
 
 #ifdef HAVE_SYS_SIGNALFD_H
     close(sfd);
@@ -629,9 +694,10 @@ services_os_action_execute(svc_action_t * op)
 {
     int stdout_fd[2];
     int stderr_fd[2];
+    int stdin_fd[2] = {-1, -1};
     int rc;
     struct stat st;
-    sigset_t *pmask;
+    sigset_t *pmask = NULL;
 
 #ifdef HAVE_SYS_SIGNALFD_H
     sigset_t mask;
@@ -694,6 +760,25 @@ services_os_action_execute(svc_action_t * op)
         return FALSE;
     }
 
+    if (safe_str_eq(op->standard, PCMK_RESOURCE_CLASS_STONITH)) {
+        if (pipe(stdin_fd) < 0) {
+            rc = errno;
+
+            close(stdout_fd[0]);
+            close(stdout_fd[1]);
+            close(stderr_fd[0]);
+            close(stderr_fd[1]);
+
+            crm_err("pipe(stdin_fd) failed. '%s': %s (%d)", op->opaque->exec, pcmk_strerror(rc), rc);
+
+            services_handle_exec_error(op, rc);
+            if (!op->synchronous) {
+                return operation_finalize(op);
+            }
+            return FALSE;
+        }
+    }
+
     if (op->synchronous) {
 #ifdef HAVE_SYS_SIGNALFD_H
         sigemptyset(&mask);
@@ -741,6 +826,10 @@ services_os_action_execute(svc_action_t * op)
             close(stdout_fd[1]);
             close(stderr_fd[0]);
             close(stderr_fd[1]);
+            if (stdin_fd[0] >= 0) {
+                close(stdin_fd[0]);
+                close(stdin_fd[1]);
+            }
 
             crm_err("Could not execute '%s': %s (%d)", op->opaque->exec, pcmk_strerror(rc), rc);
             services_handle_exec_error(op, rc);
@@ -754,6 +843,9 @@ services_os_action_execute(svc_action_t * op)
         case 0:                /* Child */
             close(stdout_fd[0]);
             close(stderr_fd[0]);
+            if (stdin_fd[1] >= 0) {
+                close(stdin_fd[1]);
+            }
             if (STDOUT_FILENO != stdout_fd[1]) {
                 if (dup2(stdout_fd[1], STDOUT_FILENO) != STDOUT_FILENO) {
                     crm_err("dup2() failed (stdout)");
@@ -766,6 +858,13 @@ services_os_action_execute(svc_action_t * op)
                 }
                 close(stderr_fd[1]);
             }
+            if ((stdin_fd[0] >= 0) &&
+                (STDIN_FILENO != stdin_fd[0])) {
+                if (dup2(stdin_fd[0], STDIN_FILENO) != STDIN_FILENO) {
+                    crm_err("dup2() failed (stdin)");
+                }
+                close(stdin_fd[0]);
+            }
 
             if (op->synchronous) {
                 sigchld_cleanup();
@@ -778,6 +877,9 @@ services_os_action_execute(svc_action_t * op)
     /* Only the parent reaches here */
     close(stdout_fd[1]);
     close(stderr_fd[1]);
+    if (stdin_fd[0] >= 0) {
+        close(stdin_fd[0]);
+    }
 
     op->opaque->stdout_fd = stdout_fd[0];
     rc = crm_set_nonblocking(op->opaque->stdout_fd);
@@ -795,6 +897,27 @@ services_os_action_execute(svc_action_t * op)
                  pcmk_strerror(rc), rc);
     }
 
+    op->opaque->stdin_fd = stdin_fd[1];
+    if (op->opaque->stdin_fd >= 0) {
+        // using buffer behind non-blocking-fd here - that could be improved
+        // as long as no other standard uses stdin_fd assume stonith
+        rc = crm_set_nonblocking(op->opaque->stdin_fd);
+        if (rc < 0) {
+            crm_warn("Could not set child input non-blocking: %s "
+                    CRM_XS " fd=%d,rc=%d",
+                    pcmk_strerror(rc), op->opaque->stdin_fd, rc);
+        }
+        pipe_in_action_stdin_parameters(op);
+        // as long as we are handling parameters directly in here just close
+        close(op->opaque->stdin_fd);
+        op->opaque->stdin_fd = -1;
+    }
+
+    // after fds are setup properly and before we plug anything into mainloop
+    if (op->opaque->fork_callback) {
+        op->opaque->fork_callback(op);
+    }
+
     if (op->synchronous) {
         action_synced_wait(op, pmask);
         sigchld_cleanup();
@@ -911,6 +1034,26 @@ resources_os_list_ocf_agents(const char *provider)
     return result;
 }
 
+gboolean
+services__ocf_agent_exists(const char *provider, const char *agent)
+{
+    char *buf = NULL;
+    gboolean rc = FALSE;
+    struct stat st;
+
+    if (provider == NULL || agent == NULL) {
+        return rc;
+    }
+
+    buf = crm_strdup_printf(OCF_ROOT_DIR "/resource.d/%s/%s", provider, agent);
+    if (stat(buf, &st) == 0) {
+        rc = TRUE;
+    }
+
+    free(buf);
+    return rc;
+}
+
 #if SUPPORT_NAGIOS
 GList *
 resources_os_list_nagios_agents(void)
@@ -936,4 +1079,24 @@ resources_os_list_nagios_agents(void)
     g_list_free_full(plugin_list, free);
     return result;
 }
+
+gboolean
+services__nagios_agent_exists(const char *name)
+{
+    char *buf = NULL;
+    gboolean rc = FALSE;
+    struct stat st;
+
+    if (name == NULL) {
+        return rc;
+    }
+
+    buf = crm_strdup_printf(NAGIOS_PLUGIN_DIR "/%s", name);
+    if (stat(buf, &st) == 0) {
+        rc = TRUE;
+    }
+
+    free(buf);
+    return rc;
+}
 #endif
