@@ -1,5 +1,7 @@
 /*
- * Copyright 2004-2018 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2004-2019 the Pacemaker project contributors
+ *
+ * The version control history for this file may have further details.
  *
  * This source code is licensed under the GNU General Public License version 2
  * or later (GPLv2+) WITHOUT ANY WARRANTY.
@@ -27,13 +29,21 @@ parse_cli_lifetime(const char *input)
     if (duration == NULL) {
         CMD_ERR("Invalid duration specified: %s", move_lifetime);
         CMD_ERR("Please refer to"
-                " http://en.wikipedia.org/wiki/ISO_8601#Durations"
+                " https://en.wikipedia.org/wiki/ISO_8601#Durations"
                 " for examples of valid durations");
         return NULL;
     }
 
     now = crm_time_new(NULL);
     later = crm_time_add(now, duration);
+    if (later == NULL) {
+        CMD_ERR("Unable to add %s to current time", move_lifetime);
+        CMD_ERR("Please report to " PACKAGE_BUGREPORT " as possible bug");
+        crm_time_free(now);
+        crm_time_free(duration);
+        return NULL;
+    }
+
     crm_time_log(LOG_INFO, "now     ", now,
                  crm_time_log_date | crm_time_log_timeofday | crm_time_log_with_timezone);
     crm_time_log(LOG_INFO, "later   ", later,
