@@ -1,26 +1,14 @@
-
 /*
- * Copyright (C) 2004 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2004-2019 the Pacemaker project contributors
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * The version control history for this file may have further details.
  *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * This source code is licensed under the GNU General Public License version 2
+ * or later (GPLv2+) WITHOUT ANY WARRANTY.
  */
 
 #include <crm_resource.h>
 
-bool print_pending = TRUE;
-
 #define cons_string(x) x?x:"NA"
 void
 cli_resource_print_cts_constraints(pe_working_set_t * data_set)
@@ -29,17 +17,17 @@ cli_resource_print_cts_constraints(pe_working_set_t * data_set)
     xmlNode *lifetime = NULL;
     xmlNode *cib_constraints = get_object_root(XML_CIB_TAG_CONSTRAINTS, data_set->input);
 
-    for (xml_obj = __xml_first_child(cib_constraints); xml_obj != NULL;
-         xml_obj = __xml_next(xml_obj)) {
+    for (xml_obj = __xml_first_child_element(cib_constraints); xml_obj != NULL;
+         xml_obj = __xml_next_element(xml_obj)) {
         const char *id = crm_element_value(xml_obj, XML_ATTR_ID);
 
         if (id == NULL) {
             continue;
         }
 
+        // @COMPAT lifetime is deprecated
         lifetime = first_named_child(xml_obj, "lifetime");
-
-        if (test_ruleset(lifetime, NULL, data_set->now) == FALSE) {
+        if (pe_evaluate_rules(lifetime, NULL, data_set->now, NULL) == FALSE) {
             continue;
         }
 
@@ -68,32 +56,16 @@ cli_resource_print_cts(resource_t * rsc)
     const char *rtype = crm_element_value(rsc->xml, XML_ATTR_TYPE);
     const char *rprov = crm_element_value(rsc->xml, XML_AGENT_ATTR_PROVIDER);
     const char *rclass = crm_element_value(rsc->xml, XML_AGENT_ATTR_CLASS);
+    pe_node_t *node = pe__current_node(rsc);
 
     if (safe_str_eq(rclass, PCMK_RESOURCE_CLASS_STONITH)) {
-        xmlNode *op = NULL;
-
         needs_quorum = FALSE;
-
-        for (op = __xml_first_child(rsc->ops_xml); op != NULL; op = __xml_next(op)) {
-            if (crm_str_eq((const char *)op->name, "op", TRUE)) {
-                const char *name = crm_element_value(op, "name");
-
-                if (safe_str_neq(name, CRMD_ACTION_START)) {
-                    const char *value = crm_element_value(op, "requires");
-
-                    if (safe_str_eq(value, "nothing")) {
-                        needs_quorum = FALSE;
-                    }
-                    break;
-                }
-            }
-        }
+    } else {
+        // @TODO check requires in resource meta-data and rsc_defaults
     }
 
-    if (rsc->running_on != NULL && g_list_length(rsc->running_on) == 1) {
-        node_t *tmp = rsc->running_on->data;
-
-        host = tmp->details->uname;
+    if (node != NULL) {
+        host = node->details->uname;
     }
 
     printf("Resource: %s %s %s %s %s %s %s %s %d %lld 0x%.16llx\n",
@@ -133,11 +105,7 @@ cli_resource_print_list(pe_working_set_t * data_set, bool raw)
     int found = 0;
 
     GListPtr lpc = NULL;
-    int opts = pe_print_printf | pe_print_rsconly;
-
-    if (print_pending) {
-        opts |= pe_print_pending;
-    }
+    int opts = pe_print_printf | pe_print_rsconly | pe_print_pending;
 
     for (lpc = data_set->resources; lpc != NULL; lpc = lpc->next) {
         resource_t *rsc = (resource_t *) lpc->data;
@@ -163,22 +131,18 @@ cli_resource_print_operations(const char *rsc_id, const char *host_uname, bool a
                          pe_working_set_t * data_set)
 {
     resource_t *rsc = NULL;
-    int opts = pe_print_printf | pe_print_rsconly | pe_print_suppres_nl;
+    int opts = pe_print_printf | pe_print_rsconly | pe_print_suppres_nl | pe_print_pending;
     GListPtr ops = find_operations(rsc_id, host_uname, active, data_set);
     GListPtr lpc = NULL;
 
-    if (print_pending) {
-        opts |= pe_print_pending;
-    }
-
     for (lpc = ops; lpc != NULL; lpc = lpc->next) {
         xmlNode *xml_op = (xmlNode *) lpc->data;
 
         const char *op_rsc = crm_element_value(xml_op, "resource");
-        const char *last = crm_element_value(xml_op, XML_RSC_OP_LAST_CHANGE);
         const char *status_s = crm_element_value(xml_op, XML_LRM_ATTR_OPSTATUS);
         const char *op_key = crm_element_value(xml_op, XML_LRM_ATTR_TASK_KEY);
         int status = crm_parse_int(status_s, "0");
+        time_t last_change = 0;
 
         rsc = pe_find_resource(data_set->resources, op_rsc);
         if(rsc) {
@@ -192,11 +156,12 @@ cli_resource_print_operations(const char *rsc_id, const char *host_uname, bool a
                 crm_element_value(xml_op, XML_ATTR_UNAME),
                 crm_element_value(xml_op, XML_LRM_ATTR_CALLID),
                 crm_element_value(xml_op, XML_LRM_ATTR_RC));
-        if (last) {
-            time_t run_at = crm_parse_int(last, "0");
 
+        if (crm_element_value_epoch(xml_op, XML_RSC_OP_LAST_CHANGE,
+                                    &last_change) == pcmk_ok) {
             fprintf(stdout, ", last-rc-change=%s, exec=%sms",
-                    crm_strip_trailing_newline(ctime(&run_at)), crm_element_value(xml_op, XML_RSC_OP_T_EXEC));
+                    crm_strip_trailing_newline(ctime(&last_change)),
+                    crm_element_value(xml_op, XML_RSC_OP_T_EXEC));
         }
         fprintf(stdout, "): %s\n", services_lrm_status_str(status));
     }
@@ -215,7 +180,7 @@ cli_resource_print_location(resource_t * rsc, const char *prefix)
     }
 
     for (lpc = list; lpc != NULL; lpc = lpc->next) {
-        rsc_to_node_t *cons = (rsc_to_node_t *) lpc->data;
+        pe__location_t *cons = lpc->data;
 
         GListPtr lpc2 = NULL;
 
@@ -297,11 +262,8 @@ int
 cli_resource_print(resource_t *rsc, pe_working_set_t *data_set, bool expanded)
 {
     char *rsc_xml = NULL;
-    int opts = pe_print_printf;
+    int opts = pe_print_printf | pe_print_pending;
 
-    if (print_pending) {
-        opts |= pe_print_pending;
-    }
     rsc->fns->print(rsc, NULL, opts, stdout);
 
     rsc_xml = dump_xml_formatted((!expanded && rsc->orig_xml)?
@@ -315,16 +277,15 @@ int
 cli_resource_print_attribute(resource_t *rsc, const char *attr, pe_working_set_t * data_set)
 {
     int rc = -ENXIO;
-    node_t *current = NULL;
+    unsigned int count = 0;
     GHashTable *params = NULL;
     const char *value = NULL;
+    node_t *current = pe__find_active_on(rsc, &count, NULL);
 
-    if (g_list_length(rsc->running_on) == 1) {
-        current = rsc->running_on->data;
-
-    } else if (g_list_length(rsc->running_on) > 1) {
+    if (count > 1) {
         CMD_ERR("%s is active on more than one node,"
                 " returning the default value for %s", rsc->id, crm_str(attr));
+        current = NULL;
     }
 
     params = crm_str_table_new();
@@ -337,9 +298,8 @@ cli_resource_print_attribute(resource_t *rsc, const char *attr, pe_working_set_t
         get_meta_attributes(params, rsc, current, data_set);
 
     } else {
-        unpack_instance_attributes(data_set->input, rsc->xml,
-                                   XML_TAG_UTILIZATION, NULL,
-                                   params, NULL, FALSE, data_set->now);
+        pe__unpack_dataset_nvpairs(rsc->xml, XML_TAG_UTILIZATION, NULL, params,
+                                   NULL, FALSE, data_set);
     }
 
     crm_debug("Looking up %s in %s", attr, rsc->id);
