@@ -1,5 +1,7 @@
 /*
- * Copyright 2004-2018 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2004-2019 the Pacemaker project contributors
+ *
+ * The version control history for this file may have further details.
  *
  * This source code is licensed under the GNU General Public License version 2
  * or later (GPLv2+) WITHOUT ANY WARRANTY.
@@ -152,44 +154,67 @@ find_resource_attr(cib_t * the_cib, const char *attr, const char *rsc, const cha
     return rc;
 }
 
-static resource_t *
-find_matching_attr_resource(resource_t * rsc, const char * rsc_id, const char * attr_set, const char * attr_id,
+/* PRIVATE. Use the find_matching_attr_resources instead. */
+static void
+find_matching_attr_resources_recursive(GList/* <resource_t*> */ ** result, resource_t * rsc, const char * rsc_id, 
+		                       const char * attr_set, const char * attr_id,
+                                       const char * attr_name, cib_t * cib, const char * cmd, int depth)
+{
+    int rc = pcmk_ok;
+    char *lookup_id = clone_strip(rsc->id);
+    char *local_attr_id = NULL;
+
+    /* visit the children */
+    for(GList *gIter = rsc->children; gIter; gIter = gIter->next) {
+        find_matching_attr_resources_recursive(result, (resource_t*)gIter->data, rsc_id, attr_set, attr_id, attr_name, cib, cmd, depth+1);
+        /* do it only once for clones */
+        if(pe_clone == rsc->variant) {
+            break;
+        }
+    }
+
+    rc = find_resource_attr(cib, XML_ATTR_ID, lookup_id, attr_set_type, attr_set, attr_id, attr_name, &local_attr_id);
+    /* Post-order traversal. 
+     * The root is always on the list and it is the last item. */
+    if((0 == depth) || (pcmk_ok == rc)) {
+        /* push the head */
+        *result = g_list_append(*result, rsc);
+    }
+
+    free(local_attr_id);
+    free(lookup_id);
+}
+
+
+/* The result is a linearized pre-ordered tree of resources. */
+static GList/*<resource_t*>*/ *
+find_matching_attr_resources(resource_t * rsc, const char * rsc_id, const char * attr_set, const char * attr_id,
                             const char * attr_name, cib_t * cib, const char * cmd)
 {
     int rc = pcmk_ok;
     char *lookup_id = NULL;
     char *local_attr_id = NULL;
-
+    GList * result = NULL;
+    /* If --force is used, update only the requested resource (clone or primitive).
+     * Otherwise, if the primitive has the attribute, use that.
+     * Otherwise use the clone. */
     if(do_force == TRUE) {
-        return rsc;
+        return g_list_append(result, rsc);
+    }
+    if(rsc->parent && pe_clone == rsc->parent->variant) {
+        int rc = pcmk_ok;
+        char *local_attr_id = NULL;
+        rc = find_resource_attr(cib, XML_ATTR_ID, rsc_id, attr_set_type, attr_set, attr_id, attr_name, &local_attr_id);
+        free(local_attr_id);
 
-    } else if(rsc->parent) {
-        switch(rsc->parent->variant) {
-            case pe_group:
-                if (BE_QUIET == FALSE) {
-                    printf("Performing %s of '%s' for '%s' will not apply to its peers in '%s'\n", cmd, attr_name, rsc_id, rsc->parent->id);
-                }
-                break;
-
-            case pe_clone:
-                rc = find_resource_attr(cib, XML_ATTR_ID, rsc_id, attr_set_type, attr_set, attr_id, attr_name, &local_attr_id);
-                free(local_attr_id);
-
-                if(rc != pcmk_ok) {
-                    rsc = rsc->parent;
-                    if (BE_QUIET == FALSE) {
-                        printf("Performing %s of '%s' on '%s', the parent of '%s'\n", cmd, attr_name, rsc->id, rsc_id);
-                    }
-                }
-                break;
-            default:
-                break;
+        if(rc != pcmk_ok) {
+            rsc = rsc->parent;
+            if (BE_QUIET == FALSE) {
+                printf("Performing %s of '%s' on '%s', the parent of '%s'\n", cmd, attr_name, rsc->id, rsc_id);
+            }
         }
-
-    } else if (rsc->parent && BE_QUIET == FALSE) {
-        printf("Forcing %s of '%s' for '%s' instead of '%s'\n", cmd, attr_name, rsc_id, rsc->parent->id);
-
-    } else if(rsc->parent == NULL && rsc->children) {
+        return g_list_append(result, rsc);
+    } else if(rsc->parent == NULL && rsc->children && pe_clone == rsc->variant) {
         resource_t *child = rsc->children->data;
 
         if(child->variant == pe_native) {
@@ -206,9 +231,11 @@ find_matching_attr_resource(resource_t * rsc, const char * rsc_id, const char *
             free(local_attr_id);
             free(lookup_id);
         }
+        return g_list_append(result, rsc);
     }
-
-    return rsc;
+    /* If the resource is a group ==> children inherit the attribute if defined. */
+    find_matching_attr_resources_recursive(&result, rsc, rsc_id, attr_set, attr_id, attr_name, cib, cmd, 0);
+    return result;
 }
 
 int
@@ -221,12 +248,11 @@ cli_resource_update_attribute(resource_t *rsc, const char *requested_name,
     int rc = pcmk_ok;
     static bool need_init = TRUE;
 
-    char *lookup_id = NULL;
     char *local_attr_id = NULL;
     char *local_attr_set = NULL;
 
-    xmlNode *xml_top = NULL;
-    xmlNode *xml_obj = NULL;
+    GList/*<resource_t*>*/ *resources = NULL;
+    const char *common_attr_id = attr_id;
 
     if(attr_id == NULL
        && do_force == FALSE
@@ -251,97 +277,115 @@ cli_resource_update_attribute(resource_t *rsc, const char *requested_name,
                 return -ENOTUNIQ;
             }
         }
+        resources = g_list_append(resources, rsc);
 
     } else {
-        rsc = find_matching_attr_resource(rsc, requested_name, attr_set,
-                                          attr_id, attr_name, cib, "update");
+        resources = find_matching_attr_resources(rsc, requested_name, attr_set,
+                                                 attr_id, attr_name, cib, "update");
     }
 
-    lookup_id = clone_strip(rsc->id); /* Could be a cloned group! */
-    rc = find_resource_attr(cib, XML_ATTR_ID, lookup_id, attr_set_type, attr_set, attr_id, attr_name,
-                            &local_attr_id);
+    /* If either attr_set or attr_id is specified,
+     * one clearly intends to modify a single resource.
+     * It is the last item on the resource list.*/
+    for(GList *gIter = (attr_set||attr_id) ? g_list_last(resources) : resources
+            ; gIter; gIter = gIter->next) {
+        char *lookup_id = NULL;
 
-    if (rc == pcmk_ok) {
-        crm_debug("Found a match for name=%s: id=%s", attr_name, local_attr_id);
-        attr_id = local_attr_id;
+        xmlNode *xml_top = NULL;
+        xmlNode *xml_obj = NULL;
+        local_attr_id = NULL;
+        local_attr_set = NULL;
+
+        rsc = (resource_t*)gIter->data;
+        attr_id = common_attr_id;
+
+        lookup_id = clone_strip(rsc->id); /* Could be a cloned group! */
+        rc = find_resource_attr(cib, XML_ATTR_ID, lookup_id, attr_set_type, attr_set, attr_id, attr_name,
+                                &local_attr_id);
+
+        if (rc == pcmk_ok) {
+            crm_debug("Found a match for name=%s: id=%s", attr_name, local_attr_id);
+            attr_id = local_attr_id;
+
+        } else if (rc != -ENXIO) {
+            free(lookup_id);
+            free(local_attr_id);
+            g_list_free(resources);
+            return rc;
+
+        } else {
+            const char *tag = crm_element_name(rsc->xml);
+
+            if (attr_set == NULL) {
+                local_attr_set = crm_concat(lookup_id, attr_set_type, '-');
+                attr_set = local_attr_set;
+            }
+            if (attr_id == NULL) {
+                local_attr_id = crm_concat(attr_set, attr_name, '-');
+                attr_id = local_attr_id;
+            }
+
+            xml_top = create_xml_node(NULL, tag);
+            crm_xml_add(xml_top, XML_ATTR_ID, lookup_id);
+
+            xml_obj = create_xml_node(xml_top, attr_set_type);
+            crm_xml_add(xml_obj, XML_ATTR_ID, attr_set);
+        }
+
+        xml_obj = crm_create_nvpair_xml(xml_obj, attr_id, attr_name, attr_value);
+        if (xml_top == NULL) {
+            xml_top = xml_obj;
+        }
+
+        crm_log_xml_debug(xml_top, "Update");
+
+        rc = cib->cmds->modify(cib, XML_CIB_TAG_RESOURCES, xml_top, cib_options);
+        if (rc == pcmk_ok && BE_QUIET == FALSE) {
+            printf("Set '%s' option: id=%s%s%s%s%s value=%s\n", lookup_id, local_attr_id,
+                   attr_set ? " set=" : "", attr_set ? attr_set : "",
+                   attr_name ? " name=" : "", attr_name ? attr_name : "", attr_value);
+        }
+
+        free_xml(xml_top);
 
-    } else if (rc != -ENXIO) {
         free(lookup_id);
         free(local_attr_id);
-        return rc;
+        free(local_attr_set);
 
-    } else {
-        const char *tag = crm_element_name(rsc->xml);
+        if(recursive && safe_str_eq(attr_set_type, XML_TAG_META_SETS)) {
+            GListPtr lpc = NULL;
 
-        if (attr_set == NULL) {
-            local_attr_set = crm_concat(lookup_id, attr_set_type, '-');
-            attr_set = local_attr_set;
-        }
-        if (attr_id == NULL) {
-            local_attr_id = crm_concat(attr_set, attr_name, '-');
-            attr_id = local_attr_id;
-        }
+            if(need_init) {
+                xmlNode *cib_constraints = get_object_root(XML_CIB_TAG_CONSTRAINTS, data_set->input);
 
-        xml_top = create_xml_node(NULL, tag);
-        crm_xml_add(xml_top, XML_ATTR_ID, lookup_id);
+                need_init = FALSE;
+                unpack_constraints(cib_constraints, data_set);
 
-        xml_obj = create_xml_node(xml_top, attr_set_type);
-        crm_xml_add(xml_obj, XML_ATTR_ID, attr_set);
-    }
+                for (lpc = data_set->resources; lpc != NULL; lpc = lpc->next) {
+                    resource_t *r = (resource_t *) lpc->data;
 
-    xml_obj = crm_create_nvpair_xml(xml_obj, attr_id, attr_name, attr_value);
-    if (xml_top == NULL) {
-        xml_top = xml_obj;
-    }
-
-    crm_log_xml_debug(xml_top, "Update");
-
-    rc = cib->cmds->modify(cib, XML_CIB_TAG_RESOURCES, xml_top, cib_options);
-    if (rc == pcmk_ok && BE_QUIET == FALSE) {
-        printf("Set '%s' option: id=%s%s%s%s%s value=%s\n", lookup_id, local_attr_id,
-               attr_set ? " set=" : "", attr_set ? attr_set : "",
-               attr_name ? " name=" : "", attr_name ? attr_name : "", attr_value);
-    }
-
-    free_xml(xml_top);
-
-    free(lookup_id);
-    free(local_attr_id);
-    free(local_attr_set);
-
-    if(recursive && safe_str_eq(attr_set_type, XML_TAG_META_SETS)) {
-        GListPtr lpc = NULL;
-
-        if(need_init) {
-            xmlNode *cib_constraints = get_object_root(XML_CIB_TAG_CONSTRAINTS, data_set->input);
-
-            need_init = FALSE;
-            unpack_constraints(cib_constraints, data_set);
-
-            for (lpc = data_set->resources; lpc != NULL; lpc = lpc->next) {
-                resource_t *r = (resource_t *) lpc->data;
-
-                clear_bit(r->flags, pe_rsc_allocating);
+                    clear_bit(r->flags, pe_rsc_allocating);
+                }
             }
-        }
 
-        crm_debug("Looking for dependencies %p", rsc->rsc_cons_lhs);
-        set_bit(rsc->flags, pe_rsc_allocating);
-        for (lpc = rsc->rsc_cons_lhs; lpc != NULL; lpc = lpc->next) {
-            rsc_colocation_t *cons = (rsc_colocation_t *) lpc->data;
-            resource_t *peer = cons->rsc_lh;
+            crm_debug("Looking for dependencies %p", rsc->rsc_cons_lhs);
+            set_bit(rsc->flags, pe_rsc_allocating);
+            for (lpc = rsc->rsc_cons_lhs; lpc != NULL; lpc = lpc->next) {
+                rsc_colocation_t *cons = (rsc_colocation_t *) lpc->data;
+                resource_t *peer = cons->rsc_lh;
 
-            crm_debug("Checking %s %d", cons->id, cons->score);
-            if (cons->score > 0 && is_not_set(peer->flags, pe_rsc_allocating)) {
-                /* Don't get into colocation loops */
-                crm_debug("Setting %s=%s for dependent resource %s", attr_name, attr_value, peer->id);
-                cli_resource_update_attribute(peer, peer->id, NULL, NULL,
-                                              attr_name, attr_value, recursive,
-                                              cib, data_set);
+                crm_debug("Checking %s %d", cons->id, cons->score);
+                if (cons->score > 0 && is_not_set(peer->flags, pe_rsc_allocating)) {
+                    /* Don't get into colocation loops */
+                    crm_debug("Setting %s=%s for dependent resource %s", attr_name, attr_value, peer->id);
+                    cli_resource_update_attribute(peer, peer->id, NULL, NULL,
+                                                  attr_name, attr_value, recursive,
+                                                  cib, data_set);
+                }
             }
         }
     }
-
+    g_list_free(resources);
     return rc;
 }
 
@@ -351,11 +395,8 @@ cli_resource_delete_attribute(resource_t *rsc, const char *requested_name,
                               const char *attr_name, cib_t *cib,
                               pe_working_set_t *data_set)
 {
-    xmlNode *xml_obj = NULL;
-
     int rc = pcmk_ok;
-    char *lookup_id = NULL;
-    char *local_attr_id = NULL;
+    GList/*<resource_t*>*/ *resources = NULL;
 
     if(attr_id == NULL
        && do_force == FALSE
@@ -365,42 +406,55 @@ cli_resource_delete_attribute(resource_t *rsc, const char *requested_name,
     }
 
     if(safe_str_eq(attr_set_type, XML_TAG_META_SETS)) {
-        rsc = find_matching_attr_resource(rsc, requested_name, attr_set,
-                                          attr_id, attr_name, cib, "delete");
+        resources = find_matching_attr_resources(rsc, requested_name, attr_set,
+                                                 attr_id, attr_name, cib, "delete");
+    } else {
+        resources = g_list_append(resources, rsc);
     }
 
-    lookup_id = clone_strip(rsc->id);
-    rc = find_resource_attr(cib, XML_ATTR_ID, lookup_id, attr_set_type, attr_set, attr_id, attr_name,
-                            &local_attr_id);
+    for(GList *gIter = resources; gIter; gIter = gIter->next) {
+        char *lookup_id = NULL;
+        xmlNode *xml_obj = NULL;
+        char *local_attr_id = NULL;
+
+        rsc = (resource_t*)gIter->data;
+
+        lookup_id = clone_strip(rsc->id);
+        rc = find_resource_attr(cib, XML_ATTR_ID, lookup_id, attr_set_type, attr_set, attr_id, attr_name,
+                                &local_attr_id);
+
+        if (rc == -ENXIO) {
+            free(lookup_id);
+            rc = pcmk_ok;
+            continue;
+
+        } else if (rc != pcmk_ok) {
+            free(lookup_id);
+            g_list_free(resources);
+            return rc;
+        }
+
+        if (attr_id == NULL) {
+            attr_id = local_attr_id;
+        }
+
+        xml_obj = crm_create_nvpair_xml(NULL, attr_id, attr_name, NULL);
+        crm_log_xml_debug(xml_obj, "Delete");
+
+        CRM_ASSERT(cib);
+        rc = cib->cmds->remove(cib, XML_CIB_TAG_RESOURCES, xml_obj, cib_options);
+
+        if (rc == pcmk_ok && BE_QUIET == FALSE) {
+            printf("Deleted '%s' option: id=%s%s%s%s%s\n", lookup_id, local_attr_id,
+                   attr_set ? " set=" : "", attr_set ? attr_set : "",
+                   attr_name ? " name=" : "", attr_name ? attr_name : "");
+        }
 
-    if (rc == -ENXIO) {
         free(lookup_id);
-        return pcmk_ok;
-
-    } else if (rc != pcmk_ok) {
-        free(lookup_id);
-        return rc;
+        free_xml(xml_obj);
+        free(local_attr_id);
     }
-
-    if (attr_id == NULL) {
-        attr_id = local_attr_id;
-    }
-
-    xml_obj = crm_create_nvpair_xml(NULL, attr_id, attr_name, NULL);
-    crm_log_xml_debug(xml_obj, "Delete");
-
-    CRM_ASSERT(cib);
-    rc = cib->cmds->remove(cib, XML_CIB_TAG_RESOURCES, xml_obj, cib_options);
-
-    if (rc == pcmk_ok && BE_QUIET == FALSE) {
-        printf("Deleted '%s' option: id=%s%s%s%s%s\n", lookup_id, local_attr_id,
-               attr_set ? " set=" : "", attr_set ? attr_set : "",
-               attr_name ? " name=" : "", attr_name ? attr_name : "");
-    }
-
-    free(lookup_id);
-    free_xml(xml_obj);
-    free(local_attr_id);
+    g_list_free(resources);
     return rc;
 }
 
@@ -434,7 +488,7 @@ send_lrm_rsc_op(crm_ipc_t * crmd_channel, const char *op,
     } else {
         node_t *node = pe_find_node(data_set->nodes, host_uname);
 
-        if (node && is_remote_node(node)) {
+        if (pe__is_guest_or_remote_node(node)) {
             node = pe__current_node(node->details->remote_rsc);
             if (node == NULL) {
                 CMD_ERR("No cluster connection to Pacemaker Remote node %s detected",
@@ -641,7 +695,7 @@ clear_rsc_fail_attrs(resource_t *rsc, const char *operation,
     int attr_options = attrd_opt_none;
     char *rsc_name = rsc_fail_name(rsc);
 
-    if (is_remote_node(node)) {
+    if (pe__is_guest_or_remote_node(node)) {
         attr_options |= attrd_opt_remote;
     }
     rc = attrd_clear_delegate(NULL, node->details->uname, rsc_name, operation,
@@ -781,7 +835,7 @@ cli_cleanup_all(crm_ipc_t *crmd_channel, const char *node_name,
             CMD_ERR("Unknown node: %s", node_name);
             return -ENXIO;
         }
-        if (is_remote_node(node)) {
+        if (pe__is_guest_or_remote_node(node)) {
             attr_options |= attrd_opt_remote;
         }
     }
@@ -988,8 +1042,8 @@ get_active_resources(const char *host, GList *rsc_list)
     return active;
 }
 
-static GList*
-subtract_lists(GList *from, GList *items) 
+GList*
+subtract_lists(GList *from, GList *items, GCompareFunc cmp)
 {
     GList *item = NULL;
     GList *result = g_list_copy(from);
@@ -999,7 +1053,7 @@ subtract_lists(GList *from, GList *items)
         for (candidate = from; candidate != NULL; candidate = candidate->next) {
             crm_info("Comparing %s with %s", (const char *) candidate->data,
                      (const char *) item->data);
-            if(strcmp(candidate->data, item->data) == 0) {
+            if(cmp(candidate->data, item->data) == 0) {
                 result = g_list_remove(result, candidate->data);
                 break;
             }
@@ -1125,7 +1179,7 @@ update_dataset(cib_t *cib, pe_working_set_t * data_set, bool simulate)
             goto cleanup;
         }
 
-        do_calculations(data_set, data_set->input, NULL);
+        pcmk__schedule_actions(data_set, data_set->input, NULL);
         run_simulation(data_set, shadow_cib, NULL, TRUE);
         rc = update_dataset(shadow_cib, data_set, FALSE);
 
@@ -1282,6 +1336,7 @@ cli_resource_restart(pe_resource_t *rsc, const char *host, int timeout_ms,
         rc = -ENOMEM;
         goto done;
     }
+    set_bit(data_set->flags, pe_flag_no_counts);
     rc = update_dataset(cib, data_set, FALSE);
     if(rc != pcmk_ok) {
         fprintf(stdout, "Could not get new resource list: %s (%d)\n", pcmk_strerror(rc), rc);
@@ -1332,7 +1387,7 @@ cli_resource_restart(pe_resource_t *rsc, const char *host, int timeout_ms,
     target_active = get_active_resources(host, data_set->resources);
     dump_list(target_active, "Target");
 
-    list_delta = subtract_lists(current_active, target_active);
+    list_delta = subtract_lists(current_active, target_active, (GCompareFunc) strcmp);
     fprintf(stdout, "Waiting for %d resources to stop:\n", g_list_length(list_delta));
     display_list(list_delta, " * ");
 
@@ -1361,7 +1416,7 @@ cli_resource_restart(pe_resource_t *rsc, const char *host, int timeout_ms,
             }
             current_active = get_active_resources(host, data_set->resources);
             g_list_free(list_delta);
-            list_delta = subtract_lists(current_active, target_active);
+            list_delta = subtract_lists(current_active, target_active, (GCompareFunc) strcmp);
             dump_list(current_active, "Current");
             dump_list(list_delta, "Delta");
         }
@@ -1405,7 +1460,7 @@ cli_resource_restart(pe_resource_t *rsc, const char *host, int timeout_ms,
     if (list_delta) {
         g_list_free(list_delta);
     }
-    list_delta = subtract_lists(target_active, current_active);
+    list_delta = subtract_lists(target_active, current_active, (GCompareFunc) strcmp);
     fprintf(stdout, "Waiting for %d resources to start again:\n", g_list_length(list_delta));
     display_list(list_delta, " * ");
 
@@ -1440,7 +1495,7 @@ cli_resource_restart(pe_resource_t *rsc, const char *host, int timeout_ms,
              */
             current_active = get_active_resources(NULL, data_set->resources);
             g_list_free(list_delta);
-            list_delta = subtract_lists(target_active, current_active);
+            list_delta = subtract_lists(target_active, current_active, (GCompareFunc) strcmp);
             dump_list(current_active, "Current");
             dump_list(list_delta, "Delta");
         }
@@ -1587,6 +1642,7 @@ wait_till_stable(int timeout_ms, cib_t * cib)
     if (data_set == NULL) {
         return -ENOMEM;
     }
+    set_bit(data_set->flags, pe_flag_no_counts);
 
     do {
 
@@ -1610,7 +1666,7 @@ wait_till_stable(int timeout_ms, cib_t * cib)
             pe_free_working_set(data_set);
             return rc;
         }
-        do_calculations(data_set, data_set->input, NULL);
+        pcmk__schedule_actions(data_set, data_set->input, NULL);
 
         if (!printed_version_warning) {
             /* If the DC has a different version than the local node, the two
@@ -1638,95 +1694,60 @@ wait_till_stable(int timeout_ms, cib_t * cib)
 }
 
 int
-cli_resource_execute(resource_t *rsc, const char *requested_name,
-                     const char *rsc_action, GHashTable *override_hash,
-                     int timeout_ms, cib_t * cib, pe_working_set_t *data_set)
+cli_resource_execute_from_params(const char *rsc_name, const char *rsc_class,
+                                 const char *rsc_prov, const char *rsc_type,
+                                 const char *action, GHashTable *params,
+                                 GHashTable *override_hash, int timeout_ms)
 {
+    GHashTable *params_copy = NULL;
     int rc = pcmk_ok;
     svc_action_t *op = NULL;
-    const char *rid = NULL;
-    const char *rtype = NULL;
-    const char *rprov = NULL;
-    const char *rclass = NULL;
-    const char *action = NULL;
-    GHashTable *params = NULL;
 
-    if (safe_str_eq(rsc_action, "validate")) {
-        action = "validate-all";
-
-    } else if (safe_str_eq(rsc_action, "force-check")) {
-        action = "monitor";
-
-    } else if (safe_str_eq(rsc_action, "force-stop")) {
-        action = rsc_action+6;
-
-    } else if (safe_str_eq(rsc_action, "force-start")
-               || safe_str_eq(rsc_action, "force-demote")
-               || safe_str_eq(rsc_action, "force-promote")) {
-        action = rsc_action+6;
-
-        if(pe_rsc_is_clone(rsc)) {
-            rc = cli_resource_search(rsc, requested_name, data_set);
-            if(rc > 0 && do_force == FALSE) {
-                CMD_ERR("It is not safe to %s %s here: the cluster claims it is already active",
-                        action, rsc->id);
-                CMD_ERR("Try setting target-role=Stopped first or specifying "
-                        "the force option");
-                crm_exit(CRM_EX_UNSAFE);
-            }
-        }
-    }
-
-    if(pe_rsc_is_clone(rsc)) {
-        /* Grab the first child resource in the hope it's not a group */
-        rsc = rsc->children->data;
-    }
-
-    if(rsc->variant == pe_group) {
-        CMD_ERR("Sorry, the %s option doesn't support group resources",
-                rsc_action);
-        crm_exit(CRM_EX_UNIMPLEMENT_FEATURE);
-    }
-
-    rclass = crm_element_value(rsc->xml, XML_AGENT_ATTR_CLASS);
-    rprov = crm_element_value(rsc->xml, XML_AGENT_ATTR_PROVIDER);
-    rtype = crm_element_value(rsc->xml, XML_ATTR_TYPE);
-
-    if (safe_str_eq(rclass, PCMK_RESOURCE_CLASS_STONITH)) {
+    if (safe_str_eq(rsc_class, PCMK_RESOURCE_CLASS_STONITH)) {
         CMD_ERR("Sorry, the %s option doesn't support %s resources yet",
-                rsc_action, rclass);
+                action, rsc_class);
         crm_exit(CRM_EX_UNIMPLEMENT_FEATURE);
     }
 
-    params = generate_resource_params(rsc, data_set);
+    /* If no timeout was provided, grab the default. */
+    if (timeout_ms == 0) {
+        timeout_ms = crm_get_msec(CRM_DEFAULT_OP_TIMEOUT_S);
+    }
 
     /* add meta_timeout env needed by some resource agents */
-    if (timeout_ms == 0) {
-        timeout_ms = pe_get_configured_timeout(rsc, action, data_set);
-    }
     g_hash_table_insert(params, strdup("CRM_meta_timeout"),
                         crm_strdup_printf("%d", timeout_ms));
 
     /* add crm_feature_set env needed by some resource agents */
     g_hash_table_insert(params, strdup(XML_ATTR_CRM_VERSION), strdup(CRM_FEATURE_SET));
 
-    rid = pe_rsc_is_anon_clone(rsc->parent)? requested_name : rsc->id;
+    /* resources_action_create frees the params hash table it's passed, but we
+     * may need to reuse it in a second call to resources_action_create.  Thus
+     * we'll make a copy here so that gets freed and the original remains for
+     * reuse.
+     */
+    params_copy = crm_str_table_dup(params);
 
-    op = resources_action_create(rid, rclass, rprov, rtype, action, 0,
-                                 timeout_ms, params, 0);
+    op = resources_action_create(rsc_name, rsc_class, rsc_prov, rsc_type, action, 0,
+                                 timeout_ms, params_copy, 0);
     if (op == NULL) {
         /* Re-run with stderr enabled so we can display a sane error message */
         crm_enable_stderr(TRUE);
-        op = resources_action_create(rid, rclass, rprov, rtype, action, 0,
-                                     timeout_ms, params, 0);
+        params_copy = crm_str_table_dup(params);
+        op = resources_action_create(rsc_name, rsc_class, rsc_prov, rsc_type, action, 0,
+                                     timeout_ms, params_copy, 0);
+
+        /* Callers of cli_resource_execute expect that the params hash table will
+         * be freed.  That function uses this one, so for that reason and for
+         * making the two act the same, we should free the hash table here too.
+         */
+        g_hash_table_destroy(params);
 
         /* We know op will be NULL, but this makes static analysis happy */
         services_action_free(op);
-
-        return crm_exit(CRM_EX_DATAERR);
+        crm_exit(CRM_EX_DATAERR);
     }
 
-
     setenv("HA_debug", resource_verbose > 0 ? "1" : "0", 1);
     if(resource_verbose > 1) {
         setenv("OCF_TRACE_RA", "1", 1);
@@ -1746,7 +1767,7 @@ cli_resource_execute(resource_t *rsc, const char *requested_name,
         g_hash_table_iter_init(&iter, override_hash);
         while (g_hash_table_iter_next(&iter, (gpointer *) & name, (gpointer *) & value)) {
             printf("Overriding the cluster configuration for '%s' with '%s' = '%s'\n",
-                   rsc->id, name, value);
+                   rsc_name, name, value);
             g_hash_table_replace(op->params, strdup(name), strdup(value));
         }
     }
@@ -1755,13 +1776,15 @@ cli_resource_execute(resource_t *rsc, const char *requested_name,
         int more, lpc, last;
         char *local_copy = NULL;
 
+        rc = op->rc;
+
         if (op->status == PCMK_LRM_OP_DONE) {
             printf("Operation %s for %s (%s:%s:%s) returned: '%s' (%d)\n",
-                   action, rsc->id, rclass, rprov ? rprov : "", rtype,
+                   action, rsc_name, rsc_class, rsc_prov ? rsc_prov : "", rsc_type,
                    services_ocf_exitcode_str(op->rc), op->rc);
         } else {
             printf("Operation %s for %s (%s:%s:%s) failed: '%s' (%d)\n",
-                   action, rsc->id, rclass, rprov ? rprov : "", rtype,
+                   action, rsc_name, rsc_class, rsc_prov ? rsc_prov : "", rsc_type,
                    services_lrm_status_str(op->status), op->status);
         }
 
@@ -1797,10 +1820,84 @@ cli_resource_execute(resource_t *rsc, const char *requested_name,
             }
             free(local_copy);
         }
+    } else {
+        rc = op->rc == 0 ? pcmk_err_generic : op->rc;
     }
-  done:
-    rc = op->rc;
+
+done:
     services_action_free(op);
+    /* See comment above about why we free params here. */
+    g_hash_table_destroy(params);
+    return rc;
+}
+
+int
+cli_resource_execute(resource_t *rsc, const char *requested_name,
+                     const char *rsc_action, GHashTable *override_hash,
+                     int timeout_ms, cib_t * cib, pe_working_set_t *data_set)
+{
+    int rc = pcmk_ok;
+    const char *rid = NULL;
+    const char *rtype = NULL;
+    const char *rprov = NULL;
+    const char *rclass = NULL;
+    const char *action = NULL;
+    GHashTable *params = NULL;
+
+    if (safe_str_eq(rsc_action, "validate")) {
+        action = "validate-all";
+
+    } else if (safe_str_eq(rsc_action, "force-check")) {
+        action = "monitor";
+
+    } else if (safe_str_eq(rsc_action, "force-stop")) {
+        action = rsc_action+6;
+
+    } else if (safe_str_eq(rsc_action, "force-start")
+               || safe_str_eq(rsc_action, "force-demote")
+               || safe_str_eq(rsc_action, "force-promote")) {
+        action = rsc_action+6;
+
+        if(pe_rsc_is_clone(rsc)) {
+            rc = cli_resource_search(rsc, requested_name, data_set);
+            if(rc > 0 && do_force == FALSE) {
+                CMD_ERR("It is not safe to %s %s here: the cluster claims it is already active",
+                        action, rsc->id);
+                CMD_ERR("Try setting target-role=Stopped first or specifying "
+                        "the force option");
+                crm_exit(CRM_EX_UNSAFE);
+            }
+        }
+
+    } else {
+        action = rsc_action;
+    }
+
+    if(pe_rsc_is_clone(rsc)) {
+        /* Grab the first child resource in the hope it's not a group */
+        rsc = rsc->children->data;
+    }
+
+    if(rsc->variant == pe_group) {
+        CMD_ERR("Sorry, the %s option doesn't support group resources",
+                rsc_action);
+        crm_exit(CRM_EX_UNIMPLEMENT_FEATURE);
+    }
+
+    rclass = crm_element_value(rsc->xml, XML_AGENT_ATTR_CLASS);
+    rprov = crm_element_value(rsc->xml, XML_AGENT_ATTR_PROVIDER);
+    rtype = crm_element_value(rsc->xml, XML_ATTR_TYPE);
+
+    params = generate_resource_params(rsc, data_set);
+
+    if (timeout_ms == 0) {
+        timeout_ms = pe_get_configured_timeout(rsc, action, data_set);
+    }
+
+    rid = pe_rsc_is_anon_clone(rsc->parent)? requested_name : rsc->id;
+
+    rc = cli_resource_execute_from_params(rid, rclass, rprov, rtype, action,
+                                          params, override_hash, timeout_ms);
     return rc;
 }
 
@@ -1952,7 +2049,7 @@ cli_resource_why_without_rsc_with_host(cib_t *cib_conn,GListPtr resources,node_t
     const char* host_uname =  node->details->uname;
     GListPtr allResources = node->details->allocated_rsc;
     GListPtr activeResources = node->details->running_rsc;
-    GListPtr unactiveResources = subtract_lists(allResources,activeResources);
+    GListPtr unactiveResources = subtract_lists(allResources,activeResources,(GCompareFunc) strcmp);
     GListPtr lpc = NULL;
 
     for (lpc = activeResources; lpc != NULL; lpc = lpc->next) {
