@@ -1,19 +1,10 @@
 /*
- * Copyright (C) 2009 Andrew Beekhof <andrew@beekhof.net>
+ * Copyright 2009-2019 the Pacemaker project contributors
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * The version control history for this file may have further details.
  *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * This source code is licensed under the GNU General Public License version 2
+ * or later (GPLv2+) WITHOUT ANY WARRANTY.
  */
 
 #include <crm_internal.h>
@@ -21,6 +12,7 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <stdlib.h>
+#include <time.h>
 
 #include <sys/stat.h>
 #include <sys/param.h>
@@ -30,11 +22,9 @@
 #include <crm/crm.h>
 #include <crm/cib.h>
 #include <crm/common/util.h>
-#include <crm/transition.h>
 #include <crm/common/iso8601.h>
 #include <crm/pengine/status.h>
-#include <allocate.h>
-#include "fake_transition.h"
+#include <pacemaker-internal.h>
 
 cib_t *global_cib = NULL;
 GListPtr op_fail = NULL;
@@ -50,34 +40,34 @@ extern gboolean bringing_nodes_online;
 	}					\
     } while(0)
 
-extern void cleanup_alloc_calculations(pe_working_set_t * data_set);
-
-extern xmlNode *do_calculations(pe_working_set_t * data_set, xmlNode * xml_input, crm_time_t * now);
-
 char *use_date = NULL;
 
 static void
-get_date(pe_working_set_t * data_set)
+get_date(pe_working_set_t *data_set, bool print_original)
 {
-    int value = 0;
     time_t original_date = 0;
 
-    crm_element_value_int(data_set->input, "execution-date", &value);
-    original_date = value;
+    crm_element_value_epoch(data_set->input, "execution-date", &original_date);
 
     if (use_date) {
         data_set->now = crm_time_new(use_date);
+        quiet_log(" + Setting effective cluster time: %s", use_date);
+        crm_time_log(LOG_NOTICE, "Pretending 'now' is", data_set->now,
+                     crm_time_log_date | crm_time_log_timeofday);
 
-    } else if(original_date) {
-        char *when = NULL;
+
+    } else if (original_date) {
 
         data_set->now = crm_time_new(NULL);
         crm_time_set_timet(data_set->now, &original_date);
 
-        when = crm_time_as_string(data_set->now, crm_time_log_date|crm_time_log_timeofday);
-        printf("Using the original execution date of: %s\n", when);
+        if (print_original) {
+            char *when = crm_time_as_string(data_set->now,
+                            crm_time_log_date|crm_time_log_timeofday);
 
-        free(when);
+            printf("Using the original execution date of: %s\n", when);
+            free(when);
+        }
     }
 }
 
@@ -86,7 +76,7 @@ print_cluster_status(pe_working_set_t * data_set, long options)
 {
     char *online_nodes = NULL;
     char *online_remote_nodes = NULL;
-    char *online_remote_containers = NULL;
+    char *online_guest_nodes = NULL;
     char *offline_nodes = NULL;
     char *offline_remote_nodes = NULL;
 
@@ -97,7 +87,7 @@ print_cluster_status(pe_working_set_t * data_set, long options)
         const char *node_mode = NULL;
         char *node_name = NULL;
 
-        if (is_container_remote_node(node)) {
+        if (pe__is_guest_node(node)) {
             node_name = crm_strdup_printf("%s:%s", node->details->uname, node->details->remote_rsc->container->id);
         } else {
             node_name = crm_strdup_printf("%s", node->details->uname);
@@ -135,9 +125,9 @@ print_cluster_status(pe_working_set_t * data_set, long options)
             }
 
         } else if (node->details->online) {
-            if (is_container_remote_node(node)) {
-                online_remote_containers = add_list_element(online_remote_containers, node_name);
-            } else if (is_baremetal_remote_node(node)) {
+            if (pe__is_guest_node(node)) {
+                online_guest_nodes = add_list_element(online_guest_nodes, node_name);
+            } else if (pe__is_remote_node(node)) {
                 online_remote_nodes = add_list_element(online_remote_nodes, node_name);
             } else {
                 online_nodes = add_list_element(online_nodes, node_name);
@@ -146,9 +136,9 @@ print_cluster_status(pe_working_set_t * data_set, long options)
             continue;
 
         } else {
-            if (is_baremetal_remote_node(node)) {
+            if (pe__is_remote_node(node)) {
                 offline_remote_nodes = add_list_element(offline_remote_nodes, node_name);
-            } else if (is_container_remote_node(node)) {
+            } else if (pe__is_guest_node(node)) {
                 /* ignore offline container nodes */
             } else {
                 offline_nodes = add_list_element(offline_nodes, node_name);
@@ -157,9 +147,9 @@ print_cluster_status(pe_working_set_t * data_set, long options)
             continue;
         }
 
-        if (is_container_remote_node(node)) {
-            printf("ContainerNode %s: %s\n", node_name, node_mode);
-        } else if (is_baremetal_remote_node(node)) {
+        if (pe__is_guest_node(node)) {
+            printf("GuestNode %s: %s\n", node_name, node_mode);
+        } else if (pe__is_remote_node(node)) {
             printf("RemoteNode %s: %s\n", node_name, node_mode);
         } else if (safe_str_eq(node->details->uname, node->details->id)) {
             printf("Node %s: %s\n", node_name, node_mode);
@@ -186,9 +176,9 @@ print_cluster_status(pe_working_set_t * data_set, long options)
         printf("RemoteOFFLINE: [%s ]\n", offline_remote_nodes);
         free(offline_remote_nodes);
     }
-    if (online_remote_containers) {
-        printf("Containers: [%s ]\n", online_remote_containers);
-        free(online_remote_containers);
+    if (online_guest_nodes) {
+        printf("GuestOnline: [%s ]\n", online_guest_nodes);
+        free(online_guest_nodes);
     }
 
     fprintf(stdout, "\n");
@@ -205,11 +195,12 @@ print_cluster_status(pe_working_set_t * data_set, long options)
 }
 
 static char *
-create_action_name(action_t * action)
+create_action_name(pe_action_t *action)
 {
     char *action_name = NULL;
-    const char *prefix = NULL;
+    const char *prefix = "";
     const char *action_host = NULL;
+    const char *clone_name = NULL;
     const char *task = action->task;
 
     if (action->node) {
@@ -220,15 +211,21 @@ create_action_name(action_t * action)
 
     if (safe_str_eq(action->task, RSC_CANCEL)) {
         prefix = "Cancel ";
-        task = "monitor";       /* TO-DO: Hack! */
+        task = action->cancel_task;
     }
 
     if (action->rsc && action->rsc->clone_name) {
-        char *key = NULL;
-        const char *name = action->rsc->clone_name;
-        const char *interval_s = g_hash_table_lookup(action->meta, XML_LRM_ATTR_INTERVAL);
+        clone_name = action->rsc->clone_name;
+    }
 
-        int interval = crm_parse_int(interval_s, "0");
+    if (clone_name) {
+        char *key = NULL;
+        const char *interval_ms_s = NULL;
+        guint interval_ms = 0;
+
+        interval_ms_s = g_hash_table_lookup(action->meta,
+                                            XML_LRM_ATTR_INTERVAL_MS);
+        interval_ms = crm_parse_ms(interval_ms_s);
 
         if (safe_str_eq(action->task, RSC_NOTIFY)
             || safe_str_eq(action->task, RSC_NOTIFIED)) {
@@ -237,35 +234,35 @@ create_action_name(action_t * action)
 
             CRM_ASSERT(n_type != NULL);
             CRM_ASSERT(n_task != NULL);
-            key = generate_notify_key(name, n_type, n_task);
+            key = generate_notify_key(clone_name, n_type, n_task);
 
         } else {
-            key = generate_op_key(name, task, interval);
+            key = generate_op_key(clone_name, task, interval_ms);
         }
 
         if (action_host) {
-            action_name = crm_strdup_printf("%s%s %s", prefix ? prefix : "", key, action_host);
+            action_name = crm_strdup_printf("%s%s %s", prefix, key, action_host);
         } else {
-            action_name = crm_strdup_printf("%s%s", prefix ? prefix : "", key);
+            action_name = crm_strdup_printf("%s%s", prefix, key);
         }
         free(key);
 
     } else if (safe_str_eq(action->task, CRM_OP_FENCE)) {
         const char *op = g_hash_table_lookup(action->meta, "stonith_action");
 
-        action_name = crm_strdup_printf("%s%s '%s' %s", prefix ? prefix : "", action->task, op, action_host);
+        action_name = crm_strdup_printf("%s%s '%s' %s", prefix, action->task, op, action_host);
 
     } else if (action->rsc && action_host) {
-        action_name = crm_strdup_printf("%s%s %s", prefix ? prefix : "", action->uuid, action_host);
+        action_name = crm_strdup_printf("%s%s %s", prefix, action->uuid, action_host);
 
     } else if (action_host) {
-        action_name = crm_strdup_printf("%s%s %s", prefix ? prefix : "", action->task, action_host);
+        action_name = crm_strdup_printf("%s%s %s", prefix, action->task, action_host);
 
     } else {
         action_name = crm_strdup_printf("%s", action->uuid);
     }
 
-    if(action_numbers) {
+    if (action_numbers) { // i.e. verbose
         char *with_id = crm_strdup_printf("%s (%d)", action_name, action->id);
 
         free(action_name);
@@ -307,13 +304,13 @@ create_dotfile(pe_working_set_t * data_set, const char *dot_file, gboolean all_a
             color = "red";
             font = "purple";
             if (all_actions == FALSE) {
-                goto dont_write;
+                goto do_not_write;
             }
 
         } else if (is_set(action->flags, pe_action_optional)) {
             color = "blue";
             if (all_actions == FALSE) {
-                goto dont_write;
+                goto do_not_write;
             }
 
         } else {
@@ -327,7 +324,7 @@ create_dotfile(pe_working_set_t * data_set, const char *dot_file, gboolean all_a
                 action_name, style, color, font);
         fprintf(dot_strm, "\"%s\" [ style=%s color=\"%s\" fontcolor=\"%s\"]\n",
                 action_name, style, color, font);
-  dont_write:
+  do_not_write:
         free(action_name);
     }
 
@@ -350,8 +347,6 @@ create_dotfile(pe_working_set_t * data_set, const char *dot_file, gboolean all_a
             } else if (is_set(action->flags, pe_action_pseudo)
                        && (before->type & pe_order_stonith_stop)) {
                 continue;
-            } else if (before->state == pe_link_dup) {
-                continue;
             } else if (before->type == pe_order_none) {
                 continue;
             } else if (is_set(before->action->flags, pe_action_dumped)
@@ -374,10 +369,8 @@ create_dotfile(pe_working_set_t * data_set, const char *dot_file, gboolean all_a
     }
 
     fprintf(dot_strm, "}\n");
-    if (dot_strm != NULL) {
-        fflush(dot_strm);
-        fclose(dot_strm);
-    }
+    fflush(dot_strm);
+    fclose(dot_strm);
 }
 
 static void
@@ -403,11 +396,11 @@ setup_input(const char *input, const char *output)
 
         if (rc != pcmk_ok) {
             fprintf(stderr, "Live CIB query failed: %s (%d)\n", pcmk_strerror(rc), rc);
-            crm_exit(rc);
+            crm_exit(crm_errno2exit(rc));
 
         } else if (cib_object == NULL) {
             fprintf(stderr, "Live CIB query failed: empty result\n");
-            crm_exit(ENOTCONN);
+            crm_exit(CRM_EX_NOINPUT);
         }
 
     } else if (safe_str_eq(input, "-")) {
@@ -423,12 +416,12 @@ setup_input(const char *input, const char *output)
 
     if (cli_config_update(&cib_object, NULL, FALSE) == FALSE) {
         free_xml(cib_object);
-        crm_exit(ENOKEY);
+        crm_exit(CRM_EX_CONFIG);
     }
 
     if (validate_xml(cib_object, NULL, FALSE) != TRUE) {
         free_xml(cib_object);
-        crm_exit(pcmk_err_schema_validation);
+        crm_exit(CRM_EX_CONFIG);
     }
 
     if (output == NULL) {
@@ -445,8 +438,9 @@ setup_input(const char *input, const char *output)
     cib_object = NULL;
 
     if (rc < 0) {
-        fprintf(stderr, "Could not create '%s': %s\n", output, strerror(errno));
-        crm_exit(rc);
+        fprintf(stderr, "Could not create '%s': %s\n",
+                output, pcmk_strerror(rc));
+        crm_exit(CRM_EX_CANTCREAT);
     }
     setenv("CIB_file", output, 1);
     free(local_output);
@@ -468,6 +462,7 @@ static struct crm_option long_options[] = {
     {"show-scores",   0, 0, 's', "Show allocation scores"},
     {"show-utilization",   0, 0, 'U', "Show utilization information"},
     {"profile",       1, 0, 'P', "Run all tests in the named directory to create profiling data"},
+    {"repeat",        1, 0, 'N', "With --profile, repeat each test N times and print timings"},
     {"pending",       0, 0, 'j', "\tDisplay pending state if 'record-pending' is enabled", pcmk_option_hidden},
 
     {"-spacer-",     0, 0, '-', "\nSynthetic Cluster Events:"},
@@ -475,14 +470,16 @@ static struct crm_option long_options[] = {
     {"node-down",    1, 0, 'd', "\tTake a node offline"},
     {"node-fail",    1, 0, 'f', "\tMark a node as failed"},
     {"op-inject",    1, 0, 'i', "\tGenerate a failure for the cluster to react to in the simulation"},
-    {"-spacer-",     0, 0, '-', "\t\tValue is of the form ${resource}_${task}_${interval}@${node}=${rc}."},
+    {"-spacer-",     0, 0, '-', "\t\tValue is of the form ${resource}_${task}_${interval_in_ms}@${node}=${rc}."},
     {"-spacer-",     0, 0, '-', "\t\tEg. memcached_monitor_20000@bart.example.com=7"},
-    {"-spacer-",     0, 0, '-', "\t\tFor more information on OCF return codes, refer to: http://www.clusterlabs.org/doc/en-US/Pacemaker/1.1/html/Pacemaker_Explained/s-ocf-return-codes.html"},
+    {"-spacer-",     0, 0, '-', "\t\tFor more information on OCF return codes, refer to: https://clusterlabs.org/pacemaker/doc/en-US/Pacemaker/2.0/html/Pacemaker_Administration/s-ocf-return-codes.html"},
     {"op-fail",      1, 0, 'F', "\tIf the specified task occurs during the simulation, have it fail with return code ${rc}"},
-    {"-spacer-",     0, 0, '-', "\t\tValue is of the form ${resource}_${task}_${interval}@${node}=${rc}."},
+    {"-spacer-",     0, 0, '-', "\t\tValue is of the form ${resource}_${task}_${interval_in_ms}@${node}=${rc}."},
     {"-spacer-",     0, 0, '-', "\t\tEg. memcached_stop_0@bart.example.com=1\n"},
     {"-spacer-",     0, 0, '-', "\t\tThe transition will normally stop at the failed action.  Save the result with --save-output and re-run with --xml-file"},
-    {"set-datetime", 1, 0, 't', "Set date/time"},
+    {   "set-datetime", required_argument, NULL, 't',
+        "Set date/time (ISO 8601 format, see https://en.wikipedia.org/wiki/ISO_8601)"
+    },
     {"quorum",       1, 0, 'q', "\tSpecify a value for quorum"},
     {"watchdog",     1, 0, 'w', "\tAssume a watchdog device is active"},
     {"ticket-grant",     1, 0, 'g', "Grant a ticket"},
@@ -499,7 +496,7 @@ static struct crm_option long_options[] = {
     {"all-actions",  0, 0, 'a', "\tDisplay all possible actions in the DOT graph - even ones not part of the transition"},
 
     {"-spacer-",    0, 0, '-', "\nData Source:"},
-    {"live-check",  0, 0, 'L', "\tConnect to the CIB and use the current contents as input"},
+    {"live-check",  0, 0, 'L', "\tConnect to the CIB mamager and use the current CIB contents as input"},
     {"xml-file",    1, 0, 'x', "\tRetrieve XML from the named file"},
     {"xml-pipe",    0, 0, 'p', "\tRetrieve XML from stdin"},
 
@@ -514,17 +511,22 @@ static struct crm_option long_options[] = {
 /* *INDENT-ON* */
 
 static void
-profile_one(const char *xml_file)
+profile_one(const char *xml_file, long long repeat, pe_working_set_t *data_set)
 {
     xmlNode *cib_object = NULL;
-    pe_working_set_t data_set;
+    clock_t start = 0;
+
+    printf("* Testing %s ...", xml_file);
+    fflush(stdout);
 
-    printf("* Testing %s\n", xml_file);
     cib_object = filename2xml(xml_file);
+    start = clock();
+
     if (get_object_root(XML_CIB_TAG_STATUS, cib_object) == NULL) {
         create_xml_node(cib_object, XML_CIB_TAG_STATUS);
     }
 
+
     if (cli_config_update(&cib_object, NULL, FALSE) == FALSE) {
         free_xml(cib_object);
         return;
@@ -535,25 +537,26 @@ profile_one(const char *xml_file)
         return;
     }
 
-    set_working_set_defaults(&data_set);
+    for (int i = 0; i < repeat; ++i) {
+        xmlNode *input = (repeat == 1)? cib_object : copy_xml(cib_object);
 
-    data_set.input = cib_object;
-    get_date(&data_set);
-    do_calculations(&data_set, cib_object, NULL);
-
-    cleanup_alloc_calculations(&data_set);
+        data_set->input = input;
+        get_date(data_set, false);
+        pcmk__schedule_actions(data_set, input, NULL);
+        pe_reset_working_set(data_set);
+    }
+    printf(" %.2f secs\n", (clock() - start) / (float) CLOCKS_PER_SEC);
 }
 
 #ifndef FILENAME_MAX
 #  define FILENAME_MAX 512
 #endif
 
-static int
-profile_all(const char *dir)
+static void
+profile_all(const char *dir, long long repeat, pe_working_set_t *data_set)
 {
     struct dirent **namelist;
 
-    int lpc = 0;
     int file_num = scandir(dir, &namelist, 0, alphasort);
 
     if (file_num > 0) {
@@ -569,44 +572,20 @@ profile_all(const char *dir)
                 free(namelist[file_num]);
                 continue;
             }
-
-            lpc++;
             snprintf(buffer, sizeof(buffer), "%s/%s", dir, namelist[file_num]->d_name);
             if (stat(buffer, &prop) == 0 && S_ISREG(prop.st_mode)) {
-                profile_one(buffer);
+                profile_one(buffer, repeat, data_set);
             }
             free(namelist[file_num]);
         }
         free(namelist);
     }
-
-    return lpc;
-}
-
-static int
-count_resources(pe_working_set_t * data_set, resource_t * rsc)
-{
-    int count = 0;
-    GListPtr gIter = NULL;
-
-    if (rsc == NULL) {
-        gIter = data_set->resources;
-    } else if (rsc->children) {
-        gIter = rsc->children;
-    } else {
-        return is_not_set(rsc->flags, pe_rsc_orphan);
-    }
-
-    for (; gIter != NULL; gIter = gIter->next) {
-        count += count_resources(data_set, gIter->data);
-    }
-    return count;
 }
 
 int
 main(int argc, char **argv)
 {
-    int rc = 0;
+    int rc = pcmk_ok;
     guint modified = 0;
 
     gboolean store = FALSE;
@@ -615,7 +594,7 @@ main(int argc, char **argv)
     gboolean all_actions = FALSE;
     gboolean have_stdout = FALSE;
 
-    pe_working_set_t data_set;
+    pe_working_set_t *data_set = NULL;
 
     const char *xml_file = "-";
     const char *quorum = NULL;
@@ -625,10 +604,12 @@ main(int argc, char **argv)
     const char *graph_file = NULL;
     const char *input_file = NULL;
     const char *output_file = NULL;
+    const char *repeat_s = NULL;
 
     int flag = 0;
     int index = 0;
     int argerr = 0;
+    long long repeat = 1;
 
     GListPtr node_up = NULL;
     GListPtr node_down = NULL;
@@ -646,7 +627,7 @@ main(int argc, char **argv)
                     long_options, "Tool for simulating the cluster's response to events");
 
     if (argc < 2) {
-        crm_help('?', EX_USAGE);
+        crm_help('?', CRM_EX_USAGE);
     }
 
     while (1) {
@@ -668,7 +649,7 @@ main(int argc, char **argv)
                 break;
             case '?':
             case '$':
-                crm_help(flag, EX_OK);
+                crm_help(flag, CRM_EX_OK);
                 break;
             case 'p':
                 xml_file = "-";
@@ -774,6 +755,9 @@ main(int argc, char **argv)
             case 'P':
                 test_dir = optarg;
                 break;
+            case 'N':
+                repeat_s = optarg;
+                break;
             default:
                 ++argerr;
                 break;
@@ -785,84 +769,104 @@ main(int argc, char **argv)
     }
 
     if (argerr) {
-        crm_help('?', EX_USAGE);
+        crm_help('?', CRM_EX_USAGE);
+    }
+
+    data_set = pe_new_working_set();
+    if (data_set == NULL) {
+        crm_perror(LOG_ERR, "Could not allocate working set");
+        rc = -ENOMEM;
+        goto done;
     }
 
     if (test_dir != NULL) {
-        return profile_all(test_dir);
+        if (repeat_s != NULL) {
+            repeat = crm_int_helper(repeat_s, NULL);
+            if (errno || (repeat < 1)) {
+                fprintf(stderr, "--repeat must be positive integer, not '%s' -- using 1",
+                        repeat_s);
+                repeat = 1;
+            }
+        }
+        profile_all(test_dir, repeat, data_set);
+        return CRM_EX_OK;
     }
 
     setup_input(xml_file, store ? xml_file : output_file);
 
     global_cib = cib_new();
-    global_cib->cmds->signon(global_cib, crm_system_name, cib_command);
-
-    set_working_set_defaults(&data_set);
-
-    if (data_set.now != NULL) {
-        quiet_log(" + Setting effective cluster time: %s", use_date);
-        crm_time_log(LOG_WARNING, "Set fake 'now' to", data_set.now,
-                     crm_time_log_date | crm_time_log_timeofday);
+    rc = global_cib->cmds->signon(global_cib, crm_system_name, cib_command);
+    if (rc != pcmk_ok) {
+        fprintf(stderr, "Could not connect to the CIB manager: %s\n",
+                pcmk_strerror(rc));
+        goto done;
     }
 
     rc = global_cib->cmds->query(global_cib, NULL, &input, cib_sync_call | cib_scope_local);
-    CRM_ASSERT(rc == pcmk_ok);
-
-    data_set.input = input;
-    get_date(&data_set);
-    if(xml_file) {
-        set_bit(data_set.flags, pe_flag_sanitized);
+    if (rc != pcmk_ok) {
+        fprintf(stderr, "Could not get local CIB: %s\n", pcmk_strerror(rc));
+        goto done;
     }
-    cluster_status(&data_set);
+
+    data_set->input = input;
+    get_date(data_set, true);
+    if(xml_file) {
+        set_bit(data_set->flags, pe_flag_sanitized);
+    }
+    set_bit(data_set->flags, pe_flag_stdout);
+    cluster_status(data_set);
 
     if (quiet == FALSE) {
         int options = print_pending ? pe_print_pending : 0;
 
-        if(is_set(data_set.flags, pe_flag_maintenance_mode)) {
+        if (is_set(data_set->flags, pe_flag_maintenance_mode)) {
             quiet_log("\n              *** Resource management is DISABLED ***");
             quiet_log("\n  The cluster will not attempt to start, stop or recover services");
             quiet_log("\n");
         }
 
-        if(data_set.disabled_resources || data_set.blocked_resources) {
-            quiet_log("%d of %d resources DISABLED and %d BLOCKED from being started due to failures\n",
-                      data_set.disabled_resources, count_resources(&data_set, NULL), data_set.blocked_resources);
+        if (data_set->disabled_resources || data_set->blocked_resources) {
+            quiet_log("%d of %d resource instances DISABLED and %d BLOCKED "
+                      "from further action due to failure\n",
+                      data_set->disabled_resources, data_set->ninstances,
+                      data_set->blocked_resources);
         }
 
         quiet_log("\nCurrent cluster status:\n");
-        print_cluster_status(&data_set, options);
+        print_cluster_status(data_set, options);
     }
 
     if (modified) {
         quiet_log("Performing requested modifications\n");
-        modify_configuration(&data_set, global_cib, quorum, watchdog, node_up, node_down, node_fail, op_inject,
+        modify_configuration(data_set, global_cib, quorum, watchdog, node_up, node_down, node_fail, op_inject,
                              ticket_grant, ticket_revoke, ticket_standby, ticket_activate);
 
         rc = global_cib->cmds->query(global_cib, NULL, &input, cib_sync_call);
         if (rc != pcmk_ok) {
-            fprintf(stderr, "Could not connect to the CIB for input: %s\n", pcmk_strerror(rc));
+            fprintf(stderr, "Could not get modified CIB: %s\n", pcmk_strerror(rc));
             goto done;
         }
 
-        cleanup_calculations(&data_set);
-        data_set.input = input;
-        get_date(&data_set);
+        cleanup_calculations(data_set);
+        data_set->input = input;
+        get_date(data_set, true);
 
         if(xml_file) {
-            set_bit(data_set.flags, pe_flag_sanitized);
+            set_bit(data_set->flags, pe_flag_sanitized);
         }
-        cluster_status(&data_set);
+        set_bit(data_set->flags, pe_flag_stdout);
+        cluster_status(data_set);
     }
 
     if (input_file != NULL) {
         rc = write_xml_file(input, input_file, FALSE);
         if (rc < 0) {
-            fprintf(stderr, "Could not create '%s': %s\n", input_file, strerror(errno));
+            fprintf(stderr, "Could not create '%s': %s\n",
+                    input_file, pcmk_strerror(rc));
             goto done;
         }
     }
 
-    rc = 0;
     if (process || simulate) {
         crm_time_t *local_date = NULL;
 
@@ -874,15 +878,15 @@ main(int argc, char **argv)
             printf("Utilization information:\n");
         }
 
-        do_calculations(&data_set, input, local_date);
+        pcmk__schedule_actions(data_set, input, local_date);
         input = NULL;           /* Don't try and free it twice */
 
         if (graph_file != NULL) {
-            write_xml_file(data_set.graph, graph_file, FALSE);
+            write_xml_file(data_set->graph, graph_file, FALSE);
         }
 
         if (dot_file != NULL) {
-            create_dotfile(&data_set, dot_file, all_actions);
+            create_dotfile(data_set, dot_file, all_actions);
         }
 
         if (quiet == FALSE) {
@@ -892,29 +896,33 @@ main(int argc, char **argv)
                       || modified ? "\n" : "");
             fflush(stdout);
 
-            LogNodeActions(&data_set, TRUE);
-            for (gIter = data_set.resources; gIter != NULL; gIter = gIter->next) {
+            LogNodeActions(data_set, TRUE);
+            for (gIter = data_set->resources; gIter != NULL; gIter = gIter->next) {
                 resource_t *rsc = (resource_t *) gIter->data;
 
-                LogActions(rsc, &data_set, TRUE);
+                LogActions(rsc, data_set, TRUE);
             }
         }
     }
 
+    rc = pcmk_ok;
+
     if (simulate) {
-        rc = run_simulation(&data_set, global_cib, op_fail, quiet);
+        if (run_simulation(data_set, global_cib, op_fail, quiet) != pcmk_ok) {
+            rc = pcmk_err_generic;
+        }
         if(quiet == FALSE) {
-            get_date(&data_set);
+            get_date(data_set, true);
 
             quiet_log("\nRevised cluster status:\n");
-            cluster_status(&data_set);
-            print_cluster_status(&data_set, 0);
+            set_bit(data_set->flags, pe_flag_stdout);
+            cluster_status(data_set);
+            print_cluster_status(data_set, 0);
         }
     }
 
   done:
-    cleanup_alloc_calculations(&data_set);
-
+    pe_free_working_set(data_set);
     global_cib->cmds->signoff(global_cib);
     cib_delete(global_cib);
     free(use_date);
@@ -924,5 +932,5 @@ main(int argc, char **argv)
         unlink(temp_shadow);
         free(temp_shadow);
     }
-    return crm_exit(rc);
+    crm_exit(crm_errno2exit(rc));
 }
