@@ -1,5 +1,7 @@
 /*
- * Copyright 2012-2018 Gao,Yan <ygao@suse.com>
+ * Copyright 2012-2019 the Pacemaker project contributors
+ *
+ * The version control history for this file may have further details.
  *
  * This source code is licensed under the GNU General Public License version 2
  * or later (GPLv2+) WITHOUT ANY WARRANTY.
@@ -28,7 +30,7 @@
 #include <crm/pengine/rules.h>
 #include <crm/pengine/status.h>
 
-#include <pacemaker-schedulerd.h>
+#include <pacemaker-internal.h>
 
 gboolean do_force = FALSE;
 gboolean BE_QUIET = FALSE;
@@ -43,11 +45,6 @@ char ticket_cmd = 'S';
 char *xml_file = NULL;
 int cib_options = cib_sync_call;
 
-#define CMD_ERR(fmt, args...) do {          \
-        crm_warn(fmt, ##args);              \
-        fprintf(stderr, fmt "\n", ##args);  \
-    } while(0)
-
 static ticket_t *
 find_ticket(const char *ticket_id, pe_working_set_t * data_set)
 {
@@ -420,7 +417,7 @@ modify_ticket_state(const char * ticket_id, GListPtr attr_delete, GHashTable * a
             && (ticket == NULL || ticket->granted == FALSE)
             && crm_is_true(value)) {
 
-            char *now = crm_itoa(time(NULL));
+            char *now = crm_ttoa(time(NULL));
 
             crm_xml_add(ticket_state_xml, "last-granted", now);
             free(now);
@@ -493,7 +490,7 @@ static struct crm_option long_options[] = {
     
     {"-spacer-",   1, 0, '-', "\nAdvanced Commands:"},
     {"get-attr",   1, 0, 'G', "\tDisplay the named attribute for a ticket"},
-    {"set-attr",   1, 0, 'S', "\tSet the named attribtue for a ticket"},
+    {"set-attr",   1, 0, 'S', "\tSet the named attribute for a ticket"},
     {"delete-attr",1, 0, 'D', "\tDelete the named attribute for a ticket"},
     {"cleanup",    0, 0, 'C', "\t\tDelete all state of a ticket at this cluster site"},
     
@@ -685,6 +682,7 @@ main(int argc, char **argv)
         exit_code = CRM_EX_OSERR;
         goto bail;
     }
+    set_bit(data_set->flags, pe_flag_no_counts);
 
     cib_conn = cib_new();
     if (cib_conn == NULL) {
@@ -903,5 +901,5 @@ main(int argc, char **argv)
         CMD_ERR("Use --force to ignore quorum");
     }
 
-    return crm_exit(exit_code);
+    crm_exit(exit_code);
 }
