@@ -9,7 +9,10 @@
  * Author: Intel Corporation
  *         Jeff Zheng <Jeff.Zheng@Intel.com>
  *
- * Copyright 2009-2018 International Business Machines, IBM
+ * Original copyright 2009 International Business Machines, IBM
+ * Later changes copyright 2009-2019 the Pacemaker project contributors
+ *
+ * The version control history for this file may have further details.
  *
  * This source code is licensed under the GNU Lesser General Public License
  * version 2.1 or later (LGPLv2.1+) WITHOUT ANY WARRANTY.
@@ -155,7 +158,6 @@ getStringExecOutput(const char *const args[])
         if (sizeRead > 0) {
 
             char *end = serialNumber + strlen(serialNumber) - 1;
-            char *retSerialNumber = NULL;
 
             while (end > serialNumber
                    && (*end == '\n' || *end == '\r' || *end == '\t' || *end == ' ')
@@ -163,17 +165,7 @@ getStringExecOutput(const char *const args[])
                 *end = '\0';
                 end--;
             }
-
-            retSerialNumber = malloc(strlen(serialNumber) + 1);
-
-            if (retSerialNumber) {
-
-                strcpy(retSerialNumber, serialNumber);
-
-            }
-
-            return retSerialNumber;
-
+            return strdup(serialNumber);
         }
 
         return NULL;
@@ -507,7 +499,7 @@ entity_change(enum ipmi_update_e op, ipmi_domain_t * domain, ipmi_entity_t * ent
         rv = ipmi_entity_add_sensor_update_handler(entity, sensor_change, entity);
         if (rv) {
             crm_err("ipmi_entity_set_sensor_update_handler: 0x%x", rv);
-            crm_exit(pcmk_err_generic);
+            crm_exit(CRM_EX_ERROR);
         }
     }
 }
@@ -544,7 +536,7 @@ main(int argc, char *argv[])
     os_hnd = ipmi_posix_setup_os_handler();
     if (!os_hnd) {
         crm_err("ipmi_smi_setup_con: Unable to allocate os handler");
-        crm_exit(pcmk_err_generic);
+        crm_exit(CRM_EX_ERROR);
     }
 
     /* Initialize the OpenIPMI library. */
@@ -555,9 +547,9 @@ main(int argc, char *argv[])
         for (char **arg = &argv[1]; *arg != NULL; ++arg) {
             if (!strcmp(*arg, "--help") || !strcmp(*arg, "-?")) {
                 usage(argv[0]);
-                return 0;
+                return CRM_EX_OK;
             } else if (!strcmp(*arg, "--version") || !strcmp(*arg, "-$")) {
-                crm_help('$', 0);
+                crm_help('$', CRM_EX_OK);
             }
         }
     }
@@ -567,7 +559,7 @@ main(int argc, char *argv[])
     if (rv) {
         crm_err("Error parsing command arguments, argument %d: %s", curr_arg, strerror(rv));
         usage(argv[0]);
-        crm_exit(pcmk_err_generic);
+        crm_exit(CRM_EX_USAGE);
     }
 #endif
 
@@ -581,7 +573,7 @@ main(int argc, char *argv[])
     if (rv) {
         crm_err("ipmi_ip_setup_con: %s", strerror(rv));
         crm_err("Error: Is IPMI configured correctly?");
-        crm_exit(pcmk_err_generic);
+        crm_exit(CRM_EX_ERROR);
     }
 #else
     /* If all you need is an SMI connection, this is all the code you
@@ -596,14 +588,14 @@ main(int argc, char *argv[])
     if (rv) {
         crm_err("ipmi_smi_setup_con: %s", strerror(rv));
         crm_err("Error: Is IPMI configured correctly?");
-        crm_exit(pcmk_err_generic);
+        crm_exit(CRM_EX_ERROR);
     }
 #endif
 
     rv = ipmi_open_domain("", &con, 1, setup_done, NULL, NULL, NULL, NULL, 0, NULL);
     if (rv) {
         crm_err("ipmi_init_domain: %s", strerror(rv));
-        crm_exit(pcmk_err_generic);
+        crm_exit(CRM_EX_ERROR);
     }
 
     /* This is the main loop of the event-driven program. 
@@ -613,4 +605,5 @@ main(int argc, char *argv[])
 
     /* Technically, we can't get here, but this is an example. */
     os_hnd->free_os_handler(os_hnd);
+    return CRM_EX_OK;
 }
